@@ -1,9 +1,9 @@
 <?php
 /**
- * $Horde: horde/index.php,v 2.105.4.1 2005/01/03 12:25:28 jan Exp $
+ * $Horde: horde/index.php,v 2.105.4.8 2006/07/29 16:49:19 jan Exp $
  *
- * Copyright 1999-2005 Charles J. Hagenbuch <chuck@horde.org>
- * Copyright 1999-2005 Jon Parise <jon@horde.org>
+ * Copyright 1999-2006 Charles J. Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2006 Jon Parise <jon@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL).  If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -24,7 +24,6 @@ if (!$horde_configured) {
               'mime_drivers.php' => 'This file controls the global set of MIME drivers for the Horde framework, allowing applications to make use of programs such as enscript or mswordview to render content into HTML for viewing in a browser.',
               'nls.php' => 'This file provides localisation support for the Horde framework.',
               'registry.php' => 'The registry is how Horde applications find out how to talk to each other. You should list any installed Horde applications that you have here.'));
-    exit;
 }
 
 require_once HORDE_BASE . '/lib/base.php';
@@ -35,8 +34,44 @@ if ($browser->isMobile()) {
 }
 
 $main_page = Util::getFormData('url');
+
+// Break up the requested URL in $main_page and run some sanity checks
+// on it to prevent phishing and XSS attacks. If any of the checks
+// fail, $main_page will be set to null.
+if (!empty($main_page)) {
+    // Mute errors in case of unparseable URLs
+    $req = @parse_url($main_page);
+
+    // We assume that any valid redirect URL will be in the same
+    // cookie domain. This helps prevent rogue off-site Horde installs
+    // from mimicking the real server.
+    if (isset($req['host'])) {
+        $qcookiedom = preg_quote($conf['cookie']['domain']);
+        if (!preg_match('/' . $qcookiedom . '$/', $req['host'])) {
+            $main_page = null;
+        }
+    }
+
+    // Protocol whitelist: If the URL is fully qualified, make sure it
+    // is either http or https.
+    $allowed_protocols = array('http', 'https');
+    if (empty($req['scheme']) ||
+        !in_array($req['scheme'], $allowed_protocols)) {
+        $main_page = null;
+    }
+}
+
 if (!$main_page) {
-    $main_page = Horde::initialPage();
+    $initial_app = $prefs->getValue('initial_application');
+    if (!empty($initial_app) && $registry->hasPermission($initial_app)) {
+        $main_page = Horde::url($registry->getInitialPage($initial_app), true);
+    } elseif (isset($registry->applications['horde']['initial_page'])) {
+        $main_page = Horde::applicationUrl($registry->applications['horde']['initial_page'], true);
+    } elseif (Auth::getAuth()) {
+        $main_page = Horde::applicationUrl('services/portal/', true);
+    } else {
+        $main_page = Horde::applicationUrl('login.php', true);
+    }
 }
 
 if (!Util::getFormData('frameset_loaded') &&
@@ -47,7 +82,6 @@ if (!Util::getFormData('frameset_loaded') &&
     } else {
         $scrollbar = 'auto';
     }
-    $main_page = Util::addParameter($main_page, 'frameset_loaded', 1);
     require HORDE_TEMPLATES . '/index/frames_index.inc';
 } else {
     header('Location: ' . $main_page);
