@@ -6,61 +6,58 @@
  * This class requires the 'radius' PECL extension.
  * RADIUS PECL extension: http://pecl.php.net/package/radius
  *
- * On *nix-y machines, this extension can be installed as follows:
- *   "pear install radius"
+ * On *nix-y machines, this extension can be installed as follows:<pre>
+ *   "pear install radius"</pre>
  *
- * Then, edit your php.ini file and make sure the following line is present:
+ * Then, edit your php.ini file and make sure the following line is present:<pre>
  *   For Windows machines:  extension=php_radius.dll
- *   For all others:        extension=radius.so
+ *   For all others:        extension=radius.so</pre>
  *
- * Required parameters:
- * ====================
- *   'host'    --  The RADIUS host to use (IP address or fully qualified
- *                 hostname).
- *   'method'  --  The RADIUS method to use for validating the request.
- *                 Either: 'PAP', 'CHAP_MD5', 'MSCHAPv1', or 'MSCHAPv2'.
- *                 ** CURRENTLY, only 'PAP' is supported. **
- *   'secret'  --  The RADIUS shared secret string for the host.
- *                 The RADIUS protocol ignores all but the leading 128 bytes
- *                 of the shared secret.
+ * Required parameters:<pre>
+ *   'host'    The RADIUS host to use (IP address or fully qualified
+ *             hostname).
+ *   'method'  The RADIUS method to use for validating the request.
+ *             Either: 'PAP', 'CHAP_MD5', 'MSCHAPv1', or 'MSCHAPv2'.
+ *             ** CURRENTLY, only 'PAP' is supported. **
+ *   'secret'  The RADIUS shared secret string for the host. The
+ *             RADIUS protocol ignores all but the leading 128 bytes
+ *             of the shared secret.</pre>
  *
- * Optional parameters:
- * ====================
- *   'nas'      --  The RADIUS NAS identifier to use.
- *                  DEFAULT: The value of $_SERVER['HTTP_HOST'] or, if not
- *                           defined, then 'localhost'.
- *   'port'     --  The port to use on the RADIUS server.
- *                  DEFAULT: Whatever the local system identifies as the
- *                           'radius' UDP port
- *   'retries'  --  The maximum number of repeated requests to make before
- *                  giving up.
- *                  DEFAULT: 3
- *   'suffix'   --  The domain name to add to unqualified user names.
- *                  DEFAULT: NONE
- *   'timeout'  --  The timeout for receiving replies from the server (in
- *                  seconds).
- *                  DEFAULT: 3 seconds
+ * Optional parameters:<pre>
+ *   'nas'      The RADIUS NAS identifier to use.
+ *              DEFAULT: The value of $_SERVER['HTTP_HOST'] or, if not
+ *                       defined, then 'localhost'.
+ *   'port'     The port to use on the RADIUS server.
+ *              DEFAULT: Whatever the local system identifies as the
+ *                       'radius' UDP port
+ *   'retries'  The maximum number of repeated requests to make before
+ *              giving up.
+ *              DEFAULT: 3
+ *   'suffix'   The domain name to add to unqualified user names.
+ *              DEFAULT: NONE
+ *   'timeout'  The timeout for receiving replies from the server (in
+ *              seconds).
+ *              DEFAULT: 3 seconds</pre>
  *
  *
- * $Horde: framework/Auth/Auth/radius.php,v 1.24.8.1 2005/01/03 12:18:53 jan Exp $
+ * $Horde: framework/Auth/Auth/radius.php,v 1.24.8.8 2006/01/01 21:28:07 jan Exp $
  *
- * Copyright 2002-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2002-2006 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @version $Revision: 1.24.8.1 $
  * @since   Horde 3.0
  * @package Horde_Auth
  */
 class Auth_radius extends Auth {
 
-    /** 
+    /**
      * An array of capabilities, so that the driver can report which
      * operations it supports and which it doesn't.
      *
-     * @var array $capabilities 
+     * @var array
      */
     var $capabilities = array('add'           => false,
                               'update'        => false,
@@ -72,14 +69,12 @@ class Auth_radius extends Auth {
     /**
      * Constructs a new RADIUS authentication object.
      *
-     * @access public
-     *
-     * @param array optional $params  A hash containing connection parameters.
+     * @param array $params  A hash containing connection parameters.
      */
     function Auth_radius($params = array())
     {
         if (!Util::extensionExists('radius')) {
-            Horde::fatal(PEAR::raiseError('Auth_radius requires the radius PECL extension to be loaded.'), __FILE__, __LINE__);
+            Horde::fatal('Auth_radius requires the radius PECL extension to be loaded.', __FILE__, __LINE__);
         }
 
         $this->_setParams($params);
@@ -96,17 +91,17 @@ class Auth_radius extends Auth {
     {
         /* A RADIUS host is required. */
         if (empty($params['host'])) {
-            Horde::fatal(PEAR::raiseError('Auth_radius requires a RADIUS host to connect to.'), __FILE__, __LINE__);
+            Horde::fatal('Auth_radius requires a RADIUS host to connect to.', __FILE__, __LINE__);
         }
 
         /* A RADIUS secret string is required. */
         if (empty($params['secret'])) {
-            Horde::fatal(PEAR::raiseError('Auth_radius requires a RADIUS secret string.'), __FILE__, __LINE__);
+            Horde::fatal('Auth_radius requires a RADIUS secret string.', __FILE__, __LINE__);
         }
 
         /* A RADIUS authentication method is required. */
         if (empty($params['method'])) {
-            Horde::fatal(PEAR::raiseError('Auth_radius requires a RADIUS authentication method.'), __FILE__, __LINE__);
+            Horde::fatal('Auth_radius requires a RADIUS authentication method.', __FILE__, __LINE__);
         }
 
         /* RADIUS NAS Identifier. */
@@ -154,7 +149,7 @@ class Auth_radius extends Auth {
     {
         /* Password is required. */
         if (!isset($credentials['password'])) {
-            Horde::fatal(PEAR::raiseError(_("Password required for RADIUS authentication.")), __FILE__, __LINE__);
+            Horde::fatal(_("Password required for RADIUS authentication."), __FILE__, __LINE__);
         }
 
         $res = radius_auth_open();
