@@ -1,23 +1,26 @@
 <?php
 /**
- * The Cipher_cast128:: class implements the Cipher interface enryption data
- * using the CAST128 (aka Cast5) algorithm as define in RFC2144.
+ * The Cipher_cast128:: class implements the Cipher interface encryption data
+ * using the CAST128 (aka Cast5) algorithm as defined in RFC2144.
  *
- * $Horde: framework/Cipher/Cipher/cast128.php,v 1.7 2004/01/01 15:14:10 jan Exp $
+ * $Horde: framework/Cipher/Cipher/cast128.php,v 1.7.12.8 2006/01/01 21:28:10 jan Exp $
  *
- * Copyright 2002-2004 Mike Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2002-2006 Mike Cochrane <mike@graftonhall.co.nz>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Mike Cochrane <mike@graftonhall.co.nz>
- * @version $Revision: 1.7 $
  * @since   Horde 2.2
  * @package Horde_Cipher
  */
 class Horde_Cipher_cast128 extends Horde_Cipher {
 
-    /* S boxes */
+    /**
+     * S box (s1)
+     *
+     * @var array
+     */
     var $s1 = array(
         0x30fb40d4, 0x9fa0ff0b, 0x6beccd2f, 0x3f258c7a, 0x1e213f2f, 0x9c004dd3, 0x6003e540, 0xcf9fc949,
         0xbfd4af27, 0x88bbbdb5, 0xe2034090, 0x98d09675, 0x6e63a0e0, 0x15c361d2, 0xc2e7661d, 0x22d4ff8e,
@@ -52,6 +55,11 @@ class Horde_Cipher_cast128 extends Horde_Cipher {
         0xbd91e046, 0x9a56456e, 0xdc39200c, 0x20c8c571, 0x962bda1c, 0xe1e696ff, 0xb141ab08, 0x7cca89b9,
         0x1a69e783, 0x02cc4843, 0xa2f7c579, 0x429ef47d, 0x427b169c, 0x5ac9f049, 0xdd8f0f00, 0x5c8165bf);
 
+    /**
+     * S box (s2)
+     *
+     * @var array
+     */
     var $s2 = array(
         0x1f201094, 0xef0ba75b, 0x69e3cf7e, 0x393f4380, 0xfe61cf7a, 0xeec5207a, 0x55889c94, 0x72fc0651,
         0xada7ef79, 0x4e1d7235, 0xd55a63ce, 0xde0436ba, 0x99c430ef, 0x5f0c0794, 0x18dcdb7d, 0xa1d6eff3,
@@ -86,6 +94,11 @@ class Horde_Cipher_cast128 extends Horde_Cipher {
         0x8f5ea2b3, 0xfc184642, 0x0a036b7a, 0x4fb089bd, 0x649da589, 0xa345415e, 0x5c038323, 0x3e5d3bb9,
         0x43d79572, 0x7e6dd07c, 0x06dfdf1e, 0x6c6cc4ef, 0x7160a539, 0x73bfbe70, 0x83877605, 0x4523ecf1);
 
+    /**
+     * S box (s3)
+     *
+     * @var array
+     */
     var $s3 = array(
         0x8defc240, 0x25fa5d9f, 0xeb903dbf, 0xe810c907, 0x47607fff, 0x369fe44b, 0x8c1fc644, 0xaececa90,
         0xbeb1f9bf, 0xeefbcaea, 0xe8cf1950, 0x51df07ae, 0x920e8806, 0xf0ad0548, 0xe13c8d83, 0x927010d5,
@@ -120,6 +133,11 @@ class Horde_Cipher_cast128 extends Horde_Cipher {
         0x282f9350, 0x8334b362, 0xd91d1120, 0x2b6d8da0, 0x642b1e31, 0x9c305a00, 0x52bce688, 0x1b03588a,
         0xf7baefd5, 0x4142ed9c, 0xa4315c11, 0x83323ec5, 0xdfef4636, 0xa133c501, 0xe9d3531c, 0xee353783);
 
+    /**
+     * S box (s4)
+     *
+     * @var array
+     */
     var $s4 = array(
         0x9db30420, 0x1fb6e9de, 0xa7be7bef, 0xd273a298, 0x4a4f7bdb, 0x64ad8c57, 0x85510443, 0xfa020ed1,
         0x7e287aff, 0xe60fb663, 0x095f35a1, 0x79ebf120, 0xfd059d43, 0x6497b7b1, 0xf3641f63, 0x241e4adf,
@@ -154,6 +172,11 @@ class Horde_Cipher_cast128 extends Horde_Cipher {
         0x8644213e, 0xb7dc59d0, 0x7965291f, 0xccd6fd43, 0x41823979, 0x932bcdf6, 0xb657c34d, 0x4edfd282,
         0x7ae5290c, 0x3cb9536b, 0x851e20fe, 0x9833557e, 0x13ecf0b0, 0xd3ffb372, 0x3f85c5c1, 0x0aef7ed2);
 
+    /**
+     * S box (s5)
+     *
+     * @var array
+     */
     var $s5 = array(
         0x7ec90c04, 0x2c6e74b9, 0x9b0e66df, 0xa6337911, 0xb86a7fff, 0x1dd358f5, 0x44dd9d44, 0x1731167f,
         0x08fbf1fa, 0xe7f511cc, 0xd2051b00, 0x735aba00, 0x2ab722d8, 0x386381cb, 0xacf6243a, 0x69befd7a,
@@ -188,6 +211,11 @@ class Horde_Cipher_cast128 extends Horde_Cipher {
         0xd6cd2595, 0x68ff1ebf, 0x7555442c, 0xf19f06be, 0xf9e0659a, 0xeeb9491d, 0x34010718, 0xbb30cab8,
         0xe822fe15, 0x88570983, 0x750e6249, 0xda627e55, 0x5e76ffa8, 0xb1534546, 0x6d47de08, 0xefe9e7d4);
 
+    /**
+     * S box (s6)
+     *
+     * @var array
+     */
     var $s6 = array(
         0xf6fa8f9d, 0x2cac6ce1, 0x4ca34867, 0xe2337f7c, 0x95db08e7, 0x016843b4, 0xeced5cbc, 0x325553ac,
         0xbf9f0960, 0xdfa1e2ed, 0x83f0579d, 0x63ed86b9, 0x1ab6a6b8, 0xde5ebe39, 0xf38ff732, 0x8989b138,
@@ -222,6 +250,11 @@ class Horde_Cipher_cast128 extends Horde_Cipher {
         0x3b4cbf9f, 0x4a5de3ab, 0xe6051d35, 0xa0e1d855, 0xd36b4cf1, 0xf544edeb, 0xb0e93524, 0xbebb8fbd,
         0xa2d762cf, 0x49c92f54, 0x38b5f331, 0x7128a454, 0x48392905, 0xa65b1db8, 0x851c97bd, 0xd675cf2f);
 
+    /**
+     * S box (s7)
+     *
+     * @var array
+     */
     var $s7 = array(
         0x85e04019, 0x332bf567, 0x662dbfff, 0xcfc65693, 0x2a8d7f6f, 0xab9bc912, 0xde6008a1, 0x2028da1f,
         0x0227bce7, 0x4d642916, 0x18fac300, 0x50f18b82, 0x2cb2cb11, 0xb232e75c, 0x4b3695f2, 0xb28707de,
@@ -256,6 +289,11 @@ class Horde_Cipher_cast128 extends Horde_Cipher {
         0x91da55f4, 0x40a230f3, 0xd1988f35, 0xb6e318d2, 0x3ffa50bc, 0x3d40f021, 0xc3c0bdae, 0x4958c24c,
         0x518f36b2, 0x84b1d370, 0x0fedce83, 0x878ddada, 0xf2a279c7, 0x94e01be8, 0x90716f4b, 0x954b8aa3);
 
+    /**
+     * S box (s8)
+     *
+     * @var array
+     */
     var $s8 = array(
         0xe216300d, 0xbbddfffc, 0xa7ebdabd, 0x35648095, 0x7789f8b7, 0xe6c1121b, 0x0e241600, 0x052ce8b5,
         0x11a9cfb0, 0xe5952f11, 0xece7990a, 0x9386d174, 0x2a42931c, 0x76e38111, 0xb12def3a, 0x37ddddfc,
@@ -290,25 +328,31 @@ class Horde_Cipher_cast128 extends Horde_Cipher {
         0xe97625a5, 0x0614d1b7, 0x0e25244b, 0x0c768347, 0x589e8d82, 0x0d2059d1, 0xa466bb1e, 0xf8da0a82,
         0x04f19130, 0xba6e4ec0, 0x99265164, 0x1ee7230d, 0x50b2ad80, 0xeaee6801, 0x8db2a283, 0xea8bf59e);
 
-    /* Masking Keys */
+    /**
+     * Masking Keys
+     *
+     * @var array
+     */
     var $_Km = array();
 
-    /* Rotate Keys */
+    /**
+     * Rotate Keys
+     *
+     * @var array
+     */
     var $_Kr = array();
 
-    /* The number of rounds to do */
+    /**
+     * The number of rounds to do
+     *
+     * @var integer
+     */
     var $_rounds = 16;
 
-
-    /* Constructor */
-    function Horde_Cipher_cast128($params = null)
-    {
-    }
-
     /**
-     * Set the key to be used for en/decryption
+     * Set the key to be used for en/decryption.
      *
-     * @param String $key   The key to use
+     * @param string $key  The key to use.
      */
     function setKey($key)
     {
@@ -320,9 +364,9 @@ class Horde_Cipher_cast128 extends Horde_Cipher {
     }
 
     /**
-     * Return the size of the blocks that this cipher needs
+     * Return the size of the blocks that this cipher needs.
      *
-     * @return Integer  The number of characters per block
+     * @return integer  The number of characters per block.
      */
     function getBlockSize()
     {
@@ -330,12 +374,12 @@ class Horde_Cipher_cast128 extends Horde_Cipher {
     }
 
     /**
-     * Encrypt a block on data.
+     * Encrypt a block of data.
      *
-     * @param String $block         The data to encrypt
-     * @param optional String $key  The key to use
+     * @param string $block  The data to encrypt.
+     * @param string $key    The key to use.
      *
-     * @return String the encrypted output
+     * @return string  The encrypted output.
      */
     function encryptBlock($block, $key = null)
     {
@@ -373,12 +417,12 @@ class Horde_Cipher_cast128 extends Horde_Cipher {
     }
 
     /**
-     * Decrypt a block on data.
+     * Decrypt a block of data.
      *
-     * @param String $block         The data to decrypt
-     * @param optional String $key  The key to use
+     * @param string $block  The data to decrypt.
+     * @param string $key    The key to use.
      *
-     * @return String the decrypted output
+     * @return string  The decrypted output.
      */
     function decryptBlock($block, $key = null)
     {
@@ -441,7 +485,6 @@ class Horde_Cipher_cast128 extends Horde_Cipher {
      */
     function f2($d, $Km, $Kr)
     {
-
         $I = $Km ^ $d;
 
         /* Nasty Fix as
@@ -478,10 +521,11 @@ class Horde_Cipher_cast128 extends Horde_Cipher {
     }
 
     /**
-     * Convert a string into a array of the ordinates
-     * of it's characters
+     * Convert a string into a array of the ordinates of its characters.
      *
-     * @return array    The data
+     * @param string $data  Data to convert.
+     *
+     * @return array  The converted data.
      */
     function _formatData($data)
     {
@@ -489,24 +533,26 @@ class Horde_Cipher_cast128 extends Horde_Cipher {
     }
 
     /**
-     * Converts a text key into an array
-     * padding with \0's to a length of 128 bits
+     * Converts a text key into an array padded with \0's to a length of 128
+     * bits.
      *
-     * @return array    The key
+     * @param string $key  The key to format.
+     *
+     * @return array  The padded key.
      */
     function _formatKey($key)
     {
         $res = array_values(unpack('C*', $key));
-        for($i = count($res); $i < 16; $i++) {
+        for ($i = count($res); $i < 16; $i++) {
             $res[$i] = 0;
         }
         return $res;
     }
 
     /**
-     * Create the complete key shedule
+     * Create the complete key shedule.
      *
-     * @param array $key
+     * @param array $key  The key to use.
      */
     function _keySchedule($key)
     {
@@ -634,11 +680,11 @@ class Horde_Cipher_cast128 extends Horde_Cipher {
     }
 
     /**
-     * Unpack the 32 bit value into an array of four 8 bit values
+     * Unpack the 32 bit value into an array of four 8 bit values.
      *
-     * @param Integer $x    32 bit value.
+     * @param integer $x  32 bit value.
      *
-     * @return Array        Array containing four 8 bit values MS to LS.
+     * @return array  Four 8 bit values MS to LS.
      */
     function _split($x)
     {
@@ -646,12 +692,12 @@ class Horde_Cipher_cast128 extends Horde_Cipher {
     }
 
     /**
-     * Pack four 8 bit values into one 32 bit values
+     * Pack four 8 bit values into one 32 bit value.
      *
-     * @param Array $key        Array of 8 bit values.
-     * @param Integer $start    Start position in array to get 4 values from.
+     * @param array $key      Array of 8 bit values.
+     * @param integer $start  Start position in array to get 4 values from.
      *
-     * @return Interget         32 bit value.
+     * @return integer  32 bit value.
      */
     function _combine4($key, $start = 0)
     {
