@@ -2,78 +2,87 @@
 
 require_once 'Horde/Crypt.php';
 
-/* Armor Header Lines - From RFC 2440
+/**
+ * Armor Header Lines - From RFC 2440
+ *
+ *  An Armor Header Line consists of the appropriate header line text
+ *  surrounded by five (5) dashes ('-', 0x2D) on either side of the header
+ *  line text. The header line text is chosen based upon the type of data that
+ *  is being encoded in Armor, and how it is being encoded. Header line texts
+ *  include the following strings:
+ *
+ *  All Armor Header Lines are prefixed with 'PGP'.
+ *
+ *  The Armor Tail Line is composed in the same manner as the Armor Header
+ *  Line, except the string "BEGIN" is replaced by the string "END."
+ */
 
-   An Armor Header Line consists of the appropriate header line text
-   surrounded by five (5) dashes ('-', 0x2D) on either side of the
-   header line text. The header line text is chosen based upon the
-   type of data that is being encoded in Armor, and how it is being
-   encoded. Header line texts include the following strings:
-
-   All Armor Header Lines are prefixed with 'PGP'.
-
-   The Armor Tail Line is composed in the same manner as the Armor
-   Header Line, except the string "BEGIN" is replaced by the string
-   "END." */
-
-/** @constant PGP_ARMOR_MESSAGE Used for signed, encrypted, or compressed files. */
+/** Used for signed, encrypted, or compressed files. */
 define('PGP_ARMOR_MESSAGE', 1);
 
-/** @constant PGP_ARMOR_SIGNED_MESSAGE Used for signed files. */
+/** Used for signed files. */
 define('PGP_ARMOR_SIGNED_MESSAGE', 2);
 
-/** @constant PGP_ARMOR_PUBLIC_KEY Used for armoring public keys. */
+/** Used for armoring public keys. */
 define('PGP_ARMOR_PUBLIC_KEY', 3);
 
-/** @constant PGP_ARMOR_PRIVATE_KEY Used for armoring private keys. */
+/** Used for armoring private keys. */
 define('PGP_ARMOR_PRIVATE_KEY', 4);
 
-/** @constant PGP_ARMOR_SIGNATURE Used for detached signatures, PGP/MIME signatures, and natures following clearsigned messages. */
+/**
+ * Used for detached signatures, PGP/MIME signatures, and natures following
+ * clearsigned messages.
+ */
 define('PGP_ARMOR_SIGNATURE', 5);
 
-/** @constant PGP_ARMOR_TEXT Regular text contained in an PGP message. */
+/** Regular text contained in an PGP message. */
 define('PGP_ARMOR_TEXT', 6);
 
 
-/** @constant PGP_PUBLIC_KEYSERVER The default public PGP keyserver to use. */
+/** The default public PGP keyserver to use. */
 define('PGP_KEYSERVER_PUBLIC', 'wwwkeys.pgp.net');
 
-/** @constant PGP_KEYSERVER_REFUSE The number of times the keyserver refuses connection before an error is returned. */
+/**
+ * The number of times the keyserver refuses connection before an error is
+ * returned.
+ */
 define('PGP_KEYSERVER_REFUSE', 3);
 
-/** @constant PGP_KEYSERVER_TIMEOUT The number of seconds that PHP will attempt to connect to the keyserver before it will stop processing the request. */
+/**
+ * The number of seconds that PHP will attempt to connect to the keyserver
+ * before it will stop processing the request.
+ */
 define('PGP_KEYSERVER_TIMEOUT', 10);
 
 
 /**
- * Horde_Crypt_pgp:: provides a framework for Horde applications to
- * interact with the GNU Privacy Guard program ("GnuPG").  GnuPG implements
- * the OpenPGP standard (RFC 2440).
+ * Horde_Crypt_pgp:: provides a framework for Horde applications to interact
+ * with the GNU Privacy Guard program ("GnuPG").  GnuPG implements the OpenPGP
+ * standard (RFC 2440).
  *
  * GnuPG Website: http://www.gnupg.org/
  *
  * This class has been developed with, and is only guaranteed to work with,
  * Version 1.21 or above of GnuPG.
  *
- * $Horde: framework/Crypt/Crypt/pgp.php,v 1.85.2.1 2005/01/03 12:18:57 jan Exp $
+ * $Horde: framework/Crypt/Crypt/pgp.php,v 1.85.2.15 2006/07/20 09:02:38 jan Exp $
  *
- * Copyright 2002-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2002-2006 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @version $Revision: 1.85.2.1 $
  * @since   Horde 3.0
  * @package Horde_Crypt
  */
 class Horde_Crypt_pgp extends Horde_Crypt {
 
     /**
-     * Strings in armor header lines used to distinguish between the
-     * different types of PGP decryption/encryption.
+     * Strings in armor header lines used to distinguish between the different
+     * types of PGP decryption/encryption.
      *
-     * @var array $_armor
+     * @var array
      */
     var $_armor = array(
         'MESSAGE'            =>  PGP_ARMOR_MESSAGE,
@@ -86,7 +95,7 @@ class Horde_Crypt_pgp extends Horde_Crypt {
     /**
      * The list of PGP hash algorithms (from RFC 3156).
      *
-     * @var array $_hashAlg
+     * @var array
      */
     var $_hashAlg = array(
         1  =>  'pgp-md5',
@@ -94,36 +103,48 @@ class Horde_Crypt_pgp extends Horde_Crypt {
         3  =>  'pgp-ripemd160',
         5  =>  'pgp-md2',
         6  =>  'pgp-tiger192',
-        7  =>  'pgp-hava1-5-160'
+        7  =>  'pgp-haval-5-160',
+        8  =>  'pgp-sha256',
+        9  =>  'pgp-sha384',
+        10 =>  'pgp-sha512',
     );
 
     /**
      * GnuPG program location/common options.
      *
-     * @var array $_gnupg
+     * @var array
      */
     var $_gnupg;
 
     /**
      * Filename of the temporary public keyring.
      *
-     * @var string $_publicKeyring
+     * @var string
      */
     var $_publicKeyring;
 
     /**
      * Filename of the temporary private keyring.
      *
-     * @var string $_privateKeyring
+     * @var string
      */
     var $_privateKeyring;
 
+    /**
+     * The existence of this property indicates that multiple recipient
+     * encryption is available.
+     *
+     * @since Horde 3.1
+     *
+     * @deprecated
+     *
+     * @var boolean
+     */
+    var $multipleRecipientEncryption = true;
 
     /**
      * Constructor.
      *
-     * @access public
-     *
      * @param array $params  Parameter array containing the path to the GnuPG
      *                       binary (key = 'program') and to a temporary
      *                       directory.
@@ -154,19 +175,16 @@ class Horde_Crypt_pgp extends Horde_Crypt {
     }
 
     /**
-     * Generate a personal Public/Private keypair combination.
+     * Generates a personal Public/Private keypair combination.
      *
-     * @access public
-     *
-     * @param string $realname             The name to use for the key.
-     * @param string $email                The email to use for the key.
-     * @param string $passphrase           The passphrase to use for the key.
-     * @param optional string $comment     The comment to use for the key.
-     * @param optional integer $keylength  The keylength to use for the key.
+     * @param string $realname    The name to use for the key.
+     * @param string $email       The email to use for the key.
+     * @param string $passphrase  The passphrase to use for the key.
+     * @param string $comment     The comment to use for the key.
+     * @param integer $keylength  The keylength to use for the key.
      *
      * @return array  An array consisting of the public key and the private
-     *                key.
-     *                Returns PEAR_Error object on error.
+     *                key, or PEAR_Error on error.
      * <pre>
      * Return array:
      * Key            Value
@@ -191,20 +209,20 @@ class Horde_Crypt_pgp extends Horde_Crypt {
         /* Create the config file necessary for GnuPG to run in batch mode. */
         /* TODO: Sanitize input, More user customizable? */
         $input = array();
-        $input[] = "%pubring " . $pub_file;
-        $input[] = "%secring " . $secret_file;
-        $input[] = "Key-Type: DSA";
-        $input[] = "Key-Length: " . $keylength;
-        $input[] = "Subkey-Type: ELG-E";
-        $input[] = "Subkey-Length: " . $keylength;
-        $input[] = "Name-Real: " . $realname;
+        $input[] = '%pubring ' . $pub_file;
+        $input[] = '%secring ' . $secret_file;
+        $input[] = 'Key-Type: DSA';
+        $input[] = 'Key-Length: 1024';
+        $input[] = 'Subkey-Type: ELG-E';
+        $input[] = 'Subkey-Length: ' . $keylength;
+        $input[] = 'Name-Real: ' . $realname;
         if (!empty($comment)) {
-            $input[] = "Name-Comment: " . $comment;
+            $input[] = 'Name-Comment: ' . $comment;
         }
-        $input[] = "Name-Email: " . $email;
-        $input[] = "Expire-Date: 0";
-        $input[] = "Passphrase: " . $passphrase;
-        $input[] = "%commit";
+        $input[] = 'Name-Email: ' . $email;
+        $input[] = 'Expire-Date: 0';
+        $input[] = 'Passphrase: ' . $passphrase;
+        $input[] = '%commit';
 
         /* Run through gpg binary. */
         $cmdline = array(
@@ -227,15 +245,13 @@ class Horde_Crypt_pgp extends Horde_Crypt {
     }
 
     /**
-     * Return information on a PGP data block.
-     *
-     * @access public
+     * Returns information on a PGP data block.
      *
      * @param string $pgpdata  The PGP data block.
      *
-     * @return array  An array with information on the PGP data block.
-     *                If an element is not present in the data block,
-     *                it will likewise not be set in the array.
+     * @return array  An array with information on the PGP data block. If an
+     *                element is not present in the data block, it will
+     *                likewise not be set in the array.
      * <pre>
      * Array Format:
      * -------------
@@ -264,13 +280,13 @@ class Horde_Crypt_pgp extends Horde_Crypt {
      *         )
      *     )
      * )
+     * </pre>
      *
      * Each user ID will be stored in the array 'signature' and have data
      * associated with it, including an array for information on each
-     * signature that has signed that UID. Signatures not associated with
-     * a UID (e.g. revocation signatures and sub keys) will be stored under
-     * the special keyword '_SIGNATURE'.
-     * </pre>
+     * signature that has signed that UID. Signatures not associated with a
+     * UID (e.g. revocation signatures and sub keys) will be stored under the
+     * special keyword '_SIGNATURE'.
      */
     function pgpPacketInformation($pgpdata)
     {
@@ -292,8 +308,8 @@ class Horde_Crypt_pgp extends Horde_Crypt {
         $result = $this->_callGpg($cmdline, 'r');
 
         foreach (explode("\n", $result->stdout) as $line) {
-            /* Headers are prefaced with a ':' as the first character
-               on the line. */
+            /* Headers are prefaced with a ':' as the first character on the
+               line. */
             if (strpos($line, ':') === 0) {
                 $lowerLine = String::lower($line);
 
@@ -380,8 +396,6 @@ class Horde_Crypt_pgp extends Horde_Crypt {
     /**
      * Returns human readable information on a PGP key.
      *
-     * @access public
-     *
      * @param string $pgpdata  The PGP data block.
      *
      * @return string  Tabular information on the PGP key.
@@ -421,17 +435,15 @@ class Horde_Crypt_pgp extends Horde_Crypt {
     }
 
     /**
-     * Returns only information on the first ID that matches the email
-     * address input.
-     *
-     * @access public
+     * Returns only information on the first ID that matches the email address
+     * input.
      *
      * @param string $pgpdata  The PGP data block.
      * @param string $email    An e-mail address.
      *
-     * @return array  An array with information on the PGP data block.
-     *                If an element is not present in the data block,
-     *                it will likewise not be set in the array.
+     * @return array  An array with information on the PGP data block. If an
+     *                element is not present in the data block, it will
+     *                likewise not be set in the array.
      * <pre>
      * Array Fields:
      * -------------
@@ -482,12 +494,9 @@ class Horde_Crypt_pgp extends Horde_Crypt {
     }
 
     /**
-     * Return information on a PGP signature embedded in PGP data.
-     * Similar to pgpPacketSignature(), but returns information by
-     * unique User ID Index (format id{n} where n is an integer of
-     * 1 or greater).
-     *
-     * @access public
+     * Returns information on a PGP signature embedded in PGP data.  Similar
+     * to pgpPacketSignature(), but returns information by unique User ID
+     * Index (format id{n} where n is an integer of 1 or greater).
      *
      * @param string $pgpdata   See pgpPacketSignature().
      * @param integer $uid_idx  The UID index.
@@ -515,7 +524,7 @@ class Horde_Crypt_pgp extends Horde_Crypt {
     }
 
     /**
-     * Add some data to the pgpPacketSignature*() function array.
+     * Adds some data to the pgpPacketSignature*() function array.
      *
      * @access private
      *
@@ -557,14 +566,12 @@ class Horde_Crypt_pgp extends Horde_Crypt {
     }
 
     /**
-     * Gets the short fingerprint (Key ID) of the key used to sign a
-     * block of PGP data.
-     *
-     * @access public
+     * Returns the short fingerprint (Key ID) of the key used to sign a block
+     * of PGP data.
      *
      * @param string $text  The PGP signed text block.
      *
-     * @return string   The short fingerprint of the key used to sign $text.
+     * @return string  The short fingerprint of the key used to sign $text.
      */
     function getSignersFingerprint($text)
     {
@@ -582,7 +589,7 @@ class Horde_Crypt_pgp extends Horde_Crypt {
             '2>&1'
         );
         $result = $this->_callGpg($cmdline, 'r');
-        if (preg_match("/gpg:\sSignature\smade.*ID\s+([A-F0-9]{8})\s+/", $result->stdout, $matches)) {
+        if (preg_match('/gpg:\sSignature\smade.*ID\s+([A-F0-9]{8})\s+/', $result->stdout, $matches)) {
             $fingerprint = $matches[1];
         }
 
@@ -592,15 +599,12 @@ class Horde_Crypt_pgp extends Horde_Crypt {
     /**
      * Verify a passphrase for a given public/private keypair.
      *
-     * @access public
-     *
      * @param string $public_key   The user's PGP public key.
      * @param string $private_key  The user's PGP private key.
      * @param string $passphrase   The user's passphrase.
      *
      * @return boolean  Returns true on valid passphrase, false on invalid
-     *                  passphrase.
-     *                  Returns PEAR_Error on error.
+     *                  passphrase, and PEAR_Error on error.
      */
     function verifyPassphrase($public_key, $private_key, $passphrase)
     {
@@ -626,9 +630,7 @@ class Horde_Crypt_pgp extends Horde_Crypt {
     }
 
     /**
-     * Parse a message into text and PGP components.
-     *
-     * @access public
+     * Parses a message into text and PGP components.
      *
      * @param string $text  The text to parse.
      *
@@ -636,7 +638,7 @@ class Horde_Crypt_pgp extends Horde_Crypt {
      *                text corresponding to their actual order.
      * <pre>
      * Return array:
-     * Key       Value
+     * Key         Value
      * -------------------------------------------------
      * 'type'  =>  The type of data contained in block.
      *             Valid types are defined at the top of this class
@@ -655,7 +657,7 @@ class Horde_Crypt_pgp extends Horde_Crypt {
         $temp_array['type'] = PGP_ARMOR_TEXT;
 
         foreach ($buffer as $value) {
-            if (preg_match("/^-----(BEGIN|END) PGP ([^-]+)-----\s*$/", $value, $matches)) {
+            if (preg_match('/^-----(BEGIN|END) PGP ([^-]+)-----\s*$/', $value, $matches)) {
                 if (isset($temp_array['data'])) {
                     $data[] = $temp_array;
                 }
@@ -682,16 +684,13 @@ class Horde_Crypt_pgp extends Horde_Crypt {
     }
 
     /**
-     * Get a PGP public key from a public keyserver.
+     * Returns a PGP public key from a public keyserver.
      *
-     * @access public
+     * @param string $fprint  The fingerprint of the PGP key.
+     * @param string $server  The keyserver to use.
+     * @param float $timeout  The keyserver timeout.
      *
-     * @param string $fprint           The fingerprint of the PGP key.
-     * @param optional string $server  The keyserver to use.
-     * @param optional float $timeout  The keyserver timeout.
-     *
-     * @return string  The PGP public key.
-     *                 Returns PEAR_Error object on error/failure.
+     * @return string  The PGP public key, or PEAR_Error on error.
      */
     function getPublicKeyserver($fprint, $server = PGP_KEYSERVER_PUBLIC,
                                 $timeout = PGP_KEYSERVER_TIMEOUT)
@@ -724,13 +723,11 @@ class Horde_Crypt_pgp extends Horde_Crypt {
     /**
      * Sends a PGP public key to a public keyserver.
      *
-     * @access public
+     * @param string $pubkey  The PGP public key
+     * @param string $server  The keyserver to use.
+     * @param float $timeout  The keyserver timeout.
      *
-     * @param string $pubkey           The PGP public key
-     * @param optional string $server  The keyserver to use.
-     * @param optional float $timeout  The keyserver timeout.
-     *
-     * @return object PEAR_Error  Returns PEAR_Error object on error/failure.
+     * @return PEAR_Error  PEAR_Error on error/failure.
      */
     function putPublicKeyserver($pubkey, $server = PGP_KEYSERVER_PUBLIC,
                                 $timeout = PGP_KEYSERVER_TIMEOUT)
@@ -746,15 +743,24 @@ class Horde_Crypt_pgp extends Horde_Crypt {
         /* Connect to the public keyserver. _connectKeyserver() returns a
            PEAR_Error object on error and the output text on success. */
         $pubkey = 'keytext=' . urlencode(rtrim($pubkey));
-        $cmd = "POST /pks/add HTTP/1.0\nContent-Length: " . strlen($pubkey) . "\n\n" . $pubkey . "\n";
-        $result = $this->_connectKeyserver($cmd, $server, $timeout);
+        $cmd = array(
+            'POST /pks/add HTTP/1.0',
+            'Host: ' . $server . ':11371',
+            'User-Agent: Horde Application Framework 3.1',
+            'Content-Type: application/x-www-form-urlencoded',
+            'Content-Length: ' . strlen($pubkey),
+            'Connection: close',
+            '',
+            $pubkey
+        );
+        $result = $this->_connectKeyserver(implode("\r\n", $cmd), $server, $timeout);
         if (is_a($result, 'PEAR_Error')) {
             return $result;
         }
     }
 
     /**
-     * Connect to a public key server via HKP (Horrowitz Keyserver Protocol).
+     * Connects to a public key server via HKP (Horrowitz Keyserver Protocol).
      *
      * @access private
      *
@@ -762,8 +768,8 @@ class Horde_Crypt_pgp extends Horde_Crypt {
      * @param string $server   The keyserver to use.
      * @param float $timeout   The timeout value.
      *
-     * @return mixed  Returns the text from standard output on success.
-     *                Returns PEAR_Error object on error/failure.
+     * @return string  The text from standard output on success, or PEAR_Error
+     *                 on error/failure.
      */
     function _connectKeyserver($command, $server, $timeout)
     {
@@ -803,17 +809,14 @@ class Horde_Crypt_pgp extends Horde_Crypt {
     }
 
     /**
-     * Encrypt text using PGP.
-     *
-     * @access public
+     * Encrypts text using PGP.
      *
      * @param string $text   The text to be PGP encrypted.
      * @param array $params  The parameters needed for encryption.
-     *                       See the individual _encrypt*() functions for
-     *                       the parameter requirements.
+     *                       See the individual _encrypt*() functions for the
+     *                       parameter requirements.
      *
-     * @return string  The encrypted message.
-     *                 Returns PEAR_Error object on error.
+     * @return string  The encrypted message, or PEAR_Error on error.
      */
     function encrypt($text, $params = array())
     {
@@ -827,17 +830,14 @@ class Horde_Crypt_pgp extends Horde_Crypt {
     }
 
     /**
-     * Decrypt text using PGP.
-     *
-     * @access public
+     * Decrypts text using PGP.
      *
      * @param string $text   The text to be PGP decrypted.
      * @param array $params  The parameters needed for decryption.
-     *                       See the individual _decrypt*() functions for
-     *                       the parameter requirements.
+     *                       See the individual _decrypt*() functions for the
+     *                       parameter requirements.
      *
-     * @return string  The decrypted message.
-     *                 Returns PEAR_Error object on error.
+     * @return string  The decrypted message, or PEAR_Error on error.
      */
     function decrypt($text, $params = array())
     {
@@ -856,8 +856,8 @@ class Horde_Crypt_pgp extends Horde_Crypt {
      *
      * @access private
      *
-     * @param optional string $type  The type of key to analyze.
-     *                               Either 'public' (Default) or 'private'
+     * @param string $type  The type of key to analyze. Either 'public'
+     *                      (Default) or 'private'
      *
      * @return string  Command line keystring option to use with gpg program.
      */
@@ -879,14 +879,14 @@ class Horde_Crypt_pgp extends Horde_Crypt {
     }
 
     /**
-     * Add PGP keys to the keyring.
+     * Adds PGP keys to the keyring.
      *
      * @access private
      *
-     * @param mixed $keys            A single key or an array of key(s) to
-     *                               add to the keyring.
-     * @param optional string $type  The type of key(s) to add.
-     *                               Either 'public' (Default) or 'private'
+     * @param mixed $keys   A single key or an array of key(s) to add to the
+     *                      keyring.
+     * @param string $type  The type of key(s) to add. Either 'public'
+     *                      (Default) or 'private'
      *
      * @return string  Command line keystring option to use with gpg program.
      */
@@ -913,7 +913,7 @@ class Horde_Crypt_pgp extends Horde_Crypt {
     }
 
     /**
-     * Encrypt a message in PGP format using a public key.
+     * Encrypts a message in PGP format using a public key.
      *
      * @access private
      *
@@ -923,65 +923,74 @@ class Horde_Crypt_pgp extends Horde_Crypt {
      * Parameters:
      * ===========
      * 'type'    =>  'message' (REQUIRED)
-     * 'pubkey'  =>  PGP public key. (REQUIRED)
+     * 'pubkey'  =>  PGP public key. (Optional) (DEPRECATED)
      * 'email'   =>  E-mail address of recipient. If not present, or not found
      *               in the public key, the first e-mail address found in the
-     *               key will be used instead. (Optional)
+     *               key will be used instead. (Optional) (DEPRECATED)
+     * 'recips'  =>  An array with the e-mail address of the recipient as
+     *               the key and that person's public key as the value.
+     *               (REQUIRED)
      * </pre>
      *
-     * @return string  The encrypted message.
-     *                 Return PEAR_Error object on error.
+     * @return string  The encrypted message, or PEAR_Error on error.
      */
     function _encryptMessage($text, $params)
     {
         $email = null;
 
-        /* Check for required parameters. */
-        if (!isset($params['pubkey'])) {
-            return PEAR::raiseError(_("A public PGP key is required to encrypt a message."), 'horde.error');
-        }
-
-        /* Get information on the key. */
-        if (isset($params['email'])) {
-            $key_info = $this->pgpPacketSignature($params['pubkey'], $params['email']);
-            if (!empty($key_info)) {
-                $email = $key_info['email'];
+        if (!isset($params['recips'])) {
+            /* Check for required parameters. */
+            if (!isset($params['pubkey'])) {
+                return PEAR::raiseError(_("A public PGP key is required to encrypt a message."), 'horde.error');
             }
-        }
 
-        /* If we have no email address at this point, use the first email
-           address found in the public key. */
-        if (empty($email)) {
-            $key_info = $this->pgpPacketInformation($params['pubkey']);
-            if (isset($key_info['signature']['id1']['email'])) {
-                $email = $key_info['signature']['id1']['email'];
-            } else {
-                return PEAR::raiseError(_("Could not determine the recipient's e-mail address."), 'horde.error');
+            /* Get information on the key. */
+            if (isset($params['email'])) {
+                $key_info = $this->pgpPacketSignature($params['pubkey'], $params['email']);
+                if (!empty($key_info)) {
+                    $email = $key_info['email'];
+                }
             }
+
+            /* If we have no email address at this point, use the first email
+               address found in the public key. */
+            if (empty($email)) {
+                $key_info = $this->pgpPacketInformation($params['pubkey']);
+                if (isset($key_info['signature']['id1']['email'])) {
+                    $email = $key_info['signature']['id1']['email'];
+                } else {
+                    return PEAR::raiseError(_("Could not determine the recipient's e-mail address."), 'horde.error');
+                }
+            }
+
+            $params['recips'] = array($email => $params['pubkey']);
         }
 
         /* Store public key in temporary keyring. */
-        $keyring = $this->_putInKeyring($params['pubkey']);
+        $keyring = $this->_putInKeyring(array_values($params['recips']));
 
         /* Encrypt the document. */
         $cmdline = array(
             '--armor',
             '--batch',
             '--always-trust',
-            '--recipient ' . $email,
             $keyring,
             '--encrypt'
         );
-        $result = $this->_callGpg($cmdline, 'w', $text, true);
+        foreach (array_keys($params['recips']) as $val) {
+            $cmdline[] = '--recipient ' . $val;
+        }
+        $result = $this->_callGpg($cmdline, 'w', $text, true, true);
         if (empty($result->output)) {
-            return PEAR::raiseError(_("Could not PGP encrypt message."), 'horde.error');
+            $error = preg_replace('/\n.*/', '', $result->stderr);
+            return PEAR::raiseError(_("Could not PGP encrypt message: ") . $error, 'horde.error');
         }
 
         return $result->output;
     }
 
     /**
-     * Sign a message in PGP format using a private key.
+     * Signs a message in PGP format using a private key.
      *
      * @access private
      *
@@ -999,8 +1008,7 @@ class Horde_Crypt_pgp extends Horde_Crypt {
      *                   'detach'     --  Make a detached signature (DEFAULT)
      * </pre>
      *
-     * @return string  The signed message.
-     *                 Return PEAR_Error object on error.
+     * @return string  The signed message, or PEAR_Error on error.
      */
     function _encryptSignature($text, $params)
     {
@@ -1050,17 +1058,18 @@ class Horde_Crypt_pgp extends Horde_Crypt {
         );
 
         /* Sign the document. */
-        $result = $this->_callGpg($cmdline, 'w', $params['passphrase'], true);
+        $result = $this->_callGpg($cmdline, 'w', $params['passphrase'], true, true);
         if (empty($result->output)) {
-            return PEAR::raiseError(_("Could not PGP sign message."), 'horde.error');
+            $error = preg_replace('/\n.*/', '', $result->stderr);
+            return PEAR::raiseError(_("Could not PGP sign message: ") . $error, 'horde.error');
         } else {
             return $result->output;
         }
     }
 
     /**
-     * Decrypt an PGP encrypted message using a private/public keypair
-     * and a passhprase.
+     * Decrypts an PGP encrypted message using a private/public keypair and a
+     * passhprase.
      *
      * @access private
      *
@@ -1075,11 +1084,12 @@ class Horde_Crypt_pgp extends Horde_Crypt {
      * 'passphrase'  =>  Passphrase for PGP Key. (REQUIRED)
      * </pre>
      *
-     * @return object stdClass  An object.
-     *                          'message'     -  The decrypted message.
-     *                          'sig_result'  -  The result of the signature
-     *                                           test.
-     *                          Returns PEAR_Error object on error.
+     * @return stdClass  An object with the following properties, or PEAR_Error
+     *                   on error:
+     * <pre>
+     * 'message'     -  The decrypted message.
+     * 'sig_result'  -  The result of the signature test.
+     * </pre>
      */
     function _decryptMessage($text, $params)
     {
@@ -1123,7 +1133,8 @@ class Horde_Crypt_pgp extends Horde_Crypt {
         );
         $result = $this->_callGpg($cmdline, 'w', $params['passphrase'], true, true);
         if (empty($result->output)) {
-            return PEAR::raiseError(_("Could not decrypt PGP data."), 'horde.error');
+            $error = preg_replace('/\n.*/', '', $result->stderr);
+            return PEAR::raiseError(_("Could not decrypt PGP data: ") . $error, 'horde.error');
         }
 
         /* Create the return object. */
@@ -1142,7 +1153,7 @@ class Horde_Crypt_pgp extends Horde_Crypt {
     }
 
     /**
-     * Decrypt an PGP signed message using a public key.
+     * Decrypts an PGP signed message using a public key.
      *
      * @access private
      *
@@ -1156,9 +1167,8 @@ class Horde_Crypt_pgp extends Horde_Crypt {
      * 'signature'  =>  PGP signature block. (REQUIRED for detached signature)
      * </pre>
      *
-     * @return string  The verification message from gpg.
-     *                 If no signature, returns empty string.
-     *                 Returns PEAR_Error object on error.
+     * @return string  The verification message from gpg. If no signature,
+     *                 returns empty string, and PEAR_Error on error.
      */
     function _decryptSignature($text, $params)
     {
@@ -1220,7 +1230,7 @@ class Horde_Crypt_pgp extends Horde_Crypt {
     }
 
     /**
-     * Check signature result from the GnuPG binary.
+     * Checks signature result from the GnuPG binary.
      *
      * @access private
      *
@@ -1231,9 +1241,9 @@ class Horde_Crypt_pgp extends Horde_Crypt {
     function _checkSignatureResult($result)
     {
         /* Good signature:
-             gpg: Good signature from "blah blah blah (Comment)"
-           Bad signature:
-             gpg: BAD signature from "blah blah blah (Comment)" */
+         *   gpg: Good signature from "blah blah blah (Comment)"
+         * Bad signature:
+         *   gpg: BAD signature from "blah blah blah (Comment)" */
         if (strpos($result, 'gpg: BAD signature') !== false) {
             return PEAR::raiseError($result, 'horde.error');
         } elseif (strpos($result, 'gpg: Good signature') !== false) {
@@ -1244,18 +1254,14 @@ class Horde_Crypt_pgp extends Horde_Crypt {
     }
 
     /**
-     * Sign a MIME_Part using PGP.
+     * Signs a MIME_Part using PGP.
      *
-     * @access public
+     * @param MIME_Part $mime_part  The MIME_Part object to sign.
+     * @param array $params         The parameters required for signing.
+     *                              @see _encryptSignature().
      *
-     * @param object MIME_Part $mime_part  The MIME_Part object to sign.
-     * @param array $params                The parameters required for
-     *                                     signing.
-     *                                     See _encryptSignature().
-     *
-     * @return object MIME_Part  A MIME_Part object that is signed according
-     *                           to RFC 2015/3156.
-     *                           Returns PEAR_Error object on error.
+     * @return MIME_Part  A MIME_Part object that is signed according to RFC
+     *                    2015/3156, or PEAR_Error on error.
      */
     function signMIMEPart($mime_part, $params = array())
     {
@@ -1264,9 +1270,9 @@ class Horde_Crypt_pgp extends Horde_Crypt {
         $params = array_merge($params, array('type' => 'signature', 'sigtype' => 'detach'));
 
         /* RFC 2015 Requirements for a PGP signed message:
-           + Content-Type params 'micalg' & 'protocol' are REQUIRED.
-           + The digitally signed message MUST be constrained to 7 bits.
-           + The MIME headers MUST be a part of the signed data. */
+         * + Content-Type params 'micalg' & 'protocol' are REQUIRED.
+         * + The digitally signed message MUST be constrained to 7 bits.
+         * + The MIME headers MUST be a part of the signed data. */
 
         $mime_part->strict7bit(true);
         $msg_sign = $this->encrypt($mime_part->toCanonicalString(), $params);
@@ -1279,8 +1285,8 @@ class Horde_Crypt_pgp extends Horde_Crypt {
         $pgp_sign->setDescription(_("PGP Digital Signature"));
 
         /* Get the algorithim information from the signature. Since we are
-           analyzing a signature packet, we need to use the special keyword
-           '_SIGNATURE' - see Horde_Crypt_pgp. */
+         * analyzing a signature packet, we need to use the special keyword
+         * '_SIGNATURE' - see Horde_Crypt_pgp. */
         $sig_info = $this->pgpPacketSignature($msg_sign, '_SIGNATURE');
 
         /* Setup the multipart MIME Part. */
@@ -1295,17 +1301,14 @@ class Horde_Crypt_pgp extends Horde_Crypt {
     }
 
     /**
-     * Encrypt a MIME_Part using PGP.
+     * Encrypts a MIME_Part using PGP.
      *
-     * @access public
+     * @param MIME_Part $mime_part  The MIME_Part object to encrypt.
+     * @param array $params         The parameters required for encryption.
+     *                               @see _encryptMessage().
      *
-     * @param object MIME_Part $mime_part  The MIME_Part object to encrypt.
-     * @param array $params                The parameters required for
-     *                                     encryption. See _encryptMessage().
-     *
-     * @return object MIME_Part  A MIME_Part object that is encrypted
-     *                           according to RFC 2015/3156.
-     *                           Returns PEAR_Error on error.
+     * @return MIME_Part  A MIME_Part object that is encrypted according to RFC
+     *                    2015/3156, or PEAR_Error on error.
      */
     function encryptMIMEPart($mime_part, $params = array())
     {
@@ -1331,29 +1334,25 @@ class Horde_Crypt_pgp extends Horde_Crypt {
     }
 
     /**
-     * Sign and Encrypt a MIME_Part using PGP.
+     * Signs and encrypts a MIME_Part using PGP.
      *
-     * @access public
+     * @param MIME_Part $mime_part   The MIME_Part object to sign and encrypt.
+     * @param array $sign_params     The parameters required for signing.
+     *                               @see _encryptSignature().
+     * @param array $encrypt_params  The parameters required for encryption.
+     *                               @see _encryptMessage().
      *
-     * @param object MIME_Part $mime_part  The MIME_Part object to sign and
-     *                                     encrypt.
-     * @param array $sign_params           The parameters required for
-     *                                     signing. See _encryptSignature().
-     * @param array $encrypt_params        The parameters required for
-     *                                     encryption. See _encryptMessage().
-     *
-     * @return object MIME_Part  A MIME_Part object that is signed and
-     *                           encrypted according to RFC 2015/3156.
-     *                           Returns PEAR_Error on error.
+     * @return MIME_Part  A MIME_Part object that is signed and encrypted
+     *                    according to RFC 2015/3156, or PEAR_Error on error.
      */
     function signAndEncryptMIMEPart($mime_part, $sign_params = array(),
                                     $encrypt_params = array())
     {
         include_once 'Horde/MIME/Part.php';
 
-        /* RFC 2015 requires that the entire signed message be encrypted. */
-        /* We need to explicitly call using Horde_Crypt_pgp:: because we
-           don't know whether a subclass has extended these methods. */
+        /* RFC 2015 requires that the entire signed message be encrypted.  We
+         * need to explicitly call using Horde_Crypt_pgp:: because we don't
+         * know whether a subclass has extended these methods. */
         $part = $this->signMIMEPart($mime_part, $sign_params);
         if (is_a($part, 'PEAR_Error')) {
             return $part;
@@ -1369,21 +1368,18 @@ class Horde_Crypt_pgp extends Horde_Crypt {
     }
 
     /**
-     * Generate a MIME_Part object, in accordance with RFC 2015/3156, that
+     * Generates a MIME_Part object, in accordance with RFC 2015/3156, that
      * contains a public key.
      *
-     * @access public
-     *
      * @param string $key  The public key.
      *
-     * @return object MIME_Part  A MIME_Part object that contains the public
-     *                           key.
+     * @return MIME_Part  A MIME_Part object that contains the public key.
      */
     function publicKeyMIMEPart($key)
     {
         include_once 'Horde/MIME/Part.php';
 
-        $part = &new MIME_Part('application/pgp-keys', $key);
+        $part = &new MIME_Part('application/pgp-keys', $key, NLS::getCharset());
         $part->setDescription(_("PGP Public Key"));
 
         return $part;
@@ -1394,11 +1390,11 @@ class Horde_Crypt_pgp extends Horde_Crypt {
      *
      * @access private
      *
-     * @param array $options            TODO
-     * @param string $mode              TODO
-     * @param optional array $input     TODO
-     * @param optional boolean $output  TODO
-     * @param optional boolean $stderr  TODO
+     * @param array $options   TODO
+     * @param string $mode     TODO
+     * @param array $input     TODO
+     * @param boolean $output  TODO
+     * @param boolean $stderr  TODO
      *
      * @return stdClass  TODO
      */
@@ -1419,7 +1415,7 @@ class Horde_Crypt_pgp extends Horde_Crypt {
             /* Do we need standard error output? */
             if ($stderr) {
                 $stderr_file = $this->_createTempFile('horde-pgp', false);
-                $options[] = '> ' . $stderr_file . ' 2>&1';
+                $options[] = '2> ' . $stderr_file;
             }
         }
 
@@ -1452,14 +1448,10 @@ class Horde_Crypt_pgp extends Horde_Crypt {
         pclose($fp);
 
         if ($output) {
-            $fp = @fopen($output_file, 'r');
-            $data->output = @fread($fp, filesize($output_file));
-            @fclose($fp);
+            $data->output = file_get_contents($output_file);
             unlink($output_file);
             if ($stderr) {
-                $fp = @fopen($stderr_file, 'r');
-                $data->stderr = @fread($fp, filesize($stderr_file));
-                @fclose($fp);
+                $data->stderr = file_get_contents($stderr_file);
                 unlink($stderr_file);
             }
         }
