@@ -24,11 +24,11 @@
  * The table structure for the DataTree system is in
  * scripts/sql/horde_datatree.sql.
  *
- * $Horde: framework/DataTree/DataTree/sql.php,v 1.156.2.22 2005/08/23 17:02:58 jan Exp $
+ * $Horde: framework/DataTree/DataTree/sql.php,v 1.156.2.30 2006/07/13 09:04:20 jan Exp $
  *
- * Copyright 1999-2005 Stephane Huther <shuther@bigfoot.com>
- * Copyright 2001-2005 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2005 Jan Schneider <jan@horde.org>
+ * Copyright 1999-2006 Stephane Huther <shuther@bigfoot.com>
+ * Copyright 2001-2006 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2005-2006 Jan Schneider <jan@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If
  * you did not receive this file, see
@@ -87,26 +87,39 @@ class DataTree_sql extends DataTree {
      *
      * @access private
      *
-     * @param string  $root      Which portion of the tree to load.
-     *                           Defaults to all of it.
-     * @param boolean $loadTree  Load a tree starting at $root, or just the
-     *                           requested level and direct parents?
-     *                           Defaults to single level.
-     * @param boolean $reload    Re-load already loaded values?
+     * @param string  $root         Which portion of the tree to load.
+     *                              Defaults to all of it.
+     * @param boolean $loadTree     Load a tree starting at $root, or just the
+     *                              requested level and direct parents?
+     *                              Defaults to single level.
+     * @param boolean $reload       Re-load already loaded values?
+     * @param string  $sortby_name  Attribute name to use for sorting.
+     * @param string  $sortby_key   Attribute key to use for sorting.
+     * @param integer $direction    Sort direction:
+     *                              0 - ascending
+     *                              1 - descending
      *
      * @return mixed  True on success or a PEAR_Error on failure.
      */
-    function _load($root = DATATREE_ROOT, $loadTree = false, $reload = false)
+    function _load($root = DATATREE_ROOT, $loadTree = false, $reload = false,
+                   $sortby_name = null, $sortby_key = null, $direction = 0)
     {
-        /* Do NOT use DataTree::exists() here; that would cause an
-         * infinite loop. */
+        /* Do NOT use DataTree::exists() here; that would cause an infinite
+         * loop. */
         if (!$reload &&
             (in_array($root, $this->_nameMap) ||
-             (count($this->_data) && $root == DATATREE_ROOT))) {
+             (count($this->_data) && $root == DATATREE_ROOT)) ||
+            (!is_null($this->_sortHash) &&
+             isset($this->_data[$root]['sorter'][$this->_sortHash]))) {
             return true;
         }
 
-        $query = $this->_buildLoadQuery($root, $loadTree, false);
+        $query = $this->_buildLoadQuery($root,
+                                        $loadTree,
+                                        DATATREE_BUILD_SELECT,
+                                        $sortby_name,
+                                        $sortby_key,
+                                        $direction);
         if (is_a($query, 'PEAR_Error')) {
             return $query;
         }
@@ -135,7 +148,7 @@ class DataTree_sql extends DataTree {
      */
     function _count($root = DATATREE_ROOT)
     {
-        $query = $this->_buildLoadQuery($root, true, true);
+        $query = $this->_buildLoadQuery($root, true, DATATREE_BUILD_COUNT);
         if (is_a($query, 'PEAR_Error')) {
             return $query;
         }
@@ -143,7 +156,7 @@ class DataTree_sql extends DataTree {
             return 0;
         }
         Horde::logMessage('SQL Query by DataTree_sql::_count(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
-        return $this->_db->getOne($query);
+        return (int)$this->_db->getOne($query);
     }
 
     /**
@@ -151,79 +164,133 @@ class DataTree_sql extends DataTree {
      *
      * @access private
      *
-     * @param string  $root      Which portion of the tree to load.
-     *                           Defaults to all of it.
-     * @param boolean $loadTree  Load a tree starting at $root, or just the
-     *                           requested level and direct parents?
-     *                           Defaults to single level.
-     * @param integer $count     Only count objects?
+     * @param string  $root         Which portion of the tree to load.
+     *                              Defaults to all of it.
+     * @param boolean $loadTree     Load a tree starting at $root, or just the
+     *                              requested level and direct parents?
+     *                              Defaults to single level.
+     * @param integer $operation    Type of query to build
+     * @param string  $sortby_name  Attribute name to use for sorting.
+     * @param string  $sortby_key   Attribute key to use for sorting.
+     * @param integer $direction    Sort direction:
+     *                              0 - ascending
+     *                              1 - descending
      *
      * @return mixed  True on success or a PEAR_Error on failure.
      */
-    function _buildLoadQuery($root = DATATREE_ROOT, $loadTree = false, $count = false)
+    function _buildLoadQuery($root = DATATREE_ROOT, $loadTree = false,
+                             $operation = DATATREE_BUILD_SELECT,
+                             $sortby_name = null, $sortby_key = null,
+                             $direction = 0)
     {
+        $sorted = false;
+        $where = sprintf('c.group_uid = %s ', $this->_db->quote($this->_params['group']));
+
         if (!empty($root) && $root != DATATREE_ROOT) {
-            if (strpos($root, ':') !== false) {
-                $parts = explode(':', $root);
-                $root = array_pop($parts);
+            $parent_where = $this->_buildParentIds($root, $loadTree, 'c.');
+            if (!empty($parent_where) && !is_a($parent_where, 'PEAR_Error')) {
+                $where = sprintf('%s AND (%s)', $where, $parent_where);
             }
-            $root = (string)$root;
+        }
+        if (!is_null($sortby_name)) {
+            $where = sprintf('%s AND a.attribute_name = %s ', $where, $this->_db->quote($sortby_name));
+            $sorted = true;
+        }
+        if (!is_null($sortby_key)) {
+            $where = sprintf('%s AND a.attribute_key = %s ', $where, $this->_db->quote($sortby_key));
+            $sorted = true;
+        }
 
-            $query = 'SELECT datatree_id, datatree_parents' .
-                     ' FROM ' . $this->_params['table'] .
-                     ' WHERE datatree_name = ? AND group_uid = ?' .
-                     ' ORDER BY datatree_id';
-            $values = array($root,
-                            $this->_params['group']);
+        switch ($operation) {
+        case DATATREE_BUILD_COUNT:
+            $what = 'COUNT(*)';
+            break;
 
-            Horde::logMessage('SQL Query by DataTree_sql::_buildLoadQuery(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
-            $root = $this->_db->getAssoc($query, false, $values);
-            if (is_a($root, 'PEAR_Error')) {
-                return $root;
-            }
-            if (count($root) == 0) {
-                return '';
-            }
+        default:
+            $what = 'c.datatree_id, c.datatree_name, c.datatree_parents, c.datatree_order';
+            break;
+        }
 
-            $where = '';
-            $first_time = true;
-            foreach ($root as $object_id => $object_parents) {
-                $pstring = $object_parents . ':' . $object_id . '%';
-                $pquery = '';
-                if (!empty($object_parents)) {
-                    $ids = substr($object_parents, 1);
-                    $pquery = ' OR datatree_id IN (' . str_replace(':', ', ', $ids) . ')';
-                }
-                if ($loadTree) {
-                    $pquery .= ' OR datatree_parents = ' . $this->_db->quote(substr($pstring, 0, -2));
-                }
-
-                if (!$first_time) {
-                    $where .= ' OR ';
-                }
-                $where .= sprintf('datatree_parents LIKE %s OR datatree_id = %s%s',
-                                  $this->_db->quote($pstring),
-                                  $object_id,
-                                  $pquery);
-
-                $first_time = false;
-            }
-
-            $query = sprintf('SELECT %s FROM %s WHERE (%s) AND group_uid = %s',
-                             $count ? 'COUNT(*)' : 'datatree_id, datatree_name, datatree_parents, datatree_order',
+        if ($sorted) {
+            $query = sprintf('SELECT %s FROM %s c LEFT JOIN %s a ON (c.datatree_id = a.datatree_id OR c.datatree_name=%s) '.
+                             'WHERE %s GROUP BY c.datatree_id, c.datatree_name, c.datatree_parents, c.datatree_order ORDER BY a.attribute_value %s',
+                             $what,
                              $this->_params['table'],
+                             $this->_params['table_attributes'],
+                             $this->_db->quote($root),
                              $where,
-                             $this->_db->quote($this->_params['group']));
+                             ($direction == 1) ? 'DESC' : 'ASC');
         } else {
-            $query = sprintf('SELECT %s FROM %s WHERE group_uid = %s',
-                             $count ? 'COUNT(*)' : 'datatree_id, datatree_name, datatree_parents, datatree_order',
+            $query = sprintf('SELECT %s FROM %s c WHERE %s',
+                             $what,
                              $this->_params['table'],
-                             $this->_db->quote($this->_params['group']));
+                             $where);
         }
 
         return $query;
     }
 
+    /**
+     * Builds parent ID string for selecting trees.
+     *
+     * @access private
+     *
+     * @param string  $root      Which portion of the tree to load.
+     * @param boolean $loadTree  Load a tree starting at $root, or just the
+     *                           requested level and direct parents?
+     *                           Defaults to single level.
+     * @param string  $join_name Table join name
+     *
+     * @return string  Id list.
+     */
+    function _buildParentIds($root, $loadTree = false, $join_name = '')
+    {
+        if (strpos($root, ':') !== false) {
+            $parts = explode(':', $root);
+            $root = array_pop($parts);
+        }
+        $root = (string)$root;
+
+        $query = 'SELECT datatree_id, datatree_parents' .
+            ' FROM ' . $this->_params['table'] .
+            ' WHERE datatree_name = ? AND group_uid = ?' .
+            ' ORDER BY datatree_id';
+        $values = array($root,
+                        $this->_params['group']);
+
+        Horde::logMessage('SQL Query by DataTree_sql::_buildParentIds(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
+        $root = $this->_db->getAssoc($query, false, $values);
+        if (is_a($root, 'PEAR_Error') || !count($root)) {
+            return '';
+        }
+
+        $where = '';
+        $first_time = true;
+        foreach ($root as $object_id => $object_parents) {
+            $pstring = $object_parents . ':' . $object_id . '%';
+            $pquery = '';
+            if (!empty($object_parents)) {
+                $ids = substr($object_parents, 1);
+                $pquery = ' OR ' . $join_name . 'datatree_id IN (' . str_replace(':', ', ', $ids) . ')';
+            }
+            if ($loadTree) {
+                $pquery .= ' OR ' . $join_name . 'datatree_parents = ' . $this->_db->quote(substr($pstring, 0, -2));
+            }
+
+            if (!$first_time) {
+                $where .= ' OR ';
+            }
+            $where .= sprintf($join_name . 'datatree_parents LIKE %s OR ' . $join_name . 'datatree_id = %s%s',
+                              $this->_db->quote($pstring),
+                              $object_id,
+                              $pquery);
+
+            $first_time = false;
+        }
+
+        return $where;
+    }
+
     /**
      * Loads a set of objects identified by their unique IDs, and their
      * parents, into the $_data array.
@@ -277,6 +344,9 @@ class DataTree_sql extends DataTree {
         if (is_a($parents, 'PEAR_Error')) {
             return $parents;
         }
+        if (empty($parents)) {
+            return PEAR::raiseError(_("Not found."), null, null, null, 'DataTree ids ' . implode(', ', $ids) . ' not found.');
+        }
 
         $ids = array();
         foreach ($parents as $cid => $parent) {
@@ -311,6 +381,35 @@ class DataTree_sql extends DataTree {
         return $this->set($data, $this->_params['charset']);
     }
 
+    /**
+     * Returns a tree sorted by the specified attribute name and/or key.
+     *
+     * @since Horde 3.1
+     *
+     * @param string $root         Which portion of the tree to sort.
+     *                             Defaults to all of it.
+     * @param boolean $loadTree    Sort the tree starting at $root, or just the
+     *                             requested level and direct parents?
+     *                             Defaults to single level.
+     * @param string $sortby_name  Attribute name to use for sorting.
+     * @param string $sortby_key   Attribute key to use for sorting.
+     * @param integer $direction   Sort direction:
+     *                             0 - ascending
+     *                             1 - descending
+     */
+    function getSortedTree($root, $loadTree = false, $sortby_name = null,
+                           $sortby_key = null, $direction = 0)
+    {
+        $query = $this->_buildLoadQuery($root,
+                                        $loadTree,
+                                        DATATREE_BUILD_SELECT,
+                                        $sortby_name,
+                                        $sortby_key,
+                                        $direction);
+
+        return $this->_db->getAll($query);
+    }
+
     /**
      * Adds an object.
      *
@@ -873,6 +972,10 @@ class DataTree_sql extends DataTree {
      */
     function getAttributes($cid, $keys = false)
     {
+        if (empty($cid)) {
+            return array();
+        }
+
         if ($keys) {
             $filter = sprintf(' AND attribute_key IN (\'%s\')',
                               implode("', '", $keys));
@@ -938,11 +1041,15 @@ class DataTree_sql extends DataTree {
             return 0;
         }
 
-        list($query, $values) = $this->buildAttributeQuery($criteria,
-                                                           $parent,
-                                                           $allLevels,
-                                                           $restrict,
-                                                           true);
+        $aq = $this->buildAttributeQuery($criteria,
+                                         $parent,
+                                         $allLevels,
+                                         $restrict,
+                                         DATATREE_BUILD_COUNT);
+        if (is_a($aq, 'PEAR_Error')) {
+            return $aq;
+        }
+        list($query, $values) = $aq;
 
         Horde::logMessage('SQL Query by DataTree_sql::countByAttributes(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
@@ -965,33 +1072,50 @@ class DataTree_sql extends DataTree {
      *
      * @see buildAttributeQuery()
      *
-     * @param array   $criteria   The array of criteria.
-     * @param string  $parent     The parent node to start searching from.
-     * @param boolean $allLevels  Return all levels, or just the direct
-     *                            children of $parent? Defaults to all levels.
-     * @param string  $restrict   Only return attributes with the same
-     *                            attribute_name or attribute_id.
-     * @param integer $from       The object to start to fetching
-     * @param integer $count      The number of objects to fetch
+     * @param array   $criteria     The array of criteria.
+     * @param string  $parent       The parent node to start searching from.
+     * @param boolean $allLevels    Return all levels, or just the direct
+     *                              children of $parent? Defaults to all levels.
+     * @param string  $restrict     Only return attributes with the same
+     *                              attribute_name or attribute_id.
+     * @param integer $from         The object to start to fetching
+     * @param integer $count        The number of objects to fetch
+     * @param string  $sortby_name  Attribute name to use for sorting.
+     * @param string  $sortby_key   Attribute key to use for sorting.
+     * @param integer $direction    Sort direction:
+     *                                0 - ascending
+     *                                1 - descending
      */
-    function getByAttributes($criteria, $parent = DATATREE_ROOT, $allLevels = true, $restrict = 'name', $from = 0, $count = 0)
+    function getByAttributes($criteria, $parent = DATATREE_ROOT,
+                             $allLevels = true, $restrict = 'name', $from = 0,
+                             $count = 0, $sortby_name = null,
+                             $sortby_key = null, $direction = 0)
     {
         if (!count($criteria)) {
             return PEAR::raiseError('no criteria');
         }
 
-        list($query, $values) = $this->buildAttributeQuery($criteria, $parent, $allLevels, $restrict);
-
-        if ($count) {
-            $query = $this->_db->modifyLimitQuery($query, $from, $count);
-            if (is_a($query, 'PEAR_Error')) {
-                return $query;
-            }
+        $aq = $this->buildAttributeQuery($criteria,
+                                         $parent,
+                                         $allLevels,
+                                         $restrict,
+                                         DATATREE_BUILD_SELECT,
+                                         $sortby_name,
+                                         $sortby_key,
+                                         $direction);
+        if (is_a($aq, 'PEAR_Error')) {
+            return $aq;
         }
+        list($query, $values) = $aq;
 
         Horde::logMessage('SQL Query by DataTree_sql::getByAttributes(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
-        $result = $this->_db->query($query, $values);
+        if ($count) {
+            $result = $this->_db->limitQuery($query, $from, $count, $values);
+        } else {
+            $result = $this->_db->query($query, $values);
+        }
+
         if (is_a($result, 'PEAR_Error')) {
             Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
             return $result;
@@ -1004,6 +1128,134 @@ class DataTree_sql extends DataTree {
         return $rows;
     }
 
+    /**
+     * Sorts IDs by attribute values. IDs without attributes will be added to
+     * the end of the sorted list.
+     *
+     * @param array $unordered_ids  Array of ids to sort.
+     * @param array $sortby_name    Attribute name to use for sorting.
+     * @param array $sortby_key     Attribute key to use for sorting.
+     * @param array $direction      Sort direction:
+     *                                0 - ascending
+     *                                1 - descending
+     *
+     * @return array  Sorted ids.
+     */
+    function sortByAttributes($unordered_ids, $sortby_name = null,
+                              $sortby_key = null, $direction = 0)
+    {
+        /* Select ids ordered by attribute value. */
+        $where = '';
+        if (!is_null($sortby_name)) {
+            $where = sprintf(' AND attribute_name = %s ',
+                             $this->_db->quote($sortby_name));
+        }
+        if (!is_null($sortby_key)) {
+            $where = sprintf('%s AND attribute_key = %s ',
+                             $where,
+                             $this->_db->quote($sortby_key));
+        }
+
+        $query = sprintf('SELECT datatree_id FROM %s WHERE datatree_id IN (%s) %s ORDER BY attribute_value %s',
+                         $this->_params['table_attributes'],
+                         implode(',', $unordered_ids),
+                         $where,
+                         ($direction == 1) ? 'DESC' : 'ASC');
+
+        Horde::logMessage('SQL Query by DataTree_sql::sortByAttributes(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
+        $ordered_ids = $this->_db->getCol($query);
+
+        /* Make sure that some ids didn't get lost because has no such
+         * attribute name/key. Append them to the end. */
+        if (count($ordered_ids) != count($unordered_ids)) {
+            $ordered_ids = array_unique(array_merge($ordered_ids, $unordered_ids));
+        }
+
+        return $ordered_ids;
+    }
+
+    /**
+     * Returns a list of all of the available values of the given attribute
+     * name/key combination. Either attribute_name or attribute_key MUST be
+     * supplied, and both MAY be supplied.
+     *
+     * @param string  $attribute_name  The name of the attribute.
+     * @param string  $attribute_key   The key value of the attribute.
+     * @param string  $parent          The parent node to start searching from.
+     * @param boolean $allLevels       Return all levels, or just the direct
+     *                                 children of $parent? Defaults to all
+     *                                 levels.
+     *
+     * @return array  An array of all of the available values.
+     */
+    function getAttributeValues($attribute_name = null, $attribute_key = null,
+                                $parent = DATATREE_ROOT, $allLevels = true)
+    {
+        // Build the name/key filter.
+        $where = '';
+        if (!is_null($attribute_name)) {
+            $where .= 'a.attribute_name = ' . $this->_db->quote($attribute_name);
+        }
+        if (!is_null($attribute_key)) {
+            if ($where) {
+                $where .= ' AND ';
+            }
+            $where .= 'a.attribute_key = ' . $this->_db->quote($attribute_key);
+        }
+
+        // Return if we have no criteria.
+        if (!$where) {
+            return PEAR::raiseError('no criteria');
+        }
+
+        // Add filtering by parent, and for one or all levels.
+        $levelQuery = '';
+        if ($parent != DATATREE_ROOT) {
+            $parts = explode(':', $parent);
+            $parents = '';
+            $pstring = '';
+            foreach ($parts as $part) {
+                $pstring .= (empty($pstring) ? '' : ':') . $part;
+                $pid = $this->getId($pstring);
+                if (is_a($pid, 'PEAR_Error')) {
+                    return $pid;
+                }
+                $parents .= ':' . $pid;
+            }
+
+            if ($allLevels) {
+                $levelQuery = sprintf('AND (datatree_parents = %s OR datatree_parents LIKE %s)',
+                                      $this->_db->quote($parents),
+                                      $this->_db->quote($parents . ':%'));
+            } else {
+                $levelQuery = sprintf('AND datatree_parents = %s',
+                                      $this->_db->quote($parents));
+            }
+        } elseif (!$allLevels) {
+            $levelQuery = "AND datatree_parents = ''";
+        }
+
+        // Build the FROM/JOIN clauses.
+        $joins = 'LEFT JOIN ' . $this->_params['table'] .
+            ' c ON a.datatree_id = c.datatree_id';
+
+        $query = sprintf('SELECT DISTINCT a.attribute_value FROM %s a %s WHERE c.group_uid = %s AND %s %s',
+                         $this->_params['table_attributes'],
+                         $joins,
+                         $this->_db->quote($this->_params['group']),
+                         $where,
+                         $levelQuery);
+
+        Horde::logMessage('SQL Query by DataTree_sql::getAttributeValues(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
+
+        $rows = $this->_db->getCol($query);
+        if (is_a($rows, 'PEAR_Error')) {
+            Horde::logMessage($rows, __FILE__, __LINE__, PEAR_LOG_ERR);
+        }
+
+        return $rows;
+    }
+
     /**
      * Builds an attribute query. Here is an example $criteria array:
      *
@@ -1034,15 +1286,24 @@ class DataTree_sql extends DataTree {
      *                              children of $parent? Defaults to all levels.
      * @param string  $restrict     Only return attributes with the same
      *                              attribute_name or attribute_id.
-     * @param boolean $count        Just count the number of elements that'd match.
+     * @param integer $operation    Type of query to build
+     * @param string  $sortby_name  Attribute name to use for sorting.
+     * @param string  $sortby_key   Attribute key to use for sorting.
+     * @param integer $direction    Sort direction:
+     *                                0 - ascending
+     *                                1 - descending
      *
      * @return array  An SQL query and a list of values suitable for binding
      *                as an array.
      */
-    function buildAttributeQuery($criteria, $parent = DATATREE_ROOT, $allLevels = true, $restrict = 'name', $count = false)
+    function buildAttributeQuery($criteria, $parent = DATATREE_ROOT,
+                                 $allLevels = true, $restrict = 'name',
+                                 $operation = DATATREE_BUILD_SELECT,
+                                 $sortby_name = null, $sortby_key = null,
+                                 $direction = 0)
     {
         if (!count($criteria)) {
-            return 0;
+            return array('', array());
         }
 
         /* Build the query. */
@@ -1102,18 +1363,61 @@ class DataTree_sql extends DataTree {
                 }
             }
         }
+
+        // Override sorting.
+        $sort = array();
+        if (!is_null($sortby_name) || !is_null($sortby_key)) {
+            $order_table = 'a' . $i;
+            $joins[] = 'LEFT JOIN ' . $this->_params['table_attributes'] .
+                ' ' . $order_table . ' ON ' . $order_table .
+                '.datatree_id = c.datatree_id';
+
+            if (!is_null($sortby_name)) {
+                $pairs[] = sprintf('AND %s.attribute_name = ? ', $order_table);
+                $sort[] = $sortby_name;
+            }
+            if (!is_null($sortby_key)) {
+                $pairs[] = sprintf('AND %s.attribute_key = ? ', $order_table);
+                $sort[] = $sortby_key;
+            }
+
+            $order = sprintf('%s.attribute_value %s',
+                             $order_table,
+                             ($direction == 1) ? 'DESC' : 'ASC');
+            $group_by = 'c.datatree_id, c.datatree_name, c.datatree_order, ' .
+                $order_table . '.attribute_value';
+        } else {
+            $order = 'c.datatree_order, c.datatree_name, c.datatree_id';
+            $group_by = 'c.datatree_id, c.datatree_name, c.datatree_order';
+        }
+
         $joins = implode(' ', $joins);
         $pairs = implode(' ', $pairs);
+        $tail = sprintf('GROUP BY %s ORDER BY %s', $group_by, $order);
+
+        switch ($operation) {
+        case DATATREE_BUILD_COUNT:
+            $what = 'COUNT(DISTINCT c.datatree_id)';
+            $tail = '';
+            break;
+
+        default:
+            $what = 'c.datatree_id, c.datatree_name';
+            break;
+        }
 
         return array(sprintf('SELECT %s FROM %s c %s WHERE c.group_uid = ? AND %s %s %s %s',
-                             $count ? 'COUNT(DISTINCT c.datatree_id)' : 'c.datatree_id, c.datatree_name',
+                             $what,
                              $this->_params['table'],
                              $joins,
                              $query,
                              $levelQuery,
                              $pairs,
-                             $count ? '' : 'GROUP BY c.datatree_id, c.datatree_name, c.datatree_order ORDER BY c.datatree_order, c.datatree_name, c.datatree_id'),
-                     array_merge(array($this->_params['group']), $values, $levelValues));
+                             $tail),
+                     array_merge(array($this->_params['group']),
+                                 $values,
+                                 $levelValues,
+                                 $sort));
     }
 
     /**
@@ -1140,32 +1444,23 @@ class DataTree_sql extends DataTree {
         $alias = $this->_getAlias($join);
 
         foreach ($criteria as $key => $vals) {
+            if (!empty($clause)) {
+                $clause .= ' ' . $glue . ' ';
+            }
             if (!empty($vals['OR']) || !empty($vals['AND'])) {
-                if (!empty($clause)) {
-                    $clause .= ' ' . $glue . ' ';
-                }
                 $binds = $this->_buildAttributeQuery($glue, $vals);
                 $clause .= '(' . $binds[0] . ')';
                 $values = array_merge($values, $binds[1]);
             } elseif (!empty($vals['JOIN'])) {
-                if (!empty($clause)) {
-                    $clause .= ' ' . $glue . ' ';
-                }
                 $binds = $this->_buildAttributeQuery($glue, $vals['JOIN'], true);
                 $clause .= $binds[0];
                 $values = array_merge($values, $binds[1]);
             } else {
                 if (isset($vals['field'])) {
-                    if (!empty($clause)) {
-                        $clause .= ' ' . $glue . ' ';
-                    }
                     $binds = Horde_SQL::buildClause($this->_db, $alias . '.attribute_' . $vals['field'], $vals['op'], $vals['test'], true);
                     $clause .= $binds[0];
                     $values = array_merge($values, $binds[1]);
                 } else {
-                    if (!empty($clause)) {
-                        $clause .= ' ' . $glue . ' ';
-                    }
                     $binds = $this->_buildAttributeQuery($key, $vals);
                     $clause .= $binds[0];
                     $values = array_merge($values, $binds[1]);
@@ -1329,7 +1624,14 @@ class DataTree_sql extends DataTree {
             Horde::fatal($this->_db, __FILE__, __LINE__);
         }
 
-        $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+        // Set DB portability options
+        switch ($this->_db->phptype) {
+        case 'mssql':
+            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
+            break;
+        default:
+            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+        }
 
         return true;
     }
