@@ -2,9 +2,9 @@
 /**
  * Bar graph implementation for the Horde_Graph package.
  *
- * $Horde: framework/Graph/Graph/Plot/bar.php,v 1.5.12.2 2005/04/21 11:26:41 selsky Exp $
+ * $Horde: framework/Graph/Graph/Plot/bar.php,v 1.5.12.4 2006/01/01 21:28:18 jan Exp $
  *
- * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2006 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -33,7 +33,7 @@ class Horde_Graph_Plot_bar {
         }
     }
 
-    function draw()
+    function draw($minY = false)
     {
         $data = $this->graph->_data['y'][$this->_dataset];
 
@@ -44,12 +44,14 @@ class Horde_Graph_Plot_bar {
         $count = count($data);
         for ($i = 0; $i < $count; $i++) {
             $x = $i;
-            $y = $data[$i];
-            $this->graph->translate($x, $y);
-            $x = $x - ($barWidth / 2) + $barOffset;
-            $height = $this->graph->_graphBottom - $y;
+            if ($minY === false || $data[$i] >= $minY) {
+                $y = $data[$i];
+                $this->graph->translate($x, $y);
+                $x = $x - ($barWidth / 2) + $barOffset;
+                $height = $this->graph->_graphBottom - $y;
 
-            $this->graph->img->rectangle($x, $y, $barWidth, $height, $this->_outline, $this->_color);
+                $this->graph->img->rectangle($x, $y, $barWidth, $height, $this->_outline, $this->_color);
+            }
         }
     }
 
