@@ -17,9 +17,9 @@
  *               'imap/ssl/novalidate-cert' (for a self-signed certificate).
  *               DEFAULT: 'imap'</pre>
  *
- * $Horde: framework/IMAP/IMAP/ACL/rfc2086.php,v 1.6.8.6 2005/08/24 14:04:36 selsky Exp $
+ * $Horde: framework/IMAP/IMAP/ACL/rfc2086.php,v 1.6.8.12 2006/07/26 19:34:45 chuck Exp $
  *
- * Copyright 2003-2005 Chris Hastie <imp@oak-wood.co.uk>
+ * Copyright 2003-2006 Chris Hastie <imp@oak-wood.co.uk>
  *
  * See the enclosed file COPYING for license information (LGPL). If you did
  * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -97,18 +97,19 @@ class IMAP_ACL_rfc2086 extends IMAP_ACL {
 
     function _connect()
     {
-        $this->_imap = @imap_open(sprintf('{%s:%d/%s}', 
-                                          $this->_params['hostspec'],
-                                          $this->_params['port'],
-                                          $this->_params['protocol']),
-                                  $this->_params['username'],
-                                  $this->_params['password'], 
-                                  OP_HALFOPEN);
         if (!$this->_imap) {
-            $this->_imap = PEAR::raiseError(imap_last_error());
-            return false;
+            $this->_imap = @imap_open(sprintf('{%s:%d/%s}',
+                                              $this->_params['hostspec'],
+                                              $this->_params['port'],
+                                              $this->_params['protocol']),
+                                      $this->_params['username'],
+                                      $this->_params['password'],
+                                      OP_HALFOPEN);
+            if (!$this->_imap) {
+                $this->_imap = PEAR::raiseError(imap_last_error());
+            }
         }
-        return true;
+        return !is_a($this->_imap, 'PEAR_Error');
     }
 
     /**
@@ -135,7 +136,7 @@ class IMAP_ACL_rfc2086 extends IMAP_ACL {
             }
         }
 
-        /* Can't call this as @imap_setacl() as suppressing errors leads to 
+        /* Can't call this as @imap_setacl() as suppressing errors leads to
            imap_errors() returning nothing. */
         $result = imap_setacl($this->_imap, $folder, $share_user, $acl_str);
 
@@ -154,6 +155,12 @@ class IMAP_ACL_rfc2086 extends IMAP_ACL {
             return PEAR::raiseError(sprintf(_("Couldn't give user \"%s\" the following rights for the folder \"%s\": %s"), $share_user, $folder, $acl_str));
         }
 
+        /* If PHP 5 isn't available wait a bit to ensure ACL propagates in
+         * Cyrus Murder configuration. */
+        if (!function_exists('imap_getacl')) {
+            sleep(5);
+        }
+
         return $result;
     }
 
@@ -176,8 +183,6 @@ class IMAP_ACL_rfc2086 extends IMAP_ACL {
     /**
      * Attempts to get the result of a CAPABILITY command to the current IMAP
      * server.
-     * If protocol is imap/ssl, will only attempt ssl connection with
-     * PHP >= 4.3.
      *
      * @access private
      *
@@ -278,6 +283,38 @@ class IMAP_ACL_rfc2086 extends IMAP_ACL {
      */
     function _getACL($folder, $authMech)
     {
+        /* If imap_getacl() is available, use it */
+        if (function_exists('imap_getacl')) {
+            if (!$this->_connect()) {
+                return $this->_imap;
+            }
+
+            $result = @imap_getacl($this->_imap, $folder);
+            if (!$result) {
+                $errors = imap_errors();
+                if (is_array($errors)) {
+                    $error_string = '';
+                    foreach ($errors as $err) {
+                        if ($err == 'ACL not available on this IMAP server') {
+                            $err .= _("This IMAP server does not support sharing folders.");
+                        }
+                        $error_string .= $err;
+                    }
+                    return PEAR::raiseError($error_string);
+                }
+                return PEAR::raiseError(sprintf(_("Could not retrieve ACL")));
+            }
+
+            $returnACL = array();
+            foreach ($result as $user => $rights) {
+                for ($i = 0, $iMax = strlen($rights); $i < $iMax; $i++) {
+                    $returnACL[$user][$rights[$i]] = 1;
+                }
+            }
+
+            return $returnACL;
+        }
+
         global $notification;
 
         $have_sasl = false;
