@@ -2,9 +2,9 @@
 /**
  * IMAP_Sort provides functions for sorting lists of IMAP mailboxes/folders.
  *
- * $Horde: framework/IMAP/IMAP/Sort.php,v 1.6.8.9 2005/08/12 07:00:24 slusarz Exp $
+ * $Horde: framework/IMAP/IMAP/Sort.php,v 1.6.8.13 2006/02/15 18:07:52 slusarz Exp $
  *
- * Copyright 2004-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2004-2006 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -45,11 +45,16 @@ class IMAP_Sort {
      * @param array &$mbox    The list of mailboxes to sort.
      * @param boolean $inbox  When sorting, always put 'INBOX' at the head of
      *                        the list?
+     * @param boolean $index  Maintain index association?
      */
-    function sortMailboxes(&$mbox, $inbox = true)
+    function sortMailboxes(&$mbox, $inbox = true, $index = false)
     {
         $this->_sortinbox = $inbox;
-        usort($mbox, array(&$this, '_mbox_cmp'));
+        if ($index) {
+            uasort($mbox, array(&$this, '_mbox_cmp'));
+        } else {
+            usort($mbox, array(&$this, '_mbox_cmp'));
+        }
     }
 
     /**
