@@ -7,15 +7,14 @@ require_once dirname(__FILE__) . '/../Image.php';
  * extension. It mainly provides some utility functions, such as the
  * ability to make pixels, for now.
  *
- * $Horde: framework/Image/Image/gd.php,v 1.48 2004/08/30 13:58:58 chuck Exp $
+ * $Horde: framework/Image/Image/gd.php,v 1.48.10.14 2006/01/01 21:28:22 jan Exp $
  *
- * Copyright 2002-2004 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2006 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
- * @version $Revision: 1.48 $
  * @since   Horde 3.0
  * @package Horde_Image
  */
@@ -23,7 +22,8 @@ class Horde_Image_gd extends Horde_Image {
 
     /**
      * Capabilites of this driver.
-     * @var array $_capabilities
+     *
+     * @var array
      */
     var $_capabilities = array('resize',
                                'crop',
@@ -39,20 +39,23 @@ class Horde_Image_gd extends Horde_Image {
 
     /**
      * What kind of images should GD generate? Defaults to 'png'.
-     * @var string $_type
+     *
+     * @var string
      */
     var $_type = 'png';
 
     /**
      * GD Image resource for the current image data.
-     * @var resource $_im
+     *
+     * @var resource
      */
     var $_im;
 
     /**
-     * String identifier of the current image. New image data will not
-     * be loaded if the same id is already loaded.
-     * @var string $_id
+     * String identifier of the current image. New image data will not be
+     * loaded if the same id is already loaded.
+     *
+     * @var string
      */
     var $_id = '';
 
@@ -84,11 +87,15 @@ class Horde_Image_gd extends Horde_Image {
     }
 
     /**
-     * Return the raw data for this image.
+     * Returns the raw data for this image.
+     *
+     * @param boolean $convert  If true, the image data will be returned in the
+     *                          target format, independently from any image
+     *                          operations.
      *
      * @return string  The raw image data.
      */
-    function raw()
+    function raw($convert = false)
     {
         return Util::bufferOutput('image' . $this->_type, $this->_im);
     }
@@ -167,10 +174,8 @@ class Horde_Image_gd extends Horde_Image {
     /**
      * Load the image data from a string.
      *
-     * @access public
-     *
-     * @params string $id          An arbitrary id for the image.
-     * @params string $image_data  The data to use for the image.
+     * @param string $id          An arbitrary id for the image.
+     * @param string $image_data  The data to use for the image.
      */
     function loadString($id, $image_data)
     {
@@ -191,11 +196,9 @@ class Horde_Image_gd extends Horde_Image {
     /**
      * Load the image data from a file.
      *
-     * @access public
-     *
-     * @params string $filename  The full path and filename to the file to load
-     *                           the image data from. The filename will also be
-     *                           used for the image id.
+     * @param string $filename  The full path and filename to the file to load
+     *                          the image data from. The filename will also be
+     *                          used for the image id.
      *
      * @return mixed  PEAR Error if file does not exist or could not be loaded
      *                otherwise NULL if successful or already loaded.
@@ -515,8 +518,8 @@ class Horde_Image_gd extends Horde_Image {
         $shadow = $this->_call('imageColorClosest', array($this->_im, 0, 0, 0));
         if (is_a($shadow, 'PEAR_Error')) {
             return $shadow;
-        }   
-            
+        }
+
         // Shadow offset in pixels.
         $drop = 1;
 
@@ -549,7 +552,7 @@ class Horde_Image_gd extends Horde_Image {
         // Split $text into individual lines.
         $lines = explode("\n", $text);
 
-        switch ($valign){
+        switch ($valign) {
         case 'center':
             $y = ($this->_call('imageSY', array($this->_im)) - ($fontheight * count($lines))) / 2;
             break;
@@ -639,11 +642,11 @@ class Horde_Image_gd extends Horde_Image {
     /**
      * Draw a circle.
      *
-     * @param integer $x      The x co-ordinate of the centre.
-     * @param integer $y      The y co-ordinate of the centre.
-     * @param integer $r      The radius of the circle.
-     * @param string  $color  The line color of the circle.
-     * @param string  $fill   (optional) The color to fill the circle.
+     * @param integer $x     The x co-ordinate of the centre.
+     * @param integer $y     The y co-ordinate of the centre.
+     * @param integer $r     The radius of the circle.
+     * @param string $color  The line color of the circle.
+     * @param string $fill   The color to fill the circle.
      */
     function circle($x, $y, $r, $color, $fill = null)
     {
@@ -675,10 +678,10 @@ class Horde_Image_gd extends Horde_Image {
     /**
      * Draw a polygon based on a set of vertices.
      *
-     * @param array   $vertices  An array of x and y labeled arrays
-     *                           (eg. $vertices[0]['x'], $vertices[0]['y'], ...).
-     * @param string  $color     The color you want to draw the polygon with.
-     * @param string  $fill      (optional) The color to fill the polygon.
+     * @param array $vertices  An array of x and y labeled arrays
+     *                         (eg. $vertices[0]['x'], $vertices[0]['y'], ...).
+     * @param string $color    The color you want to draw the polygon with.
+     * @param string $fill     The color to fill the polygon.
      */
     function polygon($verts, $color, $fill = 'none')
     {
@@ -716,8 +719,8 @@ class Horde_Image_gd extends Horde_Image {
      * @param integer $y       The top y-coordinate of the rectangle.
      * @param integer $width   The width of the rectangle.
      * @param integer $height  The height of the rectangle.
-     * @param string  $color   (optional) The line color of the rectangle. Defaults to black.
-     * @param string  $fill    (optional) The color to fill the rectangle with. Defaults to none.
+     * @param string $color    The line color of the rectangle.
+     * @param string $fill     The color to fill the rectangle with.
      */
     function rectangle($x, $y, $width, $height, $color = 'black', $fill = 'none')
     {
@@ -750,8 +753,8 @@ class Horde_Image_gd extends Horde_Image {
      * @param integer $width   The width of the rectangle.
      * @param integer $height  The height of the rectangle.
      * @param integer $round   The width of the corner rounding.
-     * @param string  $color   (optional) The line color of the rectangle. Defaults to black.
-     * @param string  $fill    (optional) The color to fill the rounded rectangle with. Defaults to none.
+     * @param string $color    The line color of the rectangle.
+     * @param string $fill     The color to fill the rounded rectangle with.
      */
     function roundedRectangle($x, $y, $width, $height, $round, $color = 'black', $fill = 'none')
     {
@@ -839,12 +842,12 @@ class Horde_Image_gd extends Horde_Image {
     /**
      * Draw a line.
      *
-     * @param integer $x0     The x co-ordinate of the start.
-     * @param integer $y0     The y co-ordinate of the start.
-     * @param integer $x1     The x co-ordinate of the end.
-     * @param integer $y1     The y co-ordinate of the end.
-     * @param string  $color  (optional) The line color.
-     * @param string  $width  (optional) The width of the line.
+     * @param integer $x0    The x co-ordinate of the start.
+     * @param integer $y0    The y co-ordinate of the start.
+     * @param integer $x1    The x co-ordinate of the end.
+     * @param integer $y1    The y co-ordinate of the end.
+     * @param string $color  The line color.
+     * @param string $width  The width of the line.
      */
     function line($x1, $y1, $x2, $y2, $color = 'black', $width = 1)
     {
@@ -891,12 +894,12 @@ class Horde_Image_gd extends Horde_Image {
      * @param integer $y0           The y co-ordinate of the start.
      * @param integer $x1           The x co-ordinate of the end.
      * @param integer $y1           The y co-ordinate of the end.
-     * @param string  $color        (optional) The line color.
-     * @param string  $width        (optional) The width of the line.
+     * @param string $color         The line color.
+     * @param string $width         The width of the line.
      * @param integer $dash_length  The length of a dash on the dashed line
      * @param integer $dash_space   The length of a space in the dashed line
      */
-    function dashedLine($x1, $y1, $x2, $y2, $color = 'black', $width = 1, $dash_length = 2, $dash_space = 2)
+    function dashedLine($x0, $y0, $x1, $y1, $color = 'black', $width = 1, $dash_length = 2, $dash_space = 2)
     {
         $c = $this->allocateColor($color);
         if (is_a($c, 'PEAR_Error')) {
@@ -922,17 +925,17 @@ class Horde_Image_gd extends Horde_Image {
         if (is_a($result = $this->_call('imageSetThickness', array($this->_im, $width)), 'PEAR_Error')) {
             return $result;
         }
-        return $this->_call('imageLine', array($this->_im, $x1, $y1, $x2, $y2, IMG_COLOR_STYLED));
+        return $this->_call('imageLine', array($this->_im, $x0, $y0, $x1, $y1, IMG_COLOR_STYLED));
     }
 
     /**
      * Draw a polyline (a non-closed, non-filled polygon) based on a
      * set of vertices.
      *
-     * @param array   $vertices  An array of x and y labeled arrays
-     *                           (eg. $vertices[0]['x'], $vertices[0]['y'], ...).
-     * @param string  $color     The color you want to draw the line with.
-     * @param string  $width     (optional) The width of the line.
+     * @param array $vertices  An array of x and y labeled arrays
+     *                         (eg. $vertices[0]['x'], $vertices[0]['y'], ...).
+     * @param string $color    The color you want to draw the line with.
+     * @param string $width    The width of the line.
      */
     function polyline($verts, $color, $width = 1)
     {
@@ -974,7 +977,7 @@ class Horde_Image_gd extends Horde_Image {
                 $f = $this->allocateColor($fill);
                 if (is_a($f, 'PEAR_Error')) {
                     return $f;
-                }   
+                }
                 if (is_a($result = $this->_call('imageFilledArc', array($this->_im, $x, $y, $r * 2, $r * 2, $start, $end, $f, IMG_ARC_PIE)), 'PEAR_Error')) {
                     return $result;
                 }
@@ -990,8 +993,8 @@ class Horde_Image_gd extends Horde_Image {
      * Creates an image of the given size.
      * If possible the function returns a true color image.
      *
-     * @param int $width   The image width.
-     * @param int $height  The image height.
+     * @param integer $width   The image width.
+     * @param integer $height  The image height.
      *
      * @return resource|object PEAR Error  The image handler or a PEAR_Error on
      *                                     error.
@@ -1026,7 +1029,7 @@ class Horde_Image_gd extends Horde_Image {
         }
         if (!empty($php_errormsg)) {
             require_once 'PEAR.php';
-            return PEAR::raiseError($function . ': ' . $php_errormsg);
+            $result = PEAR::raiseError($function . ': ' . $php_errormsg);
         }
         return $result;
     }
