@@ -47,15 +47,14 @@ define('MIME_STRING', 2);
 /**
  * The MIME:: class provides methods for dealing with MIME standards.
  *
- * $Horde: framework/MIME/MIME.php,v 1.139.4.2 2005/01/04 03:01:40 slusarz Exp $
+ * $Horde: framework/MIME/MIME.php,v 1.139.4.20 2006/03/24 04:29:52 chuck Exp $
  *
- * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2006 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
- * @version $Revision: 1.139.4.2 $
  * @since   Horde 1.3
  * @package Horde_MIME
  */
@@ -64,7 +63,7 @@ class MIME {
     /**
      * A listing of the allowed MIME types.
      *
-     * @var array $mime_types
+     * @var array
      */
     var $mime_types = array(
         TYPETEXT => 'text',
@@ -81,7 +80,7 @@ class MIME {
     /**
      * A listing of the allowed MIME encodings.
      *
-     * @var array $mime_encodings
+     * @var array
      */
     var $mime_encodings = array(
         ENC7BIT => '7bit',
@@ -95,15 +94,13 @@ class MIME {
     /**
      * Filter for RFC822.
      *
-     * @var array $rfc822_filter
+     * @var string
      */
     var $rfc822_filter = "()<>@,;:\\\"[]\1\2\3\4\5\6\7\10\11\12\13\14\15\16\17\20\21\22\23\24\25\26\27\30\31\32\33\34\35\36\37\177";
 
     /**
      * Determines if a string contains 8-bit characters.
      *
-     * @access public
-     *
      * @param string $string  The string to check.
      *
      * @return boolean  True if it does, false if it doesn't.
@@ -116,8 +113,6 @@ class MIME {
     /**
      * Encodes a string containing non-ASCII characters according to RFC 2047.
      *
-     * @access public
-     *
      * @param string $text     The text to encode.
      * @param string $charset  The character set of the text.
      *
@@ -187,8 +182,6 @@ class MIME {
      * Encodes a line via quoted-printable encoding.
      * Wraps lines at 76 characters.
      *
-     * @access public
-     *
      * @param string $text  The text to encode.
      * @param string $eol   The EOL sequence to use.
      *
@@ -198,41 +191,35 @@ class MIME {
     {
         $output = '';
 
-        if (Util::extensionExists('imap')) {
-            foreach (preg_split("/\r?\n/", $text) as $line) {
-                $output .= imap_8bit($line) . $eol;
-            }
-        } else {
-            foreach (preg_split("/\r?\n/", $text) as $line) {
-                /* We need to go character by character through the line. */
-                $length = strlen($line);
-                $current_line = '';
-                $current_length = 0;
+        foreach (preg_split("/\r?\n/", $text) as $line) {
+            /* We need to go character by character through the line. */
+            $length = strlen($line);
+            $current_line = '';
+            $current_length = 0;
 
-                for ($i = 0; $i < $length; $i++) {
-                    $char = substr($line, $i, 1);
-                    $ascii = ord($char);
+            for ($i = 0; $i < $length; $i++) {
+                $char = substr($line, $i, 1);
+                $ascii = ord($char);
 
-                    /* Spaces or tabs at the end of the line are NOT allowed.
-                     * Also, Characters in ASCII below 32 or above 126 AND 61
-                     * must be encoded. */
-                    if (((($ascii === 9) || ($ascii === 32)) && ($i == ($length - 1))) ||
-                        (($ascii < 32) || ($ascii > 126) || ($ascii === 61))) {
-                        $char = '=' . String::upper(sprintf('%02s', dechex($ascii)));
-                    }
-
-                    /* Lines must be 76 characters or less. */
-                    $char_length = strlen($char);
-                    $current_length += $char_length;
-                    if ($current_length > 76) {
-                        $output .= $current_line . '=' . $eol;
-                        $current_line = '';
-                        $current_length = $char_length;
-                    }
-                    $current_line .= $char;
+                /* Spaces or tabs at the end of the line are NOT allowed.
+                 * Also, Characters in ASCII below 32 or above 126 AND 61
+                 * must be encoded. */
+                if (((($ascii === 9) || ($ascii === 32)) && ($i == ($length - 1))) ||
+                    (($ascii < 32) || ($ascii > 126) || ($ascii === 61))) {
+                    $char = '=' . String::upper(sprintf('%02s', dechex($ascii)));
                 }
-                $output .= $current_line . $eol;
+
+                /* Lines must be 76 characters or less. */
+                $char_length = strlen($char);
+                $current_length += $char_length;
+                if ($current_length > 76) {
+                    $output .= $current_line . '=' . $eol;
+                    $current_line = '';
+                    $current_length = $char_length;
+                }
+                $current_line .= $char;
             }
+            $output .= $current_line . $eol;
         }
 
         return $output;
@@ -244,30 +231,41 @@ class MIME {
      * This differs from MIME::encode() because it keeps email addresses legal,
      * only encoding the personal information.
      *
-     * @access public
-     *
-     * @param string $text       The email addresses to encode.
+     * @param string $addresses  The email addresses to encode.
      * @param string $charset    The character set of the text.
      * @param string $defserver  The default domain to append to mailboxes.
      *
      * @return string  The text, encoded only if it contains non-ascii
      *                 characters.
      */
-    function encodeAddress($text, $charset = null, $defserver = null)
+    function encodeAddress($addresses, $charset = null, $defserver = null)
     {
-        /* parseAddressList() does not process the null entry
-         * 'undisclosed-recipients:;' correctly. */
-        if (preg_match('/undisclosed-recipients:\s*;/i', trim($text))) {
-            return $text;
+        if (is_array($addresses)) {
+            $addr_arr = $addresses;
+        } else {
+            /* parseAddressList() does not process the null entry
+             * 'undisclosed-recipients:;' correctly. */
+            if (preg_match('/undisclosed-recipients:\s*;/i', trim($addresses))) {
+                return $addresses;
+            }
+
+            require_once 'Mail/RFC822.php';
+            $parser = &new Mail_RFC822();
+            $addr_arr = $parser->parseAddressList($addresses, $defserver, true, false);
+            if (is_a($addr_arr, 'PEAR_Error')) {
+                return $addr_arr;
+            }
         }
 
-        require_once 'Mail/RFC822.php';
-        $parser = &new Mail_RFC822();
-        $addr_arr = $parser->parseAddressList($text, $defserver, false, false);
         $text = '';
-
         if (is_array($addr_arr)) {
             foreach ($addr_arr as $addr) {
+                // Check for groups.
+                if (!empty($addr->groupname)) {
+                    $text .= MIME::encode($addr->groupname, $charset) . ': ' . MIME::encodeAddress($addr->addresses) . ';';
+                    continue;
+                }
+
                 if (empty($addr->personal)) {
                     $personal = '';
                 } else {
@@ -290,14 +288,11 @@ class MIME {
     /**
      * Decodes an RFC 2047-encoded string.
      *
-     * @access public
-     *
      * @param string $string      The text to decode.
      * @param string $to_charset  The charset that the text should be decoded
      *                            to.
      *
-     * @return string  The decoded text, or the original string if it was not
-     *                 encoded.
+     * @return string  The decoded text.
      */
     function decode($string, $to_charset = null)
     {
@@ -343,15 +338,14 @@ class MIME {
         switch ($encoding) {
         case 'Q':
         case 'q':
-            $encoded_text = str_replace('_', '%20', $encoded_text);
-            $encoded_text = str_replace('=', '%', $encoded_text);
-            $decoded = urldecode($encoded_text);
+            $encoded_text = str_replace('_', ' ', $encoded_text);
+            $decoded = preg_replace('/=([0-9a-f]{2})/ie', 'chr(0x\1)', $encoded_text);
             $decoded = String::convertCharset($decoded, $charset, $to_charset);
             break;
 
         case 'B':
         case 'b':
-            $decoded = urldecode(base64_decode($encoded_text));
+            $decoded = base64_decode($encoded_text);
             $decoded = String::convertCharset($decoded, $charset, $to_charset);
             break;
 
@@ -366,14 +360,12 @@ class MIME {
     /**
      * Decodes an RFC 2231-encoded string.
      *
-     * @access public
-     *
      * @param string $string   The entire string to decode, including the
      *                         parameter name.
      * @param string $charset  The charset that the text should be decoded to.
      *
      * @return array  The decoded text, or the original string if it was not
-     *                 encoded.
+     *                encoded.
      */
     function decodeRFC2231($string, $to_charset = null)
     {
@@ -389,7 +381,7 @@ class MIME {
         $charset = $lang = null;
         $output = '';
 
-        /* Get the characer set and language used in the encoding, if
+        /* Get the character set and language used in the encoding, if
          * any. */
         if (preg_match("/^[^=]+\*\=([^']*)'([^']*)'/", $string, $matches)) {
             $charset = $matches[1];
@@ -426,8 +418,6 @@ class MIME {
      * If an email address has no personal information, get rid of any angle
      * brackets (<>) around it.
      *
-     * @access public
-     *
      * @param string $address  The address to trim.
      *
      * @return string  The trimmed address.
@@ -446,8 +436,6 @@ class MIME {
     /**
      * Builds an RFC 822 compliant email address.
      *
-     * @access public
-     *
      * @param string $mailbox   Mailbox name.
      * @param string $host      Domain name of mailbox's host.
      * @param string $personal  Personal name phrase.
@@ -483,8 +471,6 @@ class MIME {
      * Explodes a RFC 822 string, ignoring a delimiter if preceded by a "\"
      * character, or if delimiter is inside single or double quotes.
      *
-     * @access public
-     *
      * @param string $str        The RFC 822 string.
      * @param string $delimiter  The delimter.
      *
@@ -496,15 +482,18 @@ class MIME {
         $match = 0;
         $quotes = array('"', "'");
         $in_quote = null;
+        $in_group = false;
         $prev = null;
 
         if (empty($str)) {
             return array($str);
         }
 
-        if (in_array($str[0], $quotes)) {
-            $in_quote = $str[0];
-        } elseif ($str[0] == $delimiter) {
+        if (in_array($str{0}, $quotes)) {
+            $in_quote = $str{0};
+        } elseif ($str{0} == ':') {
+            $in_group = true;
+        } elseif ($str{0} == $delimiter) {
             $arr[] = '';
             $match = 1;
         }
@@ -519,8 +508,16 @@ class MIME {
                         $in_quote = $char;
                     }
                 }
-            } elseif (($char == $delimiter) &&
-                      ($prev !== '\\') &&
+            } elseif ($in_group) {
+                if ($char == ';') {
+                    $arr[] = substr($str, $match, $i - $match + 1);
+                    $match = $i + 1;
+                    $in_group = false;
+                }
+            } elseif ($char == ':') {
+                $in_group = true;
+            } elseif ($char == $delimiter &&
+                      $prev !== '\\' &&
                       is_null($in_quote)) {
                 $arr[] = substr($str, $match, $i - $match);
                 $match = $i + 1;
@@ -547,8 +544,6 @@ class MIME {
      *   $object->host     = The host the mailbox is on ("example.com")
      * </pre>
      *
-     * @access public
-     *
      * @param stdClass $ob   The address object to be turned into a string.
      * @param mixed $filter  A user@example.com style bare address to ignore.
      *                       Either single string or an array of strings.  If
@@ -601,8 +596,6 @@ class MIME {
      * Takes an array of address objects, as returned by imap_headerinfo(),
      * for example, and passes each of them through MIME::addrObject2String().
      *
-     * @access public
-     *
      * @param array $addresses  The array of address objects.
      * @param mixed $filter     A user@example.com style bare address to
      *                          ignore.  If any address matches $filter, it
@@ -623,7 +616,7 @@ class MIME {
             $val = MIME::addrObject2String($addr, $filter);
             if (!empty($val)) {
                 $bareAddr = String::lower(MIME::bareAddress($val));
-                if (!array_key_exists($bareAddr, $addrList)) {
+                if (!isset($addrList[$bareAddr])) {
                     $addrList[$bareAddr] = $val;
                 }
             }
@@ -639,8 +632,6 @@ class MIME {
     /**
      * Returns the bare address.
      *
-     * @access public
-     *
      * @param string $address    The address string.
      * @param string $defserver  The default domain to append to mailboxes.
      * @param boolean $multiple  Should we return multiple results?
@@ -653,15 +644,24 @@ class MIME {
     {
         $addressList = array();
 
-        /* Don't use built-in IMAP function (imap_rfc822_parse_adrlist)
-         * because it doesn't parse distribution lists properly. */
-        require_once 'Mail/RFC822.php';
-        $parser = &new Mail_RFC822();
-        $from = $parser->parseAddressList($address, $defserver, false, false);
+        /* Use built-in IMAP function only if available and if not parsing
+         * distribution lists because it doesn't parse distribution lists
+         * properly. */
+        if (Util::extensionExists('imap') && strpos($address, ':') === false) {
+            $from = imap_rfc822_parse_adrlist($address, $defserver);
+        } else {
+            require_once 'Mail/RFC822.php';
+            $parser = new Mail_RFC822();
+            $from = $parser->parseAddressList($address, $defserver, false, false);
+            if (is_a($from, 'PEAR_Error')) {
+                return $multiple ? array() : '';
+            }
+        }
 
         foreach ($from as $entry) {
             if (isset($entry->mailbox) &&
-                ($entry->mailbox != 'undisclosed-recipients')) {
+                $entry->mailbox != 'undisclosed-recipients' &&
+                $entry->mailbox != 'UNEXPECTED_DATA_AFTER_ADDRESS') {
                 if (isset($entry->host)) {
                     $addressList[] = $entry->mailbox . '@' . $entry->host;
                 } else {
@@ -670,11 +670,7 @@ class MIME {
             }
         }
 
-        if ($multiple) {
-            return $addressList;
-        } else {
-            return array_pop($addressList);
-        }
+        return $multiple ? $addressList : array_pop($addressList);
     }
 
     /**
@@ -695,7 +691,13 @@ class MIME {
             $filter = $vars['rfc822_filter'] . ' ';
         }
 
+        // Strip double quotes if they are around the string already.
+        if (substr($str, 0, 1) == '"' && substr($str, -1) == '"') {
+            $str = substr($str, 1, -1);
+        }
+
         if (strcspn($str, $filter) != strlen($str)) {
+            $str = str_replace('\"', '"', $str);
             return '"' . str_replace('"', '\\"', str_replace('\\', '\\\\', $str)) . '"';
         } else {
             return $str;
@@ -705,8 +707,6 @@ class MIME {
     /**
      * Returns the MIME type for the given input.
      *
-     * @access public
-     *
      * @param mixed $input     Either the MIME code or type string.
      * @param integer $format  If MIME_CODE, return code.
      *                         If MIME_STRING, returns lowercase string.
@@ -721,8 +721,6 @@ class MIME {
     /**
      * Returns the MIME encoding for the given input.
      *
-     * @access public
-     *
      * @param mixed $input     Either the MIME code or encoding string.
      * @param integer $format  If MIME_CODE, return code.
      *                         If MIME_STRING, returns lowercase string.
@@ -751,6 +749,9 @@ class MIME {
     function _getCode($input, $format, $type)
     {
         $numeric = is_numeric($input);
+        if (!$numeric) {
+            $input = String::lower($input);
+        }
 
         switch ($format) {
         case MIME_CODE:
@@ -758,7 +759,7 @@ class MIME {
             break;
 
         case MIME_STRING:
-            if (!$numeric) return String::lower($input);
+            if (!$numeric) return $input;
             break;
         }
 
@@ -766,7 +767,7 @@ class MIME {
 
         if ($numeric) {
             if (isset($vars[$type][$input])) {
-                return String::lower($vars[$type][$input]);
+                return $vars[$type][$input];
             }
         } else {
             if (($search = array_search($input, $vars[$type]))) {
@@ -781,8 +782,6 @@ class MIME {
      * Generates a Message-ID string conforming to RFC 2822 [3.6.4] and the
      * standards outlined in 'draft-ietf-usefor-message-id-01.txt'.
      *
-     * @access public
-     *
      * @param string  A message ID string.
      */
     function generateMessageID()
@@ -805,8 +804,6 @@ class MIME {
      * compromise is to put each parameter on a separate line and chop it
      * only if it is longer than 998 characters.
      *
-     * @access public
-     *
      * @param string $header  The header name.
      * @param string $text    The text of the header field.
      * @param string $eol     The EOL string to use.
