@@ -1,20 +1,25 @@
 <?php
-/** @constant HORDE_AGENT_HEADER The description of Horde to use in the 'User-Agent:' header. */
-define('HORDE_AGENT_HEADER', 'Horde Application Framework 3.0');
+/**
+ * The description of Horde to use in the 'User-Agent:' header.
+ */
+define('HORDE_AGENT_HEADER', 'Horde Application Framework 3.1');
 
 /**
  * The MIME_Headers:: class contains generic functions related to
  * handling the headers of mail messages.
  *
- * $Horde: framework/MIME/MIME/Headers.php,v 1.29 2004/09/24 03:39:55 slusarz Exp $
+ * The default character set to use for headers should be defined in the
+ * variable $GLOBALS['mime_headers']['default_charset'] (defaults to US-ASCII
+ * per RFC 2045).
  *
- * Copyright 2002-2004 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * $Horde: framework/MIME/MIME/Headers.php,v 1.29.10.20 2006/04/15 17:24:19 slusarz Exp $
+ *
+ * Copyright 2002-2006 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @version $Revision: 1.29 $
  * @since   Horde 3.0
  * @package Horde_MIME
  */
@@ -23,35 +28,35 @@ class MIME_Headers {
     /**
      * The internal headers array.
      *
-     * @var array $_headers
+     * @var array
      */
     var $_headers = array();
 
     /**
      * Cached output of the MIME_Structure::parseMIMEHeaders() command.
      *
-     * @var array $_allHeaders
+     * @var array
      */
     var $_allHeaders;
 
     /**
      * Cached output of the imap_fetchheader() command.
      *
-     * @var string $_headerText
+     * @var string
      */
     var $_headerText;
 
     /**
      * The header object returned from imap_headerinfo().
      *
-     * @var stdClass $_headerObject
+     * @var stdClass
      */
     var $_headerObject;
 
     /**
      * The internal flags array.
      *
-     * @var array $_flags
+     * @var array
      */
     var $_flags = array();
 
@@ -59,7 +64,7 @@ class MIME_Headers {
      * The User-Agent string to use.
      * THIS VALUE SHOULD BE OVERRIDEN BY ALL SUBCLASSES.
      *
-     * @var string $_agent
+     * @var string
      */
     var $_agent = HORDE_AGENT_HEADER;
 
@@ -70,23 +75,21 @@ class MIME_Headers {
      * To be RFC complaint, the full <CR><LF> EOL combination should be used
      * when sending a message.
      *
-     * @var string $_eol
+     * @var string
      */
     var $_eol = "\n";
 
     /**
      * The index of the message.
      *
-     * @var integer $_index
+     * @var integer
      */
     var $_index;
 
     /**
      * Constructor.
      *
-     * @access public
-     *
-     * @param optional integer $index  The message index to parse headers.
+     * @param integer $index  The message index to parse headers.
      */
     function MIME_Headers($index = null)
     {
@@ -107,14 +110,15 @@ class MIME_Headers {
     /**
      * Return the full list of headers from the imap_fetchheader() function.
      *
-     * @access public
-     *
      * @return string  See imap_fetchheader().
      */
     function getHeaderText()
     {
         if (!is_null($this->_index) && empty($this->_headerText)) {
             $this->_headerText = @imap_fetchheader($this->_getStream(), $this->_index, FT_UID);
+            if (!empty($GLOBALS['mime_headers']['default_charset'])) {
+                $this->_headerText = String::convertCharset($this->_headerText, $GLOBALS['mime_headers']['default_charset']);
+            }
         }
 
         return $this->_headerText;
@@ -123,9 +127,7 @@ class MIME_Headers {
     /**
      * Return the full list of headers.
      *
-     * @access public
-     *
-     * @param optional boolean $decode  Decode the headers?
+     * @param boolean $decode  Decode the headers?
      *
      * @return array  See MIME_Structure::parseMIMEHeaders().
      */
@@ -143,14 +145,12 @@ class MIME_Headers {
     /**
      * Return the header object from imap_headerinfo().
      *
-     * @access public
-     *
-     * @return object stdClass  See imap_headerinfo().
+     * @return stdClass  See imap_headerinfo().
      */
     function getHeaderObject()
     {
         if (!is_null($this->_index) && empty($this->_headerObject)) {
-            $stream = &$this->_getStream();
+            $stream = $this->_getStream();
             $this->_headerObject = @imap_headerinfo($stream, @imap_msgno($stream, $this->_index));
         }
 
@@ -158,26 +158,24 @@ class MIME_Headers {
     }
 
     /**
-     * Build the header array. The headers are MIME decoded.
+     * Build the header array.
      *
-     * @access public
+     * @param boolean $decode  MIME decode the headers?
      */
-    function buildHeaders()
+    function buildHeaders($decode = true)
     {
         if (!empty($this->_headers)) {
             return;
         }
 
         /* Parse through the list of all headers. */
-        foreach ($this->getAllHeaders() as $key => $val) {
+        foreach ($this->getAllHeaders($decode) as $key => $val) {
             $this->addHeader($key, $val);
         }
     }
 
     /**
      * Build the flags array.
-     *
-     * @access public
      */
     function buildFlags()
     {
@@ -187,7 +185,7 @@ class MIME_Headers {
 
         /* Get the IMAP header object. */
         $ob = $this->getHeaderObject();
-        if (!isset($ob)) {
+        if (!is_object($ob)) {
             return;
         }
 
@@ -225,8 +223,6 @@ class MIME_Headers {
     /**
      * Returns the internal header array in array format.
      *
-     * @access public
-     *
      * @return array  The headers in array format.
      */
     function toArray()
@@ -254,8 +250,6 @@ class MIME_Headers {
     /**
      * Returns the internal header array in string format.
      *
-     * @access public
-     *
      * @return string  The headers in string format.
      */
     function toString()
@@ -278,8 +272,6 @@ class MIME_Headers {
     /**
      * Generate the 'Received' header for the Web browser->Horde hop
      * (attempts to conform to guidelines in RFC 2821).
-     *
-     * @access public
      */
     function addReceivedHeader()
     {
@@ -287,11 +279,11 @@ class MIME_Headers {
             /* This indicates the user is connecting through a proxy. */
             $remote_path = explode(',', $_SERVER['HTTP_X_FORWARDED_FOR']);
             $remote_addr = $remote_path[0];
-            $remote = gethostbyaddr($remote_addr);
+            $remote = @gethostbyaddr($remote_addr);
         } else {
             $remote_addr = $_SERVER['REMOTE_ADDR'];
             if (empty($_SERVER['REMOTE_HOST'])) {
-                $remote = gethostbyaddr($remote_addr);
+                $remote = @gethostbyaddr($remote_addr);
             } else {
                 $remote = $_SERVER['REMOTE_HOST'];
             }
@@ -304,21 +296,25 @@ class MIME_Headers {
             $received .= $remote . ' ';
         }
         $received .= '[' . $remote_addr . ']) ';
-        $received .= 'by ' . $GLOBALS['conf']['server']['name'] . ' (Horde) with HTTP ';
 
-        $user = Auth::getAuth();
-        if (strpos($user, '@') === false) {
-            $user .= '@' . $GLOBALS['conf']['server']['name'];
+        if (!empty($GLOBALS['conf']['server']['name'])) {
+            $server_name = $GLOBALS['conf']['server']['name'];
+        } elseif (!empty($_SERVER['SERVER_NAME'])) {
+            $server_name = $_SERVER['SERVER_NAME'];
+        } elseif (!empty($_SERVER['HTTP_HOST'])) {
+            $server_name = $_SERVER['HTTP_HOST'];
+        } else {
+            $server_name = 'unknown';
         }
-        $received .= 'for <' . $user . '>; ' . date('r');
+        $received .= 'by ' . $server_name . ' (Horde MIME library) with HTTP; ';
+
+        $received .= date('r');
 
         $this->addHeader('Received', $received);
     }
 
     /**
      * Generate the 'Message-ID' header.
-     *
-     * @access public
      */
     function addMessageIdHeader()
     {
@@ -330,8 +326,6 @@ class MIME_Headers {
      * Generate the 'Resent' headers (conforms to guidelines in
      * RFC 2822 [3.6.6]).
      *
-     * @access public
-     *
      * @param string $from  The address to use for 'Resent-From'.
      * @param string $to    The address to use for 'Resent-To'.
      */
@@ -349,8 +343,6 @@ class MIME_Headers {
     /**
      * Generate delivery receipt headers.
      *
-     * @access public
-     *
      * @param string $to  The address the receipt should be mailed to.
      */
     function addDeliveryReceiptHeaders($to)
@@ -361,8 +353,6 @@ class MIME_Headers {
 
     /**
      * Generate the user agent description header.
-     *
-     * @access public
      */
     function addAgentHeader()
     {
@@ -372,8 +362,6 @@ class MIME_Headers {
     /**
      * Returns the user agent description header.
      *
-     * @access public
-     *
      * @return string  The user agent header.
      */
     function getAgentHeader()
@@ -384,8 +372,6 @@ class MIME_Headers {
     /**
      * Add a header to the header array.
      *
-     * @access public
-     *
      * @param string $header  The header name.
      * @param string $value   The header value.
      */
@@ -405,8 +391,6 @@ class MIME_Headers {
     /**
      * Remove a header from the header array.
      *
-     * @access public
-     *
      * @param string $header  The header name.
      */
     function removeHeader($header)
@@ -419,8 +403,6 @@ class MIME_Headers {
     /**
      * Set a value for a particular header ONLY if that header is set.
      *
-     * @access public
-     *
      * @param string $header  The header name.
      * @param string $value   The header value.
      *
@@ -441,8 +423,6 @@ class MIME_Headers {
     /**
      * Attempts to return the header in the correct case.
      *
-     * @access public
-     *
      * @param string $header  The header to search for.
      *
      * @return string  The value for the given header.
@@ -457,11 +437,10 @@ class MIME_Headers {
     /**
      * Attempt to return the value for a given header.
      * The following header fields can only have 1 entry, so if duplicate
-     * entries exist, the first value will be used (RFC 2822 [3.6]):
-     *   To, From, Cc, Bcc, Date, Sender, Reply-to, Message-ID, In-Reply-To,
-     *   References, Subject
-     *
-     * @access public
+     * entries exist, the first value will be used:
+     *   * To, From, Cc, Bcc, Date, Sender, Reply-to, Message-ID, In-Reply-To,
+     *     References, Subject (RFC 2822 [3.6])
+     *   * All List Headers (RFC 2369 [3])
      *
      * @param string $header  The header to search for.
      *
@@ -473,9 +452,10 @@ class MIME_Headers {
         $header = String::lower($header);
 
         if (isset($this->_headers[$header])) {
-            $single = array('to', 'from', 'cc', 'bcc', 'date', 'sender', 
+            $single = array('to', 'from', 'cc', 'bcc', 'date', 'sender',
                             'reply-to', 'message-id', 'in-reply-to',
                             'references', 'subject', 'x-priority');
+            $single = array_merge($single, array_keys($this->listHeaders()));
             if (is_array($this->_headers[$header]['value']) &&
                 in_array($header, $single)) {
                 return $this->_headers[$header]['value'][0];
@@ -490,8 +470,6 @@ class MIME_Headers {
     /**
      * Has the header been altered from the original?
      *
-     * @access public
-     *
      * @param string $header  The header to analyze.
      *
      * @return boolean  True if the header has been altered.
@@ -505,8 +483,6 @@ class MIME_Headers {
     /**
      * Transforms a Header value using the list of functions provided.
      *
-     * @access public
-     *
      * @param string $header  The header to alter.
      * @param mixed $funcs    A function, or an array of functions.
      *                        The functions will be performed from right to
@@ -537,9 +513,7 @@ class MIME_Headers {
     /**
      * Add any MIME headers required for the MIME_Part.
      *
-     * @access public
-     *
-     * @param object MIME_Part &$mime_part  The MIME_Part object.
+     * @param MIME_Part &$mime_part  The MIME_Part object.
      */
     function addMIMEHeaders(&$mime_part)
     {
@@ -551,8 +525,6 @@ class MIME_Headers {
     /**
      * Return the list of addresses for a header object.
      *
-     * @access public
-     *
      * @param array $obs  An array of header objects (See imap_headerinfo()
      *                    for the object structure).
      *
@@ -575,7 +547,15 @@ class MIME_Headers {
 
         foreach ($obs as $ob) {
             /* Ensure we're working with initialized values. */
-            $ob->personal = isset($ob->personal) ? trim(MIME::decode($ob->personal), '"') : '';
+            if (isset($ob->personal)) {
+                $ob->personal = MIME::decode($ob->personal);
+                if ((substr($ob->personal, 0, 1) == '"') &&
+                    (substr($ob->personal, -1) == '"')) {
+                    $ob->personal = substr($ob->personal, 1, -1);
+                }
+            } else {
+                $ob->personal = '';
+            }
 
             if (isset($ob->mailbox)) {
                 /* Don't process invalid addresses. */
@@ -607,8 +587,6 @@ class MIME_Headers {
     /**
      * Returns the list of valid mailing list headers.
      *
-     * @access public
-     *
      * @return array  The list of valid mailing list headers.
      */
     function listHeaders()
@@ -629,8 +607,6 @@ class MIME_Headers {
     /**
      * Do any mailing list headers exist?
      *
-     * @access public
-     *
      * @return boolean  True if any mailing list headers exist.
      */
     function listHeadersExist()
@@ -647,8 +623,6 @@ class MIME_Headers {
     /**
      * Sets a new string to use for EOLs.
      *
-     * @access public
-     *
      * @param string $eol  The string to use for EOLs.
      */
     function setEOL($eol)
@@ -659,8 +633,6 @@ class MIME_Headers {
     /**
      * Get the string to use for EOLs.
      *
-     * @access public
-     *
      * @return string  The string to use for EOLs.
      */
     function getEOL()
@@ -671,8 +643,6 @@ class MIME_Headers {
     /**
      * Returns the flag status.
      *
-     * @access public
-     *
      * @param string $flag  Is this flag set?
      *                      Flags: recent, unseen, answered, draft, important,
      *                             deleted
@@ -691,8 +661,6 @@ class MIME_Headers {
     /**
      * Get the primary from address (first address in the From: header).
      *
-     * @access public
-     *
      * @return string  The from address (user@host).
      */
     function getFromAddress()
@@ -709,7 +677,7 @@ class MIME_Headers {
     /**
      * Get a header from the header object.
      *
-     * @access public
+     * @todo Replace with getOb() from IMP's IMP_Headers for Horde 4.0.
      *
      * @param string $field    The object field to retrieve (see
      *                         imap_headerinfo() for the list of fields).
@@ -732,6 +700,9 @@ class MIME_Headers {
             $data = $ob->$field;
             if (!empty($decode) && !is_object($data) && !is_array($data)) {
                 include_once 'Horde/MIME.php';
+                if (!empty($GLOBALS['mime_headers']['default_charset'])) {
+                    $data = String::convertCharset($data, $GLOBALS['mime_headers']['default_charset']);
+                }
                 $data = MIME::decode($data);
             }
         }
