@@ -3,9 +3,9 @@
  * The MIME_MDN:: class implements Message Disposition Notifications as
  * described by RFC 2298.
  *
- * $Horde: framework/MIME/MIME/MDN.php,v 1.4.10.4 2005/06/08 03:01:50 selsky Exp $
+ * $Horde: framework/MIME/MIME/MDN.php,v 1.4.10.8 2006/01/24 21:32:56 slusarz Exp $
  *
- * Copyright 2004-2005 Michael Slusarz <slusarz@curecanti.org>
+ * Copyright 2004-2006 Michael Slusarz <slusarz@curecanti.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -79,8 +79,8 @@ class MIME_MDN {
         /* RFC 2298 [2.1]: Explicit confirmation is needed if there is more
          * than one distinct address in the Disposition-Notification-To
          * header. */
-        $addr_arr = $parser->parseAddressList($this->getMDNReturnAddr());
-        if (count($addr_arr) > 1) {
+        $addr_arr = $parser->parseAddressList($this->getMDNReturnAddr(), null, null, false);
+        if (is_a($addr_arr, 'PEAR_Error') || (count($addr_arr) > 1)) {
             return true;
         }
 
@@ -89,11 +89,12 @@ class MIME_MDN {
          * from the address in the Return-Path header." This comparison is
          * case-sensitive for the mailbox part and case-insensitive for the
          * host part. */
-        $ret_arr = $parser->parseAddressList($return_path);
-        if (($addr_arr[0]->mailbox != $ret_arr[0]->mailbox) ||
-            (String::lower($addr_arr[0]->host) != String::lower($ret_arr[0]->host))) {
+        $ret_arr = $parser->parseAddressList($return_path, null, null, false);
+        if (!is_a($ret_arr, 'PEAR_Error') &&
+            (($addr_arr[0]->mailbox != $ret_arr[0]->mailbox) ||
+            (String::lower($addr_arr[0]->host) != String::lower($ret_arr[0]->host)))) {
             return false;
-        } 
+        }
 
         return true;
     }
@@ -252,10 +253,6 @@ class MIME_MDN {
     {
         /* This is the RFC 2298 way of requesting a receipt. */
         $headob->addHeader('Disposition-Notification-To', $to);
-
-        /* For certain Pegasus mail installations. */
-        $headob->addHeader('X-Confirm-Reading-To', $to);
-        $headob->addHeader('X-PMRQC', 1);
     }
 
 }
