@@ -4,17 +4,16 @@
  * MIME type for various content, if it provided with different
  * levels of information.
  *
- * $Horde: framework/MIME/MIME/Magic.php,v 1.52.8.1 2005/01/03 12:19:05 jan Exp $
+ * $Horde: framework/MIME/MIME/Magic.php,v 1.52.8.12 2006/01/01 21:28:24 jan Exp $
  *
- * Copyright 1999-2005 Anil Madhavapeddy <anil@recoil.org>
- * Copyright 2002-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 1999-2006 Anil Madhavapeddy <anil@recoil.org>
+ * Copyright 2002-2006 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Anil Madhavapeddy <anil@recoil.org>
  * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @version $Revision: 1.52.8.1 $
  * @since   Horde 1.3
  * @package Horde_MIME
  */
@@ -64,8 +63,6 @@ class MIME_Magic {
      * we fall back to a custom MIME handler 'x-extension/$ext', which
      * can be used as a normal MIME type internally throughout Horde.
      *
-     * @access public
-     *
      * @param string $ext  The file extension to be mapped to a MIME type.
      *
      * @return string  The MIME type of the file extension.
@@ -94,18 +91,13 @@ class MIME_Magic {
     }
 
     /**
-     * Attempt to convert a filename to a MIME type, based on the
-     * global Horde and application specific config files.
+     * Attempt to convert a filename to a MIME type, based on the global Horde
+     * and application specific config files.
      *
-     * @access public
-     *
-     * @param string $filename           The filename to be mapped to a MIME
-     *                                   type.
-     * @param optional boolean $unknown  How should unknown extensions be
-     *                                   handled?  If true, will return
-     *                                   'x-extension/*' types.  If false,
-     *                                   will return
-     *                                   'application/octet-stream'.
+     * @param string $filename  The filename to be mapped to a MIME type.
+     * @param boolean $unknown  How should unknown extensions be handled? If
+     *                          true, will return 'x-extension/*' types.  If
+     *                          false, will return 'application/octet-stream'.
      *
      * @return string  The MIME type of the filename.
      */
@@ -137,8 +129,6 @@ class MIME_Magic {
      *
      * If we cannot map the type to a file extension, we return false.
      *
-     * @access public
-     *
      * @param string $type  The MIME type to be mapped to a file extension.
      *
      * @return string  The file extension of the MIME type.
@@ -168,28 +158,41 @@ class MIME_Magic {
      * Uses variants of the UNIX "file" command to attempt to determine the
      * MIME type of an unknown file.
      *
-     * @access public
+     * @param string $path      The path to the file to analyze.
+     * @param string $magic_db  Path to the mime magic database.
      *
-     * @param string $path  The path to the file to analyze.
-     *
-     * @return string  The MIME type of the file.  Returns false if either
-     *                 the file type isn't recognized or the file command is
-     *                 not available.
+     * @return string  The MIME type of the file.  Returns false if the file
+     *                 type isn't recognized or an error happened.
      */
-    function analyzeFile($path)
+    function analyzeFile($path, $magic_db = null)
     {
         /* If the PHP Mimetype extension is available, use that. */
         if (Util::extensionExists('fileinfo')) {
-            $res = finfo_open(FILEINFO_MIME);
-            $type = finfo_file($res, $path);
-            finfo_close($res);
-
-            /* Remove any character set information. */
-            $pos = strpos($type, ';');
-            if ($pos !== false) { 
-                $type = rtrim(substr($type, 0, $pos));
+            if (empty($magic_db)) {
+                $res = @finfo_open(FILEINFO_MIME);
+            } else {
+                $res = @finfo_open(FILEINFO_MIME, $magic_db);
             }
-            return $type;
+            if ($res) {
+                $type = finfo_file($res, $path);
+                finfo_close($res);
+
+                /* Remove any additional information. */
+                foreach (array(';', ',', '\\0') as $separator) {
+                    $pos = strpos($type, $separator);
+                    if ($pos !== false) {
+                        $type = rtrim(substr($type, 0, $pos));
+                    }
+                }
+
+                if (preg_match('|^[a-z0-9]+/[.-a-z0-9]+$|i', $type)) {
+                    return $type;
+                }
+            }
+        }
+
+        if (Util::extensionExists('mime_magic')) {
+            return trim(mime_content_type($path));
         } else {
             /* Use a built-in magic file. */
             $mime_magic = &MIME_Magic::_getMimeMagicFile();
@@ -217,16 +220,40 @@ class MIME_Magic {
      * Uses variants of the UNIX "file" command to attempt to determine the
      * MIME type of an unknown byte stream.
      *
-     * @access public
+     * @param string $data      The file data to analyze.
+     * @param string $magic_db  Path to the mime magic database.
      *
-     * @param string $data  The file data to analyze.
-     *
-     * @return string  The MIME type of the file.  Returns false if either
-     *                 the file type isn't recognized or the file command is
-     *                 not available.
+     * @return string  The MIME type of the file.  Returns false if the file
+     *                 type isn't recognized or an error happened.
      */
-    function analyzeData($data)
+    function analyzeData($data, $magic_db = null)
     {
+        /* If the PHP Mimetype extension is available, use that. */
+        if (Util::extensionExists('fileinfo')) {
+            if (empty($magic_db)) {
+                $res = @finfo_open(FILEINFO_MIME);
+            } else {
+                $res = @finfo_open(FILEINFO_MIME, $magic_db);
+            }
+            if (!$res) {
+                return false;
+            }
+
+            $type = finfo_buffer($res, $data);
+            finfo_close($res);
+
+            /* Remove any additional information. */
+            $pos = strpos($type, ';');
+            if ($pos !== false) {
+                $type = rtrim(substr($type, 0, $pos));
+            }
+            $pos = strpos($type, ',');
+            if ($pos !== false) {
+                $type = rtrim(substr($type, 0, $pos));
+            }
+            return $type;
+        }
+
         /* Use a built-in magic file. */
         $mime_magic = &MIME_Magic::_getMimeMagicFile();
         foreach ($mime_magic as $offset => $odata) {
