@@ -2,17 +2,16 @@
 /**
  * The MIME_Viewer_tnef class allows MS-TNEF attachments to be displayed.
  *
- * $Horde: framework/MIME/MIME/Viewer/tnef.php,v 1.13.10.1 2005/01/03 12:19:06 jan Exp $
+ * $Horde: framework/MIME/MIME/Viewer/tnef.php,v 1.13.10.9 2006/01/01 21:28:25 jan Exp $
  *
- * Copyright 2002-2005 Jan Schneider <jan@horde.org>
- * Copyright 2002-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * Copyright 2002-2006 Jan Schneider <jan@horde.org>
+ * Copyright 2002-2006 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Jan Schneider <jan@horde.org>
  * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
- * @version $Revision: 1.13.10.1 $
  * @since   Horde 3.0
  * @package Horde_MIME_Viewer
  */
@@ -21,9 +20,7 @@ class MIME_Viewer_tnef extends MIME_Viewer {
     /**
      * Render out the current tnef data.
      *
-     * @access public
-     *
-     * @param optional array $params  Any parameters the viewer may need.
+     * @param array $params  Any parameters the viewer may need.
      *
      * @return string  The rendered contents.
      */
@@ -51,13 +48,11 @@ class MIME_Viewer_tnef extends MIME_Viewer {
     /**
      * Return the MIME content type of the rendered content.
      *
-     * @access public
-     *
-     * @return string  The content type of the output. 
+     * @return string  The content type of the output.
      */
     function getType()
     {
-        return 'text/html';
+        return 'text/html; charset=' . NLS::getCharset();
     }
 
 }
