@@ -13,14 +13,14 @@ class Horde_Mobile_card extends Horde_Mobile_element {
     /**
      * Constructor
      *
-     * @param string $name  (optional, default: null)
-     *                      The name of this card. Can be used in anchor links.
-     * @param string $title (optional, default: null)
-     *                      If a string is provided here, it will be displayed
-     *                      in the HTML title bar, respectively somewhere on the WAP display. Using a
-     *                      title you will normally have to spend one of your few lines on your WAP
-     *                      display. Consider that some WAP phones/SDK's and handheld devices don't
-     *                      display the title at all.
+     * @param string $name   The name of this card. Can be used in anchor
+     *                       links.
+     * @param string $title  If a string is provided here, it will be displayed
+     *                       in the HTML title bar, respectively somewhere on
+     *                       the WAP display. Using a title you will normally
+     *                       have to spend one of your few lines on your WAP
+     *                       display. Consider that some WAP phones/SDK's and
+     *                       handheld devices don't display the title at all.
      */
     function Horde_Mobile_card($name = null, $title = null)
     {
@@ -33,7 +33,8 @@ class Horde_Mobile_card extends Horde_Mobile_element {
         static $linksetAdded;
 
         if (!is_a($element, 'Horde_Mobile_element')) {
-            return PEAR::raiseError('Invalid element.');
+            $error = PEAR::raiseError('Invalid element.');
+            return $error;
         } elseif (is_a($element, 'Horde_Mobile_text') ||
                   is_a($element, 'Horde_Mobile_table') ||
                   is_a($element, 'Horde_Mobile_image') ||
@@ -46,19 +47,22 @@ class Horde_Mobile_card extends Horde_Mobile_element {
             $this->_elements[] = &$element;
         } elseif (is_a($element, 'Horde_Mobile_form')) {
             if (!empty($this->_form)) {
-                return PEAR::raiseError('Cards may only contain one Form element.');
+                $error = PEAR::raiseError('Cards may only contain one Form element.');
+                return $error;
             }
             $this->_elements[] = &$element;
             $this->_form = &$element;
         } elseif (is_a($element, 'Horde_Mobile_linkset')) {
             if (!empty($linksetAdded)) {
-                return PEAR::raiseError('Cards may only contain one Linkset element.');
+                $error = PEAR::raiseError('Cards may only contain one Linkset element.');
+                return $error;
             }
             $block = &new Horde_Mobile_block($element);
             $this->_elements[] = &$block;
             $linksetAdded = true;
         } else {
-            return PEAR::raiseError('This element must be inside an appropriate container element.');
+            $error = PEAR::raiseError('This element must be inside an appropriate container element.');
+            return $error;
         }
 
         return $element;
@@ -85,7 +89,7 @@ class Horde_Mobile_card extends Horde_Mobile_element {
  * renderer object
  *
  * Do not overstuff Horde_Mobile objects. Remember that a lot of WAP
- * clients can not handle more than about 1400 byte of compiled data.
+ * clients cannot handle more than about 1400 bytes of compiled data.
  *
  * Examples:
  *
@@ -101,15 +105,14 @@ class Horde_Mobile_card extends Horde_Mobile_element {
  *
  * $myPage->render();
  *
- * $Horde: framework/Mobile/Mobile.php,v 1.32 2004/10/19 15:21:49 mdjukic Exp $
+ * $Horde: framework/Mobile/Mobile.php,v 1.32.10.9 2006/06/26 06:27:23 selsky Exp $
  *
- * Copyright 2002-2004 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2006 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
- * @version $Revision: 1.32 $
  * @since   Horde 3.0
  * @package Horde_Mobile
  */
@@ -127,13 +130,13 @@ class Horde_Mobile extends Horde_Mobile_card {
     /**
      * Constructor
      *
-     * @param string $title (optional, default: null)
-     *                      If a string is provided here, it will be displayed
-     *                      in the HTML title bar, respectively somewhere on the WAP display. Using a
-     *                      title you will normally have to spend one of your few lines on your WAP
-     *                      display. Consider that some WAP phones/SDK's and handheld devices don't
-     *                      display the title at all.
-     * @param string $agent (optional) If specified, use instead of HTTP_USER_AGENT.
+     * @param string $title  If a string is provided here, it will be displayed
+     *                       in the HTML title bar, respectively somewhere on
+     *                       the WAP display. Using a title you will normally
+     *                       have to spend one of your few lines on your WAP
+     *                       display. Consider that some WAP phones/SDK's and
+     *                       handheld devices don't display the title at all.
+     * @param string $agent  If specified, use instead of HTTP_USER_AGENT.
      */
     function Horde_Mobile($title = null, $agent = null)
     {
@@ -160,7 +163,8 @@ class Horde_Mobile extends Horde_Mobile_card {
     {
         if (is_a($element, 'Horde_Mobile_card')) {
             if (count($this->_elements)) {
-                return PEAR::raiseError('You cannot mix Horde_Mobile_cards and other elements at the deck level.');
+                $error = PEAR::raiseError('You cannot mix Horde_Mobile_cards and other elements at the deck level.');
+                return $error;
             }
 
             $this->_usingCards = true;
@@ -169,7 +173,8 @@ class Horde_Mobile extends Horde_Mobile_card {
             return $element;
         } else {
             if (count($this->_cards)) {
-                return PEAR::raiseError('You cannot mix Horde_Mobile_cards and other elements at the deck level.');
+                $error = PEAR::raiseError('You cannot mix Horde_Mobile_cards and other elements at the deck level.');
+                return $error;
             }
 
             return parent::add($element);
@@ -350,7 +355,8 @@ class Horde_Mobile_form extends Horde_Mobile_element {
             $block = &new Horde_Mobile_block($formElement);
             $this->_elements[] = &$block;
         } else {
-            return PEAR::raiseError('Specified element cannot be inside a form.');
+            $error = PEAR::raiseError('Specified element cannot be inside a form.');
+            return $error;
         }
 
         return $formElement;
@@ -513,7 +519,7 @@ class Horde_Mobile_block extends Horde_Mobile_element {
     function &add(&$element)
     {
         if (!is_a($element, 'Horde_Mobile_element')) {
-            return PEAR::raiseError('Invalid element.');
+            $error = PEAR::raiseError('Invalid element.');
         } elseif (is_a($element, 'Horde_Mobile_text') ||
                   is_a($element, 'Horde_Mobile_table') ||
                   is_a($element, 'Horde_Mobile_image') ||
@@ -525,8 +531,9 @@ class Horde_Mobile_block extends Horde_Mobile_element {
             $this->_elements[] = &$element;
             return $element;
         } else {
-            return PEAR::raiseError('The element is not allowed inside a block.');
+            $error = PEAR::raiseError('The element is not allowed inside a block.');
         }
+        return $error;
     }
 
 }
@@ -555,7 +562,7 @@ class Horde_Mobile_text extends Horde_Mobile_element {
     /**
      * Constructor
      * @param string $text       The text content of the element.
-     * @param array  $attributes (optional) Text attributes. Any of:
+     * @param array $attributes  Text attributes. Any of:
      *   'b'
      *   'u'
      *   'i'
@@ -577,6 +584,11 @@ class Horde_Mobile_text extends Horde_Mobile_element {
  * This class allows to insert bitmap images into a Horde_Mobile_block,
  * Horde_Mobile_form or Horde_Mobile_row object.
  *
+ * Examples:
+ *
+ * $image = new Horde_Mobile_image('/path/to/image.wbmp',
+ *                                 array('height' => 100, 'width' => 100));
+ *
  * @see Horde_Mobile_block
  * @see Horde_Mobile_form
  * @see Horde_Mobile_row
@@ -584,6 +596,29 @@ class Horde_Mobile_text extends Horde_Mobile_element {
  * @package Horde_Mobile
  */
 class Horde_Mobile_image extends Horde_Mobile_element {
+
+    var $_src = '';
+    var $_attributes = array();
+
+    /**
+     * Constructor
+     * @param string $src        The source location of the image.
+     * @param array $attributes  Image attributes. Any of:
+     *   'align'
+     *   'alt'
+     *   'height'
+     *   'hspace'
+     *   'vspace'
+     *   'width'
+     *   'class'
+     *   'id'
+     */
+    function Horde_Mobile_image($src, $attributes = array())
+    {
+        $this->_src = $src;
+        $this->_attributes = $attributes;
+    }
+
 }
 
 /**
@@ -622,12 +657,13 @@ class Horde_Mobile_table extends Horde_Mobile_element {
     /**
      * Adds a Horde_Mobile_row object to Horde_Mobile_table.
      *
-     * @param object Horde_Mobile_row $row  The row object to add.
+     * @param Horde_Mobile_row $row  The row object to add.
      */
     function &add(&$row)
     {
         if (!is_a($row, 'Horde_Mobile_row')) {
-            return PEAR::raiseError('Rows must be Horde_Mobile_row objects.');
+            $error = PEAR::raiseError('Rows must be Horde_Mobile_row objects.');
+            return $error;
         }
 
         $this->_rows[] = &$row;
@@ -663,10 +699,11 @@ class Horde_Mobile_row extends Horde_Mobile_element {
     /**
      * Adds a column element to a Horde_Mobile_row object.
      *
-     * @param object Horde_Mobile_element $cellElement (optional) Can be a Horde_Mobile_text object,
-     *                                                 a Horde_Mobile_image object, a Horde_Mobile_link
-     *                                                 object or null (default). The latter results in
-     *                                                 an empty cell.
+     * @param Horde_Mobile_element $cellElement  Can be a Horde_Mobile_text
+     *                                           object, a Horde_Mobile_image
+     *                                           object, a Horde_Mobile_link
+     *                                           object or null. The latter
+     *                                           results in an empty cell.
      */
     function &add($cellElement = null)
     {
@@ -674,7 +711,8 @@ class Horde_Mobile_row extends Horde_Mobile_element {
             if (!is_a($cellElement, 'Horde_Mobile_text') &&
                 !is_a($cellElement, 'Horde_Mobile_link') &&
                 !is_a($cellElement, 'Horde_Mobile_image')) {
-                return PEAR::raiseError('Table cells can only contain text, links, or images.');
+                $error = PEAR::raiseError('Table cells can only contain text, links, or images.');
+                return $error;
             }
             $this->_columns[] = &$cellElement;
             return $cellElement;
@@ -684,6 +722,7 @@ class Horde_Mobile_row extends Horde_Mobile_element {
             return $t;
         } else {
             $this->_columns[] = &$cellElement;
+            return $cellElement;
         }
     }
 
@@ -695,8 +734,7 @@ class Horde_Mobile_row extends Horde_Mobile_element {
 }
 
 /**
- * This class provides a text input field in a Horde_Mobile_form
- * object.
+ * This class provides a text input field in a Horde_Mobile_form object.
  *
  * Examples:
  *
@@ -720,13 +758,16 @@ class Horde_Mobile_input extends Horde_Mobile_formElement {
     /**
      * Constructor
      *
-     * @param name Variable in which the input is sent to the destination URL.
-     * @param value Initial value (string!) that will be presented in the input field.
-     * @param label Describes your input field on the surfer's screen/display.
-     * @param format (optional, default: '*M') Input format code according to the WAP standard.
-     *                                         Allows the WAP user client e.g. to input only
-     *                                         digits and no characters. On an HTML generated page
-     *                                         this format has no significance.
+     * @param string $name    Variable in which the input is sent to the
+     *                        destination URL.
+     * @param string $value   Initial value that will be presented in the
+     *                        input field.
+     * @param string $label   Describes your input field on the surfer's
+     *                        screen/display.
+     * @param string $format  Input format code according to the WAP standard.
+     *                        Allows the WAP user client e.g. to input only
+     *                        digits and no characters. On an HTML generated
+     *                        page this format has no significance.
      */
     function Horde_Mobile_input($name, $value, $label = '', $format = '*M')
     {
@@ -760,11 +801,14 @@ class Horde_Mobile_textarea extends Horde_Mobile_formElement {
     /**
      * Constructor.
      *
-     * @param string  $name   Variable in which the input is sent to the destination URL.
-     * @param string  $value  Initial value (string!) that will be presented in the textarea.
-     * @param string  $label  Describes your textarea on the surfer's screen/display.
-     * @param integer $rows   Rows (optional, default: 3)
-     * @param integer $cols   Columns (optional, default: 16)
+     * @param string $name   Variable in which the input is sent to the
+     *                       destination URL.
+     * @param string $value  Initial value that will be presented in the
+     *                       textarea.
+     * @param string $label  Describes your textarea on the surfer's
+     *                       screen/display.
+     * @param integer $rows  Rows.
+     * @param integer $cols  Columns.
      */
     function Horde_Mobile_textarea($name, $value, $label, $rows = 3, $cols = 16)
     {
@@ -804,11 +848,12 @@ class Horde_Mobile_select extends Horde_Mobile_formElement {
     /**
      * Constructor
      *
-     * @param string $name  Variable in which the information about the selected option is sent to
-     *                      the destination URL.
-     * @param string $type  (optional) Type of select area:
-     *                      'popup': popup the whole selection list (default)
-     *                      'spin':  rotate options on a WAP device screen (OW 1.3 GUI only).
+     * @param string $name  Variable in which the information about the
+     *                      selected option is sent to the destination URL.
+     * @param string $type  Type of select area:
+     *                      'popup': popup the whole selection list
+     *                      'spin':  rotate options on a WAP device screen (OW
+     *                               1.3 GUI only).
      */
     function Horde_Mobile_select($name, $type = 'popup')
     {
@@ -820,9 +865,11 @@ class Horde_Mobile_select extends Horde_Mobile_formElement {
     /**
      * Adds one option to a Horde_Mobile_select object.
      *
-     * @param string $label         Describes the option on the surfer's screen/display.
-     * @param string $value         Value sent in the "name" variable, if this is the option selected.
-     * @param boolean $is_selected  (optional) Allowed values are true or false (default).
+     * @param string $label         Describes the option on the surfer's
+     *                              screen/display.
+     * @param string $value         Value sent in the "name" variable, if this
+     *                              is the option selected.
+     * @param boolean $is_selected  Allowed values are true or false.
      */
     function add($label, $value, $is_selected = false)
     {
@@ -837,8 +884,7 @@ class Horde_Mobile_select extends Horde_Mobile_formElement {
 }
 
 /**
- * This class provides a radio button element in a Horde_Mobile_form
- * object.
+ * This class provides a radio button element in a Horde_Mobile_form object.
  *
  * Examples:
  *
@@ -870,9 +916,11 @@ class Horde_Mobile_radio extends Horde_Mobile_formElement {
     /**
      * Adds one radio button to a Horde_Mobile_radio object.
      *
-     * @param string $label        Describes the radiobutton on the surfer's screen/display.
-     * @param string $value        Value sent in the "name" variable, if this button is selected.
-     * @param boolean $is_checked  (optional) Allowed values are true or false.
+     * @param string $label        Describes the radiobutton on the surfer's
+     *                             screen/display.
+     * @param string $value        Value sent in the "name" variable, if this
+     *                             button is selected.
+     * @param boolean $is_checked  Allowed values are true or false.
      */
     function add($label, $value, $is_checked = false)
     {
@@ -887,8 +935,7 @@ class Horde_Mobile_radio extends Horde_Mobile_formElement {
 }
 
 /**
- * This class provides a single checkbox element in a
- * Horde_Mobile_form object.
+ * This class provides a single checkbox element in a Horde_Mobile_form object.
  *
  * Examples:
  *
@@ -909,11 +956,12 @@ class Horde_Mobile_checkbox extends Horde_Mobile_formElement {
     /**
      * Constructor
      *
-     * @param string  $name     Variable in which "value" is sent to the destination URL,
-     *                          if the box is checked.
-     * @param string  $value    See name.
-     * @param string  $label    Describes the checkbox on the surfer's screen/display.
-     * @param boolean $checked  (optional) Allowed values are true or false (default false).
+     * @param string $name      Variable in which "value" is sent to the
+     *                          destination URL, if the box is checked.
+     * @param string $value     See name.
+     * @param string $label     Describes the checkbox on the surfer's
+     *                          screen/display.
+     * @param boolean $checked  Allowed values are true or false.
      */
     function Horde_Mobile_checkbox($name, $value, $label, $checked = false)
     {
@@ -958,9 +1006,8 @@ class Horde_Mobile_hidden extends Horde_Mobile_formElement {
 }
 
 /**
- * This class provides a submit button for a Horde_Mobile_form object.
- * One Horde_Mobile_form object can contain only one
- * Horde_Mobile_submit object.
+ * This class provides a submit button for a Horde_Mobile_form object. One
+ * Horde_Mobile_form object can contain only one Horde_Mobile_submit object.
  *
  * Examples:
  * $mySubmit = &new Horde_Mobile_submit('Submit');
@@ -978,7 +1025,8 @@ class Horde_Mobile_submit extends Horde_Mobile_formElement {
      * Constructor
      *
      * @param string $label  What's written on the button.
-     * @param string $name   (optional) Variable in which "label" is sent to the destination URL.
+     * @param string $name   Variable in which "label" is sent to the
+     *                       destination URL.
      */
     function Horde_Mobile_submit($label, $name = '')
     {
@@ -989,8 +1037,8 @@ class Horde_Mobile_submit extends Horde_Mobile_formElement {
 }
 
 /**
- * This class provides a link in a Horde_Mobile, Horde_Mobile_linkset
- * or Horde_Mobile_table object.
+ * This class provides a link in a Horde_Mobile, Horde_Mobile_linkset or
+ * Horde_Mobile_table object.
  *
  * Examples:
  *
@@ -1017,10 +1065,12 @@ class Horde_Mobile_link extends Horde_Mobile_element {
      *
      * @param string $label  Describes the link on the surfer's screen/display.
      * @param string $url    Next destination address. MUST be valid XML (&amp; instead of &, etc.).
-     * @param string $title  (optional) If a string is provided here, it will be displayed
-     *                       in the HTML browser status bar during "MouseOver", respectively somewhere
-     *                       on the WAP display. In order to work well with a broad range of user agents,
-     *                       keep your title under 6 characters.
+     * @param string $title  If a string is provided here, it will be displayed
+     *                       in the HTML browser status bar during
+     *                       "MouseOver", respectively somewhere on the WAP
+     *                       display. In order to work well with a broad range
+     *                       of user agents, keep your title under 6
+     *                       characters.
      */
     function Horde_Mobile_link($label, $url, $title = '')
     {
@@ -1036,9 +1086,9 @@ class Horde_Mobile_link extends Horde_Mobile_element {
 }
 
 /**
- * This class provides a phone number in a Horde_Mobile object. If
- * supported by their mobile device, users can establish a voice
- * connection to the specified number.
+ * This class provides a phone number in a Horde_Mobile object. If supported by
+ * their mobile device, users can establish a voice connection to the
+ * specified number.
  *
  * Examples:
  *
@@ -1059,9 +1109,11 @@ class Horde_Mobile_phone extends Horde_Mobile_element {
      * Constructor
      *
      * @param string $phone_number  Phone number to dial.
-     * @param string $title         (optional) If a string is provided here, the call button on a
-     *                              WAP/HDML device will be enabled. In order to work well with
-     *                              a broad range of user agents, keep your title under 6 characters.
+     * @param string $title         If a string is provided here, the call
+     *                              button on a WAP/HDML device will be
+     *                              enabled. In order to work well with
+     *                              a broad range of user agents, keep your
+     *                              title under 6 characters.
      */
     function Horde_Mobile_phone($phone_number, $title = '')
     {
@@ -1111,26 +1163,29 @@ class Horde_Mobile_linkset extends Horde_Mobile_element {
     /**
      * Adds a Horde_Mobile_link object to Horde_Mobile_linkset.
      *
-     * @param object Horde_Mobile_link $link  The link object to add.
+     * @param Horde_Mobile_link $link  The link object to add.
      *
      * @see Horde_Mobile_link
      */
     function &add(&$link)
     {
         if (!is_a($link, 'Horde_Mobile_link')) {
-            return PEAR::raiseError('Links must be Horde_Mobile_link objects.');
+            $error = PEAR::raiseError('Links must be Horde_Mobile_link objects.');
+            return $error;
         }
 
         $this->_elements[] = &$link;
         $link->set('accesskey', count($this->_elements));
+
+        return $link;
     }
 
 }
 
 /**
- * This class will cause a horizontal rule to be drawn across the
- * screen.  You can use it to separate text paragraphs in Horde_Mobile
- * or Horde_Mobile_form objects.
+ * This class will cause a horizontal rule to be drawn across the screen.  You
+ * can use it to separate text paragraphs in Horde_Mobile or Horde_Mobile_form
+ * objects.
  *
  * Examples:
  *
@@ -1154,9 +1209,9 @@ class Horde_Mobile_rule extends Horde_Mobile_element {
     /**
      * Constructor
      *
-     * @param integer $width (optional) Percentage of screen width or absolute value in
-     *                                  number of pixels (e.g. "50%", 100).
-     * @param integer $size  (optional) Height of the line to be drawn in pixels.
+     * @param integer $width  Percentage of screen width or absolute value in
+     *                        number of pixels (e.g. "50%", 100).
+     * @param integer $size   Height of the line to be drawn in pixels.
      */
     function Horde_Mobile_rule($width = '', $size = '')
     {
