@@ -1,6 +1,6 @@
 <?php
 
-require_once(HORDE_BASE . '/lib/SessionHandler/sql.php');
+require_once dirname(__FILE__) . '/sql.php';
 
 /**
  * SessionHandler implementation for PHP's PEAR database abstraction layer.
@@ -8,63 +8,68 @@ require_once(HORDE_BASE . '/lib/SessionHandler/sql.php');
  * If you access your database through ODBC, you will almost certainly need
  * to change PHP's default value for odbc.defaultlrl (this is a php.ini
  * setting). The default is 4096, which is too small (your session data will
- * data will be chopped off), and setting it to 0 DOES NOT work - that
- * doesn't mean no limit, for some reason. odbc.defaultlrl = 32768
- * seems to work pretty well (using MSSQL-2000).
+ * be chopped off), and setting it to 0 DOES NOT work - that doesn't mean no
+ * limit, for some reason. odbc.defaultlrl = 32768 seems to work pretty well
+ * (using MSSQL-2000).
  *
- * Required values for $params:<pre>
- *     'hostspec'  --  The hostname of the database server.
- *     'protocol'  --  The communication protocol ('tcp', 'unix', etc.).
- *     'username'  --  The username with which to connect to the database.
- *     'password'  --  The password associated with 'username'.
- *     'database'  --  The name of the database.
- *     'table'     --  The name of the sessiondata table in 'database'.</pre>
+ * Required parameters:<pre>
+ *   'hostspec'  The hostname of the database server.
+ *   'protocol'  The communication protocol ('tcp', 'unix', etc.).
+ *   'username'  The username with which to connect to the database.
+ *   'password'  The password associated with 'username'.
+ *   'database'  The name of the database.</pre>
  *
- * The table structure can be created by the scripts/db/sessionhandler.sql
- * script.
+ * Optional parameters:<pre>
+ *   'table'     The name of the sessiondata table in 'database'.  Default is
+ *               'horde_sessionhandler'.</pre>
  *
- * $Horde: framework/SessionHandler/SessionHandler/sapdb.php,v 1.13.12.1 2005/01/03 12:19:13 jan Exp $
+ * The table structure for the SessionHandler can be found in
+ * horde/scripts/sql/horde_sessionhandler.sapdb.sql.
  *
- * Copyright 2002-2005 Mike Cochrane <mike@graftonhall.co.nz>
+ * $Horde: framework/SessionHandler/SessionHandler/sapdb.php,v 1.13.12.8 2006/01/01 21:28:34 jan Exp $
+ *
+ * Copyright 2002-2006 Mike Cochrane <mike@graftonhall.co.nz>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Mike Cochrane <mike@graftonhall.co.nz>
- * @version $Revision: 1.13.12.1 $
  * @since   Horde 3.0
  * @package Horde_SessionHandler
  */
 class SessionHandler_sapdb extends SessionHandler_sql {
 
-    /** Hash containing connection parameters. */
-    var $_params = array();
-
-    /** Handle for the current database connection.
-        @var object DB $db */
-    var $_db;
-
-    /**
-     * Boolean indicating whether or not we're connected to the SQL
-     * server. */
-    var $_connected = false;
-
     /**
      * Constructs a new SQL SessionHandler object.
      *
-     * @param array  $params    A hash containing connection parameters.
+     * @param array $params  A hash containing connection parameters.
      */
     function SessionHandler_sapdb($params = array())
     {
-        $this->_params = $params;
-        $this->_params['phptype'] = 'odbc';
+        $params['phptype'] = 'odbc';
+        parent::SessionHandler_sql($params);
     }
 
+    /**
+     * Read the data for a particular session identifier from the
+     * SessionHandler backend.
+     *
+     * @param string $id  The session identifier.
+     *
+     * @return string  The session data.
+     */
     function read($id)
     {
         /* Make sure we have a valid database connection. */
         $this->_connect();
 
+        /* Begin a transaction. */
+        $result = $this->_db->autocommit(false);
+        if (is_a($result, 'PEAR_Error')) {
+            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+            return '';
+        }
+
         /* Build the SQL query. */
         $query = sprintf('SELECT session_data FROM %s WHERE session_id = %s',
                          $this->_params['table'],
@@ -76,7 +81,7 @@ class SessionHandler_sapdb extends SessionHandler_sql {
 
         /* Execute the query */
         $result = odbc_exec($this->_db->connection, $query);
-        odbc_longreadlen($result, 1024*1024);
+        odbc_longreadlen($result, 1048576);
 
         /* Fetch the value */
         odbc_fetch_row($result, 0);
