@@ -1,16 +1,24 @@
 <?php
+/**
+ * @package Horde_iCalendar
+ */
+
+/**
+ * String package
+ */
+include_once 'Horde/String.php';
+
 /**
  * Class representing iCalendar files.
  *
- * $Horde: framework/iCalendar/iCalendar.php,v 1.57 2004/12/05 17:09:40 chuck Exp $
+ * $Horde: framework/iCalendar/iCalendar.php,v 1.57.4.34 2006/08/05 13:43:47 karsten Exp $
  *
- * Copyright 2003-2004 Mike Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2003-2006 Mike Cochrane <mike@graftonhall.co.nz>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Mike Cochrane <mike@graftonhall.co.nz>
- * @version $Revision: 1.57 $
  * @since   Horde 3.0
  * @package Horde_iCalendar
  */
@@ -19,82 +27,126 @@ class Horde_iCalendar {
     /**
      * The parent (containing) iCalendar object.
      *
-     * @var object Horde_iCalendar $_container
+     * @var Horde_iCalendar
      */
     var $_container = false;
 
+    /**
+     * The name/value pairs of attributes for this object (UID,
+     * DTSTART, etc.). Which are present depends on the object and on
+     * what kind of component it is.
+     *
+     * @var array
+     */
     var $_attributes = array();
 
+    /**
+     * Any children (contained) iCalendar components of this object.
+     *
+     * @var array
+     */
     var $_components = array();
 
     /**
-     * According to RFC 2425, we should always use CRLF-terminated
-     * lines.
+     * According to RFC 2425, we should always use CRLF-terminated lines.
      *
-     * @var string $_newline
+     * @var string
      */
     var $_newline = "\r\n";
 
     /**
-     * Return a reference to a new component.
+     * iCalendar format version (different behavior for 1.0 and 2.0
+     * especially with recurring events).
      *
-     * @param string $type       The type of component to return
-     * @param object $container  A container that this component
-     *                           will be associated with.
-     *
-     * @return object  Reference to a Horde_iCalendar_* object as specified.
+     * @var string
      */
-    function &newComponent($type, &$container)
+    var $_version;
+
+    function Horde_iCalendar($version = '2.0')
     {
-        require_once 'Horde/String.php';
-        $type = String::lower($type);
-        $class = 'Horde_iCalendar_' . $type;
-        @include_once dirname(__FILE__) . '/iCalendar/' . $type . '.php';
-        if (class_exists($class)) {
-            $component = &new $class();
-            if ($container !== false) {
-                $component->_container = &$container;
-            }
-            return $component;
-        } else {
-            // Should return an dummy x-unknown type class here.
-            return false;
-        }
+        $this->_version = $version;
+        $this->setAttribute('VERSION', $version);
     }
 
     /**
-     * Set the value of an attribute.
+     * Return a reference to a new component.
      *
-     * @param string  $name   The name of the attribute.
-     * @param string  $value  The value of the attribute.
-     * @param array   $params (optional) Array containing any addition
-     *                        parameters for this attribute.
-     * @param boolean $append (optional) True to append the attribute, False
-     *                        to replace the first matching attribute found.
-     * @param array $values   (optional) array representation of $value.
-     *                        For comma/semicolon seperated lists of values.
-     *                        If not set use $value as single array element.
+     * @param string          $type       The type of component to return
+     * @param Horde_iCalendar $container  A container that this component
+     *                                    will be associated with.
+     *
+     * @return object  Reference to a Horde_iCalendar_* object as specified.
+     *
+     * @static
      */
-    function setAttribute($name, $value, $params = array(), $append = true, $values = false)
+    function &newComponent($type, &$container)
     {
-        $found = $append;
+        $type = String::lower($type);
+        $class = 'Horde_iCalendar_' . $type;
+        if (!class_exists($class)) {
+            include_once 'Horde/iCalendar/' . $type . '.php';
+        }
+        if (class_exists($class)) {
+            $component = new $class();
+            if ($container !== false) {
+                $component->_container = &$container;
+                // Use version of container, not default set by component
+                // constructor.
+                $component->_version = $container->_version;
+            }
+        } else {
+            // Should return an dummy x-unknown type class here.
+            $component = false;
+        }
+
+        return $component;
+    }
+
+    /**
+     * Sets the value of an attribute.
+     *
+     * @param string $name     The name of the attribute.
+     * @param string $value    The value of the attribute.
+     * @param array $params    Array containing any addition parameters for
+     *                         this attribute.
+     * @param boolean $append  True to append the attribute, False to replace
+     *                         the first matching attribute found.
+     * @param array $values    Array representation of $value.  For
+     *                         comma/semicolon seperated lists of values.  If
+     *                         not set use $value as single array element.
+     */
+    function setAttribute($name, $value, $params = array(), $append = true,
+                          $values = false)
+    {
+        // Make sure we update the internal format version if
+        // setAttribute('VERSION', ...) is called.
+        if ($name == 'VERSION') {
+            $this->_version = $value;
+            if ($this->_container !== false) {
+                $this->_container->_version = $value;
+            }
+        }
+
         if (!$values) {
             $values = array($value);
         }
-        $keys = array_keys($this->_attributes);
-        foreach ($keys as $key) {
-            if ($found) break;
-            if ($this->_attributes[$key]['name'] == $name) {
-                $this->_attributes[$key]['params'] = $params;
-                $this->_attributes[$key]['value'] = $value;
-                $this->_attributes[$key]['values'] = $values;
-                $found = true;
+        $found = false;
+        if (!$append) {
+            $keys = array_keys($this->_attributes);
+            foreach ($keys as $key) {
+                if ($this->_attributes[$key]['name'] == String::upper($name)) {
+                    $this->_attributes[$key]['params'] = $params;
+                    $this->_attributes[$key]['value'] = $value;
+                    $this->_attributes[$key]['values'] = $values;
+                    $found = true;
+                    break;
+                }
             }
         }
 
         if ($append || !$found) {
             $this->_attributes[] = array(
-                'name'      => $name,
+                'name'      => String::upper($name),
                 'params'    => $params,
                 'value'     => $value,
                 'values'    => $values
@@ -106,18 +158,18 @@ class Horde_iCalendar {
      * Sets parameter(s) for an (already existing) attribute.  The
      * parameter set is merged into the existing set.
      *
-     * @param string $name    The name of the attribute.
-     * @param array  $params  Array containing any additional
-     *                        parameters for this attribute.
+     * @param string $name   The name of the attribute.
+     * @param array $params  Array containing any additional parameters for
+     *                       this attribute.
      * @return boolean  True on success, false if no attribute $name exists.
      */
-    function setParameter($name, $params)
+    function setParameter($name, $params = array())
     {
         $keys = array_keys($this->_attributes);
         foreach ($keys as $key) {
             if ($this->_attributes[$key]['name'] == $name) {
                 $this->_attributes[$key]['params'] =
-                    array_merge($this->_attributes[$key]['params'] , $params);
+                    array_merge($this->_attributes[$key]['params'], $params);
                 return true;
             }
         }
@@ -128,9 +180,9 @@ class Horde_iCalendar {
     /**
      * Get the value of an attribute.
      *
-     * @param string  $name    The name of the attribute.
-     * @param boolean $params  Return the parameters for this attribute
-     *                         instead of its value.
+     * @param string $name     The name of the attribute.
+     * @param boolean $params  Return the parameters for this attribute instead
+     *                         of its value.
      *
      * @return mixed (object)  PEAR_Error if the attribute does not exist.
      *               (string)  The value of the attribute.
@@ -149,7 +201,7 @@ class Horde_iCalendar {
                 }
             }
         }
-        if (count($result) == 0) {
+        if (!count($result)) {
             require_once 'PEAR.php';
             return PEAR::raiseError('Attribute "' . $name . '" Not Found');
         } if (count($result) == 1 && !$params) {
@@ -191,9 +243,9 @@ class Horde_iCalendar {
      * Returns the value of an attribute, or a specified default value
      * if the attribute does not exist.
      *
-     * @param string $name     The name of the attribute.
-     * @param mixed  $default  (optional) What to return if the attribute
-     *                         specified by $name does not exist.
+     * @param string $name    The name of the attribute.
+     * @param mixed $default  What to return if the attribute specified by
+     *                        $name does not exist.
      *
      * @return mixed (string) The value of $name.
      *               (mixed)  $default if $name does not exist.
@@ -207,7 +259,7 @@ class Horde_iCalendar {
     /**
      * Remove all occurences of an attribute.
      *
-     * @param string  $name   The name of the attribute.
+     * @param string $name  The name of the attribute.
      */
     function removeAttribute($name)
     {
@@ -222,9 +274,10 @@ class Horde_iCalendar {
     /**
      * Get attributes for all tags or for a given tag.
      *
-     * @param string  $tag   (optional) return attributes for this tag.
-     *                       or all attributes if not given
-     * @return array  Array containing all the attributes and their types.
+     * @param string $tag  Return attributes for this tag, or all attributes if
+     *                     not given.
+     *
+     * @return array  An array containing all the attributes and their types.
      */
     function getAllAttributes($tag = false)
     {
@@ -243,7 +296,7 @@ class Horde_iCalendar {
     /**
      * Add a vCalendar component (eg vEvent, vTimezone, etc.).
      *
-     * @param object Horde_iCalendar $component  Component (subclass) to add.
+     * @param Horde_iCalendar $component  Component (subclass) to add.
      */
     function addComponent($component)
     {
@@ -263,6 +316,11 @@ class Horde_iCalendar {
         return $this->_components;
     }
 
+    function getType()
+    {
+        return 'vcalendar';
+    }
+
     /**
      * Return the classes (entry types) we have.
      *
@@ -323,7 +381,6 @@ class Horde_iCalendar {
      */
     function &findComponent($childclass)
     {
-        require_once 'Horde/String.php';
         $childclass = 'Horde_iCalendar_' . String::lower($childclass);
         $keys = array_keys($this->_components);
         foreach ($keys as $key) {
@@ -336,8 +393,8 @@ class Horde_iCalendar {
     }
 
     /**
-     * Clears the iCalendar object (resets the components and
-     * attributes arrays).
+     * Clears the iCalendar object (resets the components and attributes
+     * arrays).
      */
     function clear()
     {
@@ -345,14 +402,39 @@ class Horde_iCalendar {
         $this->_attributes = array();
     }
 
+    /**
+     * Checks if entry is vcalendar 1.0, vcard 2.1 or vnote 1.1.
+     *
+     * These 'old' formats are defined by www.imc.org. The 'new' (non-old)
+     * formats icalendar 2.0 and vcard 3.0 are defined in rfc2426 and rfc2445
+     * respectively.
+     *
+     * @since Horde 3.1.2
+     */
+    function isOldFormat()
+    {
+        if ($this->_container !== false) {
+            return $this->_container->isOldFormat();
+        }
+        if ($this->getType() == 'vcard') {
+            return $this->_version < 3 ? true : false;
+        }
+        if ($this->getType() == 'vNote') {
+            return $this->_version < 2 ? true : false;
+        }
+        if ($this->_version >= 2) {
+            return false;
+        }
+        return true;
+    }
+
     /**
      * Export as vCalendar format.
      */
     function exportvCalendar()
     {
         // Default values.
-        $requiredAttributes['VERSION'] = '2.0';
-        $requiredAttributes['PRODID'] = '-//The Horde Project//Horde_iCalendar Library, Horde 3.0-cvs //EN';
+        $requiredAttributes['PRODID'] = '-//The Horde Project//Horde_iCalendar Library' . (defined('HORDE_VERSION') ? ', Horde ' . constant('HORDE_VERSION') : '') . '//EN';
         $requiredAttributes['METHOD'] = 'PUBLISH';
 
         foreach ($requiredAttributes as $name => $default_value) {
@@ -367,7 +449,7 @@ class Horde_iCalendar {
     /**
      * Export this entry as a hash array with tag names as keys.
      *
-     * @param boolean (optional) $paramsInKeys
+     * @param boolean $paramsInKeys
      *                If false, the operation can be quite lossy as the
      *                parameters are ignored when building the array keys.
      *                So if you export a vcard with
@@ -394,16 +476,18 @@ class Horde_iCalendar {
     }
 
     /**
-     * Parse a string containing vCalendar data.
+     * Parses a string containing vCalendar data.
      *
-     * @param string  $text  The data to parse.
-     * @param string  $base  The type of the base object.
-     * @param string  $charset (optional) The encoding charset for $text. Defaults to utf-8
-     * @param boolean $clear (optional) True to clear() the iCal object before parsing.
+     * @param string $text     The data to parse.
+     * @param string $base     The type of the base object.
+     * @param string $charset  The encoding charset for $text. Defaults to
+     *                         utf-8.
+     * @param boolean $clear   If true clears the iCal object before parsing.
      *
      * @return boolean  True on successful import, false otherwise.
      */
-    function parsevCalendar($text, $base = 'VCALENDAR', $charset = 'utf-8', $clear = true)
+    function parsevCalendar($text, $base = 'VCALENDAR', $charset = 'utf-8',
+                            $clear = true)
     {
         if ($clear) {
             $this->clear();
@@ -418,10 +502,10 @@ class Horde_iCalendar {
 
         // All subcomponents.
         $matches = null;
-        if (preg_match_all('/BEGIN:([\W\w]*)(\r\n|\r|\n)([\W\w]*)END:\1(\r\n|\r|\n)/U', $vCal, $matches)) {
+        if (preg_match_all('/BEGIN:([\W\w]*)(\r\n|\r|\n)([\W\w]*)END:\1(\r\n|\r|\n)/Ui', $vCal, $matches)) {
             foreach ($matches[0] as $key => $data) {
-                $type = $matches[1][$key];
-                $component = &Horde_iCalendar::newComponent(trim($type), $this);
+                $type = trim($matches[1][$key]);
+                $component = &Horde_iCalendar::newComponent($type, $this);
                 if ($component === false) {
                     return PEAR::raiseError("Unable to create object for type $type");
                 }
@@ -435,13 +519,13 @@ class Horde_iCalendar {
         }
 
         // Unfold any folded lines.
-        $vCal = preg_replace ('/(\r|\n)+ /', '', $vCal);
+        $vCal = preg_replace('/[\r\n]+[ \t]/', '', $vCal);
 
         // Unfold "quoted printable" folded lines like:
         //  BODY;ENCODING=QUOTED-PRINTABLE:=
         //  another=20line=
         //  last=20line
-        if (preg_match_all('/^([^:]+;\s*ENCODING=QUOTED-PRINTABLE(.*=\r?\n)+(.*[^=])?\r?\n)/mU', $vCal, $matches)) {
+        while (preg_match_all('/^([^:]+;\s*ENCODING=QUOTED-PRINTABLE(.*=\r?\n)+(.*[^=])?\r?\n)/mU', $vCal, $matches)) {
             foreach ($matches[1] as $s) {
                 $r = preg_replace('/=\r?\n/', '', $s);
                 $vCal = str_replace($s, $r, $vCal);
@@ -452,7 +536,7 @@ class Horde_iCalendar {
         if (preg_match_all('/(.*):([^\r\n]*)[\r\n]+/', $vCal, $matches)) {
             foreach ($matches[0] as $attribute) {
                 preg_match('/([^;^:]*)((;[^:]*)?):([^\r\n]*)[\r\n]*/', $attribute, $parts);
-                $tag = $parts[1];
+                $tag = String::upper($parts[1]);
                 $value = $parts[4];
                 $params = array();
 
@@ -461,13 +545,13 @@ class Horde_iCalendar {
                     preg_match_all('/;(([^;=]*)(=([^;]*))?)/', $parts[2], $param_parts);
                     foreach ($param_parts[2] as $key => $paramName) {
                         $paramValue = $param_parts[4][$key];
-                        $params[$paramName] = $paramValue;
+                        $params[String::upper($paramName)] = $paramValue;
                     }
                 }
 
                 // Charset and encoding handling.
                 if ((isset($params['ENCODING'])
-                     && $params['ENCODING'] == 'QUOTED-PRINTABLE')
+                     && String::upper($params['ENCODING']) == 'QUOTED-PRINTABLE')
                     || isset($params['QUOTED-PRINTABLE'])) {
 
                     $value = quoted_printable_decode($value);
@@ -477,29 +561,31 @@ class Horde_iCalendar {
                     } else {
                         $value = String::convertCharset($value, 'utf-8');
                     }
-                }
-
-                if (isset($params['CHARSET'])) {
+                } elseif (isset($params['CHARSET'])) {
                     $value = String::convertCharset($value, $params['CHARSET']);
                 } else {
-                    // As per RFC 2279, assume UTF8 if we don't have
-                    // an explicit charset parameter.
+                    // As per RFC 2279, assume UTF8 if we don't have an
+                    // explicit charset parameter.
                     $value = String::convertCharset($value, $charset);
                 }
 
                 switch ($tag) {
                 // Date fields.
-                case 'DTSTAMP':
                 case 'COMPLETED':
                 case 'CREATED':
                 case 'LAST-MODIFIED':
-                case 'BDAY':
                     $this->setAttribute($tag, $this->_parseDateTime($value), $params);
                     break;
 
+                case 'BDAY':
+                    $this->setAttribute($tag, $this->_parseDate($value), $params);
+                    break;
+
                 case 'DTEND':
                 case 'DTSTART':
+                case 'DTSTAMP':
                 case 'DUE':
+                case 'AALARM':
                 case 'RECURRENCE-ID':
                     if (isset($params['VALUE']) && $params['VALUE'] == 'DATE') {
                         $this->setAttribute($tag, $this->_parseDate($value), $params);
@@ -527,17 +613,9 @@ class Horde_iCalendar {
                     preg_match_all('/,([^,]*)/', ',' . $value, $values);
 
                     foreach ($values[1] as $value) {
-                        if (isset($params['VALUE'])) {
-                            if ($params['VALUE'] == 'DATE-TIME') {
-                                $dates[] = $this->_parseDateTime($value);
-                            } elseif ($params['VALUE'] == 'DATE') {
-                                $dates[] = $this->_parseDate($value);
-                            }
-                        } else {
-                            $dates[] = $this->_parseDateTime($value);
-                        }
+                        $dates[] = $this->_parseDate($value);
                     }
-                    $this->setAttribute($tag, $dates, $params);
+                    $this->setAttribute($tag, isset($dates[0]) ? $dates[0] : null, $params, true, $dates);
                     break;
 
                 // Duration fields.
@@ -553,7 +631,7 @@ class Horde_iCalendar {
                         $periods[] = $this->_parsePeriod($value);
                     }
 
-                    $this->setAttribute($tag, $periods, $params);
+                    $this->setAttribute($tag, isset($periods[0]) ? $periods[0] : null, $params, true, $periods);
                     break;
 
                 // UTC offset fields.
@@ -584,18 +662,38 @@ class Horde_iCalendar {
                     $this->setAttribute($tag, trim($value), $params);
                     break;
 
-                // ADR an N are lists seperated by unescaped semi-colons.
+                // ADR, and N are lists seperated by unescaped
+                // semicolons with a specific number of slots.
                 case 'ADR':
                 case 'N':
                     $value = trim($value);
-                    // As of rfc 2426 2.4.2 semi-colon, comma, and
-                    // colon must be escaped.
-                    $value = str_replace('\\n', $this->_newline, $value);
-                    $value = str_replace('\\,', ',', $value);
-                    $value = str_replace('\\:', ':', $value);
+                    // As of rfc 2426 2.4.2 semicolon, comma, and
+                    // colon must be escaped (comma is unescaped after
+                    // splitting below).
+                    $value = str_replace(array('\\n', '\\;', '\\:'),
+                                         array($this->_newline, ';', ':'),
+                                         $value);
 
-                    // Split by unescaped semi-colons:
-                    $values = preg_split('/(?<!\\\\);/',$value);
+                    // Split by unescaped semicolons:
+                    $values = preg_split('/(?<!\\\\);/', $value);
+                    $value = str_replace('\\;', ';', $value);
+                    $values = str_replace('\\;', ';', $values);
+                    $this->setAttribute($tag, trim($value), $params, true, $values);
+                    break;
+
+                // ORG is a list seperated by unescaped semicolons but
+                // with no empty values.
+                case 'ORG':
+                    $value = trim($value);
+                    // As of rfc 2426 2.4.2 semicolon, comma, and
+                    // colon must be escaped (comma is unescaped after
+                    // splitting below).
+                    $value = str_replace(array('\\n', '\\;', '\\:'),
+                                         array($this->_newline, ';', ':'),
+                                         $value);
+
+                    // Split by unescaped semicolons:
+                    $values = preg_split('/(?<!\\\\);/', $value, -1, PREG_SPLIT_NO_EMPTY);
                     $value = str_replace('\\;', ';', $value);
                     $values = str_replace('\\;', ';', $values);
                     $this->setAttribute($tag, trim($value), $params, true, $values);
@@ -603,19 +701,31 @@ class Horde_iCalendar {
 
                 // String fields.
                 default:
-                    $value = trim($value);
-                    // As of rfc 2426 2.4.2 semi-colon, comma, and
-                    // colon must be escaped.
-                    $value = str_replace('\\n', $this->_newline, $value);
-                    $value = str_replace('\\;', ';', $value);
-                    $value = str_replace('\\:', ':', $value);
+                    if ($this->isOldFormat()) {
+                        // vcalendar 1.0 and vcard 2.1 only escape semicolons
+                        // and use unescaped semicolons to create lists.
+                        $value = trim($value);
+                        // Split by unescaped semicolons:
+                        $values = preg_split('/(?<!\\\\);/', $value);
+                        $value = str_replace('\\;', ';', $value);
+                        $values = str_replace('\\;', ';', $values);
+                        $this->setAttribute($tag, trim($value), $params, true, $values);
+                    } else {
+                        $value = trim($value);
+                        // As of rfc 2426 2.4.2 semicolon, comma, and
+                        // colon must be escaped (comma is unescaped after
+                        // splitting below).
+                        $value = str_replace(array('\\n', '\\;', '\\:'),
+                                             array($this->_newline, ';', ':'),
+                                             $value);
 
-                    // Split by unescaped commas:
-                    $values = preg_split('/(?<!\\\\),/',$value);
-                    $value = str_replace('\\,', ',', $value);
-                    $values = str_replace('\\,', ',', $values);
+                        // Split by unescaped commas:
+                        $values = preg_split('/(?<!\\\\),/', $value);
+                        $value = str_replace('\\,', ',', $value);
+                        $values = str_replace('\\,', ',', $values);
 
-                    $this->setAttribute($tag, trim($value), $params, true, $values);
+                        $this->setAttribute($tag, trim($value), $params, true, $values);
+                    }
                     break;
                 }
             }
@@ -627,20 +737,21 @@ class Horde_iCalendar {
     /**
      * Export this component in vCal format.
      *
-     * @param string $base  (optional) The type of the base object.
+     * @param string $base  The type of the base object.
      *
      * @return string  vCal format data.
      */
     function _exportvData($base = 'VCALENDAR')
     {
-        $result  = 'BEGIN:' . String::upper($base) . $this->_newline;
+        $result = 'BEGIN:' . String::upper($base) . $this->_newline;
 
-        // Ensure that version is the first attribute.
-        $v = $this->getAttributeDefault('VERSION', false);
-        if ($v) {
-            $result .= 'VERSION:' . $v . $this->_newline;
+        // VERSION is not allowed for entries enclosed in VCALENDAR/ICALENDAR,
+        // as it is part of the enclosing VCALENDAR/ICALENDAR. See rfc2445
+        if ($base !== 'VEVENT' && $base !== 'VTODO' && $base !== 'VALARM' &&
+            $base !== 'VJOURNAL' && $base !== 'VFREEBUSY') {
+            // Ensure that version is the first attribute.
+            $result .= 'VERSION:' . $this->_version . $this->_newline;
         }
-
         foreach ($this->_attributes as $attribute) {
             $name = $attribute['name'];
             if ($name == 'VERSION') {
@@ -648,11 +759,15 @@ class Horde_iCalendar {
                 continue;
             }
 
-            $params = $attribute['params'];
             $params_str = '';
-
-            if (count($params)) {
+            $params = $attribute['params'];
+            if ($params) {
                 foreach ($params as $param_name => $param_value) {
+                    // Skip CHARSET for iCalendar 2.0 data, not
+                    // allowed.
+                    if ($param_name == 'CHARSET' && !$this->isOldFormat()) {
+                        continue;
+                    }
                     $params_str .= ";$param_name=$param_value";
                 }
             }
@@ -660,7 +775,6 @@ class Horde_iCalendar {
             $value = $attribute['value'];
             switch ($name) {
             // Date fields.
-            case 'DTSTAMP':
             case 'COMPLETED':
             case 'CREATED':
             case 'DCREATED':
@@ -670,7 +784,9 @@ class Horde_iCalendar {
 
             case 'DTEND':
             case 'DTSTART':
+            case 'DTSTAMP':
             case 'DUE':
+            case 'AALARM':
             case 'RECURRENCE-ID':
                 if (isset($params['VALUE'])) {
                     if ($params['VALUE'] == 'DATE') {
@@ -752,16 +868,100 @@ class Horde_iCalendar {
             // Recurrence fields.
             case 'EXRULE':
             case 'RRULE':
+                break;
 
+            // Attendance
+            case 'ATTENDEE':
+                // Kronolith creates attendee field in vcalendar2.0 format.
+                // Convert to vcalendar1.0 if necessary.
+                // Example of 1.0 style:
+                // ATTENDEE;ROLE=OWNER;STATUS=CONFIRMED:John Smith <jsmith@host1.com>
+                if ($this->isOldFormat()) {
+                    $value = preg_replace('/MAILTO:/', '', $value);
+                    $value =  $value . ' <' . $value . '>';
+                    $params_str = str_replace('PARTSTAT=', 'STATUS=', $params_str);
+                    $params_str = str_replace('STATUS=NEEDS-ACTION', 'STATUS=NEEDS ACTION', $params_str);
+                    $params_str = str_replace('ROLE=REQ-PARTICIPANT', 'EXPECT=REQUIRE', $params_str);
+                    $params_str = str_replace('ROLE=OPT-PARTICIPANT', 'EXPECT=REQUEST', $params_str);
+                    $params_str = str_replace('ROLE=NON-PARTICIPANT', 'EXPECT=FYI', $params_str);
+                    $params_str = str_replace('RSVP=TRUE', 'RSVP=YES', $params_str);
+                    $params_str = str_replace('RSVP=FALSE', 'RSVP=NO', $params_str);
+                }
+                break;
             default:
+                if ($this->isOldFormat()) {
+                    if (is_array($attribute['values'])
+                            && count($attribute['values'])>1) {
+                        $values = $attribute['values'];
+                        if ($name == 'N' || $name == 'ADR') {
+                            $glue = ';';
+                        } else {
+                            $glue = ',';
+                        }
+                        $values  = str_replace(array(';'),
+                                             array('\\;'),
+                                             $values);
+                        $value = implode($glue, $values);
+                    } else {
+                        /* vcard 2.1 and vcalendar 1.0 escape only
+                         * semicolons */
+                        $value = str_replace(array(';'),
+                                             array('\\;'),
+                                             $value);
+                    }
+                    if ($name == 'STATUS') {
+                        // vcalendar 1.0 has STATUS:NEEDS ACTION while 2.0 has
+                        // STATUS:NEEDS-ACTION.
+                        $value = str_replace('NEEDS-ACTION', 'NEEDS ACTION', $value);
+                    }
+                    // Text containing newlines or ASCII >= 127 must be BASE64
+                    // or QUOTED-PRINTABLE encoded. Currently we use
+                    // QUOTED-PRINTABLE as default.
+                    // FIXME: deal with base64 encodings!
+                    if (preg_match("/[^\x20-\x7F]/", $value)
+                            && empty($params['ENCODING']))  {
+                        $params['ENCODING'] = 'QUOTED-PRINTABLE';
+                        $params_str .= ';ENCODING=QUOTED-PRINTABLE';
+                        // Add CHARSET as well. At least the synthesis client
+                        // gets confused otherwise
+                        if (empty($params['CHARSET'])) {
+                            $params['CARSET'] = NLS::getCharset();
+                            $params_str .= ';CHARSET=' . $params['CARSET'];
+                        }
+                    }
+                } else {
+                    if (is_array($attribute['values'])
+                            && count($attribute['values'])>1) {
+                        $values = $attribute['values'];
+                        if ($name == 'N' || $name == 'ADR') {
+                            $glue = ';';
+                        } else {
+                            $glue = ',';
+                        }
+                        $values = str_replace(array(';', ':', ',', 'MAILTO\\:'),
+                                             array('\\;', '\\:', '\\,', 'MAILTO:'),
+                                             $values);
+                        $value = implode($glue, $values);
+                    } else {
+                        // As of rfc 2426 2.4.2 semicolon, comma, and colon
+                        // must be escaped. Exclude MAILTO: though. This is a
+                        // hack!
+                        $value = str_replace(array(';', ':', ',', 'MAILTO\\:'),
+                                             array('\\;', '\\:', '\\,', 'MAILTO:'),
+                                             $value);
+                    }
+                }
                 break;
             }
 
             if (!empty($params['ENCODING']) &&
                 $params['ENCODING'] == 'QUOTED-PRINTABLE' && strlen(trim($value)) > 0) {
+                /* quotedPrintableEncode does not escape CRLFs, but strange
+                 * enough single LFs. so convert everything to LF only and
+                 * replace afterwards. */
                 $value = str_replace("\r", '', $value);
                 $result .= "$name$params_str:=" . $this->_newline
-                    . $this->_quotedPrintableEncode($value)
+                    . str_replace('=0A', '=0D=0A', $this->_quotedPrintableEncode($value))
                     . $this->_newline;
             } else {
                 $attr_string = "$name$params_str:$value";
@@ -769,8 +969,8 @@ class Horde_iCalendar {
             }
         }
 
-        foreach ($this->getComponents() as $component) {
-            $result .= $component->exportvCalendar() . $this->_newline;
+        foreach ($this->_components as $component) {
+            $result .= $component->exportvCalendar();
         }
 
         return $result . 'END:' . $base . $this->_newline;
@@ -842,23 +1042,26 @@ class Horde_iCalendar {
     }
 
     /**
-     * Parse a DateTime field into a unix timestamp.
+     * Parses a DateTime field into a unix timestamp.
+     *
+     * @todo This function should be moved to Horde_Date and made public.
+     * @static
      */
     function _parseDateTime($text)
     {
         $dateParts = explode('T', $text);
         if (count($dateParts) != 2 && !empty($text)) {
             // Not a datetime field but may be just a date field.
-            if (!$date = $this->_parseDate($text)) {
+            if (!$date = Horde_iCalendar::_parseDate($text)) {
                 return $date;
             }
             return @gmmktime(0, 0, 0, $date['month'], $date['mday'], $date['year']);
         }
 
-        if (!$date = $this->_parseDate($dateParts[0])) {
+        if (!$date = Horde_iCalendar::_parseDate($dateParts[0])) {
             return $date;
         }
-        if (!$time = $this->_parseTime($dateParts[1])) {
+        if (!$time = Horde_iCalendar::_parseTime($dateParts[1])) {
             return $time;
         }
 
@@ -878,8 +1081,8 @@ class Horde_iCalendar {
     {
         $temp = array();
         if (!is_object($value) && !is_array($value)) {
-            $TZOffset  = 3600 * substr(date('O'), 0, 3);
-            $TZOffset += 60 * substr(date('O'), 3, 2);
+            $tz = date('O', $value);
+            $TZOffset = (3600 * substr($tz, 0, 3)) + (60 * substr(date('O', $value), 3, 2));
             $value -= $TZOffset;
 
             $temp['zone']   = 'UTC';
@@ -890,44 +1093,17 @@ class Horde_iCalendar {
             $temp['minute'] = date('i', $value);
             $temp['second'] = date('s', $value);
         } else {
-            $dateOb = (object)$value;
-
-            // Minutes.
-            $TZOffset = substr(date('O'), 3, 2);
-            $thisMin = $dateOb->min - $TZOffset;
-
-            // Hours.
-            $TZOffset = substr(date('O'), 0, 3);
-            $thisHour = $dateOb->hour - $TZOffset;
-
-            if ($thisMin < 0) {
-                $thisHour -= 1;
-                $thisMin += 60;
-            }
-
-            if ($thisHour < 0) {
-                require_once 'Date/Calc.php';
-                $prevday = Date_Calc::prevDay($dateOb->mday, $dateOb->month, $dateOb->year);
-                $dateOb->mday  = substr($prevday, 6, 2);
-                $dateOb->month = substr($prevday, 4, 2);
-                $dateOb->year  = substr($prevday, 0, 4);
-                $thisHour += 24;
-            }
-
-            $temp['zone']   = 'UTC';
-            $temp['year']   = $dateOb->year;
-            $temp['month']  = $dateOb->month;
-            $temp['mday']   = $dateOb->mday;
-            $temp['hour']   = $thisHour;
-            $temp['minute'] = $dateOb->min;
-            $temp['second'] = $dateOb->sec;
+            $dateOb = new Horde_Date($value);
+            return Horde_iCalendar::_exportDateTime($dateOb->timestamp());
         }
 
         return Horde_iCalendar::_exportDate($temp) . 'T' . Horde_iCalendar::_exportTime($temp);
     }
 
     /**
-     * Parse a Time field.
+     * Parses a Time field.
+     *
+     * @static
      */
     function _parseTime($text)
     {
@@ -947,7 +1123,7 @@ class Horde_iCalendar {
     }
 
     /**
-     * Export a Time field.
+     * Exports a Time field.
      */
     function _exportTime($value)
     {
@@ -960,19 +1136,24 @@ class Horde_iCalendar {
     }
 
     /**
-     * Parse a Date field.
+     * Parses a Date field.
+     *
+     * @static
      */
     function _parseDate($text)
     {
-        if (strlen($text) != 8) {
+        $parts = explode('T', $text);
+        if (count($parts) == 2) {
+            $text = $parts[0];
+        }
+
+        if (!preg_match('/^(\d{4})-?(\d{2})-?(\d{2})$/', $text, $match)) {
             return false;
         }
 
-        $date['year']  = intval(substr($text, 0, 4));
-        $date['month'] = intval(substr($text, 4, 2));
-        $date['mday']  = intval(substr($text, 6, 2));
-
-        return $date;
+        return array('year' => $match[1],
+                     'month' => $match[2],
+                     'mday' => $match[3]);
     }
 
     /**
@@ -1101,9 +1282,9 @@ class Horde_iCalendar {
      *
      * Uses imap_8bit if available.
      *
-     * @param  string $input  The string to be encoded.
+     * @param string $input  The string to be encoded.
      *
-     * @return string         The quoted-printable encoded string.
+     * @return string  The quoted-printable encoded string.
      */
     function _quotedPrintableEncode($input = '')
     {
