@@ -2,15 +2,14 @@
 /**
  * Class representing vEvents.
  *
- * $Horde: framework/iCalendar/iCalendar/vevent.php,v 1.31.10.1 2005/01/03 12:19:23 jan Exp $
+ * $Horde: framework/iCalendar/iCalendar/vevent.php,v 1.31.10.8 2006/01/01 21:28:47 jan Exp $
  *
- * Copyright 2003-2005 Mike Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2003-2006 Mike Cochrane <mike@graftonhall.co.nz>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Mike Cochrane <mike@graftonhall.co.nz>
- * @version $Revision: 1.31.10.1 $
  * @since   Horde 3.0
  * @package Horde_iCalendar
  */
@@ -32,21 +31,21 @@ class Horde_iCalendar_vevent extends Horde_iCalendar {
         $requiredAttributes = array();
         $requiredAttributes['DTSTAMP'] = time();
         $requiredAttributes['ORGANIZER'] = 'Unknown Organizer';
-        $requiredAttributes['UID'] = $this->_exportDateTime(time()) . '@' . $_SERVER['SERVER_NAME'];
+        $requiredAttributes['UID'] = $this->_exportDateTime(time()) . '@' . (isset($_SERVER['SERVER_NAME']) ? $_SERVER['SERVER_NAME'] : 'localhost');
 
         $method = !empty($this->_container) ?
             $this->_container->getAttribute('METHOD') : 'PUBLISH';
 
         switch ($method) {
         case 'PUBLISH':
-            $requiredAttributes['DTSTART']  = time();
-            $requiredAttributes['SUMMARY']  = '';
+            $requiredAttributes['DTSTART'] = time();
+            $requiredAttributes['SUMMARY'] = '';
             break;
 
         case 'REQUEST':
             $requiredAttributes['ATTENDEE'] = '';
-            $requiredAttributes['DTSTART']  = time();
-            $requiredAttributes['SUMMARY']  = '';
+            $requiredAttributes['DTSTART'] = time();
+            $requiredAttributes['SUMMARY'] = '';
             break;
 
         case 'REPLY':
@@ -54,9 +53,9 @@ class Horde_iCalendar_vevent extends Horde_iCalendar {
             break;
 
         case 'ADD':
-            $requiredAttributes['DTSTART']  = time();
+            $requiredAttributes['DTSTART'] = time();
             $requiredAttributes['SEQUENCE'] = 1;
-            $requiredAttributes['SUMMARY']  = '';
+            $requiredAttributes['SUMMARY'] = '';
             break;
 
         case 'CANCEL':
@@ -128,7 +127,7 @@ class Horde_iCalendar_vevent extends Horde_iCalendar {
     /**
      * Update this event with details from another event.
      *
-     * @param object Horde_iCalendar_vEvent $vevent  The vEvent with latest details.
+     * @param Horde_iCalendar_vEvent $vevent  The vEvent with latest details.
      */
     function updateFromvEvent($vevent)
     {
@@ -178,13 +177,13 @@ class Horde_iCalendar_vevent extends Horde_iCalendar {
      * Update just the attendess of event with details from another
      * event.
      *
-     * @param object Horde_iCalendar_vEvent $vevent  The vEvent with latest details
+     * @param Horde_iCalendar_vEvent $vevent  The vEvent with latest details
      */
     function updateAttendeesFromvEvent($vevent)
     {
         $newAttributes = $vevent->getAllAttributes();
         foreach ($newAttributes as $newAttribute) {
-            if (!$newAttribute['name'] == 'ATTENDEE') {
+            if ($newAttribute['name'] != 'ATTENDEE') {
                 continue;
             }
             $currentValue = $this->getAttribute($newAttribute['name']);
