@@ -3,12 +3,12 @@
  * Net_SMS_clickatell_http Class implements the HTTP API for accessing the
  * Clickatell (www.clickatell.com) SMS gateway.
  *
- * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
+ * Copyright 2003-2006 Marko Djukic <marko@oblo.com>
  *
  * See the enclosed file COPYING for license information (LGPL). If you did
  * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
- * $Horde: framework/Net_SMS/SMS/clickatell_http.php,v 1.13.10.6 2005/08/16 10:13:54 jan Exp $
+ * $Horde: framework/Net_SMS/SMS/clickatell_http.php,v 1.13.10.9 2006/08/07 00:42:30 chuck Exp $
  *
  * @author Marko Djukic <marko@oblo.com>
  * @package Net_SMS
@@ -332,7 +332,7 @@ class Net_SMS_clickatell_http extends Net_SMS {
      */
     function _callURL($url)
     {
-        $options['method'] = 'POST';
+        $options['method'] = 'GET';
         $options['timeout'] = 5;
         $options['allowRedirects'] = true;
 
