@@ -3,15 +3,14 @@
 /**
  * Strategy for directly writing output file to VFS.
  *
- * $Horde: framework/VFS_ISOWriter/ISOWriter/RealOutputStrategy/direct.php,v 1.1 2004/11/08 17:07:17 eraserhd Exp $
+ * $Horde: framework/VFS_ISOWriter/ISOWriter/RealOutputStrategy/direct.php,v 1.1.8.5 2006/01/01 21:28:44 jan Exp $
  *
- * Copyright 2004 Cronosys, LLC <http://www.cronosys.com/>
+ * Copyright 2004-2006 Cronosys, LLC <http://www.cronosys.com/>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Jason M. Felice <jfelice@cronosys.com>
- * @version $Revision: 1.1 $
  * @package VFS_ISO
  * @since   Horde 3.0
  */
@@ -28,7 +27,7 @@ class VFS_ISOWriter_RealOutputStrategy_direct extends VFS_ISOWriter_RealOutputSt
         $dir = dirname($filename);
         while (!@is_dir($dir)) {
             if (!@mkdir($dir, 0755)) {
-                return PEAR::raiseError(sprintf(_("Could not mkdir '%s'."),
+                return PEAR::raiseError(sprintf(_("Could not mkdir \"%s\"."),
                                                 $dir));
             }
             $dir = dirname($dir);
