@@ -1,14 +1,14 @@
 <?php
 /**
- * $Horde: framework/XML_WBXML/WBXML/ContentHandler.php,v 1.9.10.1 2005/01/03 12:19:22 jan Exp $
+ * $Horde: framework/XML_WBXML/WBXML/ContentHandler.php,v 1.9.10.7 2006/05/01 12:05:15 jan Exp $
  *
- * Copyright 2003-2005 Anthony Mills <amills@pyramid6.com>
+ * Copyright 2003-2006 Anthony Mills <amills@pyramid6.com>
  *
- * See the enclosed file COPYING for license information (LGPL).  If you
- * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ * See the enclosed file COPYING for license information (LGPL).  If you did
+ * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
- * From Binary XML Content Format Specification Version 1.3, 25 July
- * 2001 found at http://www.wapforum.org
+ * From Binary XML Content Format Specification Version 1.3, 25 July 2001
+ * found at http://www.wapforum.org
  *
  * @package XML_WBXML
  */
@@ -66,20 +66,25 @@ class XML_WBXML_ContentHandler {
         return $this->_output;
     }
 
+    function getOutputSize()
+    {
+        return strlen($this->_output);
+    }
+
     function startElement($uri, $element, $attrs)
     {
         $this->_output .= '<' . $element;
 
         $currentUri = $this->_currentUri->top();
 
-        if ((!$currentUri) || ($currentUri != $uri)) {
+        if (((!$currentUri) || ($currentUri != $uri)) && $uri) {
             $this->_output .= ' xmlns="' . $uri . '"';
         }
 
         $this->_currentUri->push($uri);
 
         foreach ($attrs as $attr) {
-            $this->_output .= ' ' . $attr['attiribute'] . '="' . $attr['value'] . '"';
+            $this->_output .= ' ' . $attr['attribute'] . '="' . $attr['value'] . '"';
         }
 
         $this->_output .= '>';
@@ -99,12 +104,7 @@ class XML_WBXML_ContentHandler {
 
     function opaque($o)
     {
-        // I can check the first chanracter and see if it is WBXML.
-        if (ord($o[0]) < 10) {
-            // Should decode this, I really need a call back function.
-        } else {
-            $this->_output .= $o;
-        }
+        $this->_output .= $o;
     }
 
     function setOpaqueHandler($opaqueHandler)
@@ -117,6 +117,15 @@ class XML_WBXML_ContentHandler {
         unset($this->_opaqueHandler);
     }
 
+    function createSubHandler()
+    {
+        $name = get_class($this); // clone current class
+        $sh = &new $name();
+        $sh->setCharset($this->getCharsetStr());
+        $sh->setVersion($this->getVersion());
+        return $sh;
+    }
+
 }
 
 class XML_WBXML_LifoQueue {
@@ -129,7 +138,7 @@ class XML_WBXML_LifoQueue {
 
     function push($obj)
     {
-        array_push($this->_queue, $obj);
+        $this->_queue[] = $obj;
     }
 
     function pop()
